package gui;

import gui.highlighting.JavaScanner;
import gui.highlighting.Scanner;
import gui.highlighting.SyntaxHighlighter;

import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;

import javax.swing.AbstractAction;
import javax.swing.JMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.KeyStroke;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.text.Caret;
import javax.swing.text.DefaultCaret;
import javax.swing.text.DefaultEditorKit;
import javax.swing.text.Document;
import javax.swing.text.Element;
import javax.swing.text.JTextComponent;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.UndoManager;
import javax.swing.undo.UndoableEdit;
/**
 * This class represents the editor within the editor panel. Each tab
 * in the editor panel contains an EditorArray instance. EditorArray class
 * cover all functionalities of a basic editor as well as advanced features
 * such as highlighting.
 */
public class EditorArray {

	private JScrollPane editScrollPane = null;
	private JTextPane editFieldEditorPane = null;
	public int changeFlag; // keeps if there is a change on the file.
	public int arrayIndex; 
	private String fileName;
	private String filePath;
	private EditorPane editorPane;
	public int changeCount=0;
	public int undoRedoFlag;
	public Scanner scanner = new JavaScanner();
	public PMenuBar menuBar;
	private JMenuItem findReplaceMenuItem = null;
	private JMenuItem selectAllMenuItem = null;
	public PFrame pFrame;
	public ArrayList bpList = new ArrayList(); 
	public ArrayList tempList = new ArrayList();
	//public TokenHighlighter usageHighlighter = null;
	
	/**
	 * This method returns the relative file path
	 * @return file path
	 */
	public String getFilePath() {
		return filePath;
	}

	/**
	 * This method changes the file path
	 * @param filePath
	 */
	public void setFilePath(String filePath) {
		this.filePath = filePath;
	}

	/**
	 * Constructor for the EditorArray class. The constructor initializes
	 * the file name which appears as tab name. The class needs the main 
	 * editor panel and the associated menu bar object, as well.
	 * @param ePane
	 * @param s
	 * @param pm
	 */
	public EditorArray(EditorPane ePane, String s, PMenuBar pm, PFrame p) {
		// TODO Auto-generated constructor stub
		//this.getEditFieldEditorPane();
		//this.getEditScrollPane();
		this.changeFlag = 0;
		this.arrayIndex = 0;
		this.fileName = s;
		editorPane = ePane;
		this.menuBar = pm;
		this.pFrame = p;
	}

	/**
	 * This method initializes and returns the text field of the EditorArray object.
	 * The text field has syntax highlihter, copy/paste and undo/redio listeners attached.
	 * @return JTextPane
	 */
	@SuppressWarnings("serial")
	public JTextPane getEditFieldEditorPane() {
		
		if (this.editFieldEditorPane == null) {
			
			String extension = getFileName();
			int limit = 0;
			for (int i = 0; i < extension.length(); i++) {
				
				if(extension.charAt(i) == '.')
				limit = i;
			}
				
			extension = extension.substring(limit+1, extension.length());
			System.out.println("EXTENSION: "+extension);
			if(extension.equals("asm"))
			{
				this.editFieldEditorPane = new SyntaxHighlighter(24, 80, scanner);
				
			}
			else this.editFieldEditorPane = new JTextPane();
			
			
			menuBar.getEditMenu().remove(7);
			menuBar.getEditMenu().add(getSelectAllMenuItem(),7);
			menuBar.getEditMenu().remove(9);
			menuBar.getEditMenu().add(getFindReplaceMenuItem(),9);
			menuBar.getCutMenuItem().setEnabled(true);
			menuBar.getCopyMenuItem().setEnabled(true);
			menuBar.getPasteMenuItem().setEnabled(true);
			
			final UndoManager undo = new UndoManager();
			Document doc = this.editFieldEditorPane.getDocument();
//			 Listen for undo and redo events
			 doc.addUndoableEditListener(new UndoableEditListener() {
			        public void undoableEditHappened(UndoableEditEvent evt) {
			        	//System.out.println("oldu");
			        	UndoableEdit e = evt.getEdit();
			        	if (e.getPresentationName().startsWith("style")) return;
			        	//System.out.println("aas: "+e.getPresentationName());
			        	//if (e.getPresentationName().startsWith("addition")) return;
			        	undo.addEdit(evt.getEdit());
			        }
			    });
						 
//			 Create an undo action and add it to the text component
			this.editFieldEditorPane.getActionMap().put("Undo",
			        new AbstractAction("Undo") {
	            /**
						 * 
						 */
						private static final long serialVersionUID = 1L;

				public void actionPerformed(ActionEvent evt) {
	                try {
	                    if (undo.canUndo()) {
	                    		undo.undo();
	                    }
	                } catch (CannotUndoException e) {
	                	 Toolkit.getDefaultToolkit().beep();
	                	}
	            }
	         });
			
//			 Bind the undo action to ctl-Z
			 this.editFieldEditorPane.getInputMap().put(KeyStroke.getKeyStroke("control Z"), "Undo");
			
//			 Create a redo action and add it to the text component
			 this.editFieldEditorPane.getActionMap().put("Redo",
			        new AbstractAction("Redo") {
			            public void actionPerformed(ActionEvent evt) {
			                try {
			                    if (undo.canRedo()) {
			                    		undo.redo();
			                    }
			                } catch (CannotRedoException e) {
			                	 Toolkit.getDefaultToolkit().beep();
			                }
			            }
			        });
			    
			    // Bind the redo action to ctl-Y
			 this.editFieldEditorPane.getInputMap().put(KeyStroke.getKeyStroke("control Y"), "Redo");

			/*
			this.editFieldEditorPane.setBorder(new Border() {
			
				public void paintBorder(Component c, Graphics g, int x, int y, int width,
						int height) {
					// TODO Auto-generated method stub
					g.drawLine(28, 0, 28, c.getHeight());
				}
			
				public boolean isBorderOpaque() {
					// TODO Auto-generated method stub
					return false;
				}
			
				public Insets getBorderInsets(Component c) {
					// TODO Auto-generated method stub
					return new Insets(0,30,0,0);
				}
			
			});	
			
			*/
			
			this.editFieldEditorPane.addKeyListener( 
					new KeyListener(){

						public void keyPressed(KeyEvent e) {
							// TODO Auto-generated method stub
							
						}

						public void keyReleased(KeyEvent e) {
							// TODO Auto-generated method stub
							
						}

						public void keyTyped(KeyEvent e) {
							// TODO Auto-generated method stub
							changeFlag = 1;
							int index = editorPane.activeFileIndex;
							editorPane.setTitleAt(index, "*"+fileName);
						}
					});
			
			this.editFieldEditorPane.addCaretListener(new CaretListener(){

				public void caretUpdate(CaretEvent e) {
					// TODO Auto-generated method stub
					
					pFrame.statusBarLabel.setText(""+getLineAtCaret(editFieldEditorPane));
					//System.out.println("caret at: "+getLineAtCaret(editFieldEditorPane));
				}});
			
//			usageHighlighter = new TokenHighlighter();
//	        this.editFieldEditorPane.setHighlighter(usageHighlighter);	
		}
		return this.editFieldEditorPane;
	}


	/**
	 * This method initializes and returns the scroll panel under the text 
	 * field.
	 * @return JScrollPane
	 */
	public JScrollPane getEditScrollPane() {
		
		if (this.editScrollPane == null) {
			this.editScrollPane = new JScrollPane();
			this.editScrollPane.setViewportView(this.getEditFieldEditorPane());
		}
		return this.editScrollPane;
	}

	/**
	 * This method returns the name of the file opened by the editor instance.
	 * @return String
	 */
	public String getFileName() {
		return fileName;
	}

	/**
	 * This method changes the name of the file opened by the editor instance.
	 * @param fileName
	 */
	public void setFileName(String fileName) {
		this.fileName = fileName;
	}
	
	/**
	 * This method initializes and returns the find/replace menu item.
	 * At each find/replace operation, the menu item is recreated 	
	 * @return JMenuItem	
	 */
	private JMenuItem getFindReplaceMenuItem() {
		if (findReplaceMenuItem == null) {
			findReplaceMenuItem = new JMenuItem();
			findReplaceMenuItem.setText("Find/Replace");
			findReplaceMenuItem.setEnabled(true);
			findReplaceMenuItem.addActionListener(new ActionListener(){
			
				@SuppressWarnings("serial")
				public void actionPerformed(ActionEvent arg0) {
					// TODO Auto-generated method stub
					
					int index = editorPane.activeFileIndex;
					System.out.println("index: "+index);
					JTextPane text = null;
					if(index > -1)
					{
						text = editorPane.list.get(index).getEditFieldEditorPane();						
						
						Caret selected = new DefaultCaret()
						{
							 public void focusLost(FocusEvent e)
							 {
							 	setVisible(false);
							 }
						};
						selected.setBlinkRate( text.getCaret().getBlinkRate() );
						text.setCaret( selected );

						try
						{
							text.read( new FileReader( new File( "FindReplace.java" ) ), "" );
						}
						catch(IOException e)
						{
							System.out.println( e );
						}
						text.getDocument().putProperty( DefaultEditorKit.EndOfLineStringProperty, "\n" );



						//  display
						
						PFrame frame = editorPane.callerFrame;
						FindReplace.getSharedInstance( frame ).showReplace( text );
					}
				}});
		}
		return findReplaceMenuItem;
	}

	/**
	 * This method initializes and returns menu item which selects all
	 * text content in the text field	
	 * @return JMenuItem	
	 */
	public JMenuItem getSelectAllMenuItem() {
		if (selectAllMenuItem == null) {
			selectAllMenuItem = new JMenuItem();
			selectAllMenuItem.setText("Select All");
			selectAllMenuItem.setEnabled(true);
			selectAllMenuItem.addActionListener(new ActionListener(){

				public void actionPerformed(ActionEvent e) {
					// TODO Auto-generated method stub
					getEditFieldEditorPane().selectAll();
				}});
		}
		return selectAllMenuItem;
	}
	
 
	

	/**
	*  Return the current line number at the Caret position.
	*/
	public static int getLineAtCaret(JTextComponent component)
	{
		int caretPosition = component.getCaretPosition();
		Element root = component.getDocument().getDefaultRootElement();
 
		return root.getElementIndex( caretPosition ) + 1;
	}

//	 public class TokenHighlighter extends DefaultHighlighter {
//	        public HighlightPainter painter;
//	 
//	        public TokenHighlighter() {
//	            this.painter = new DefaultHighlightPainter(new Color(204, 201, 237));
//	        }
//	    }

}