/**
 * 
 */
package gui;

import java.awt.ComponentOrientation;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.text.Caret;
import javax.swing.text.DefaultCaret;
import javax.swing.text.DefaultEditorKit;
import javax.swing.tree.DefaultMutableTreeNode;

import simulator.SimulatorFrame;
/**
 * This class is the toolbar object of the PIDE. It contains all toolbar buttons
 * and binds them to related action listeners. 
 * @author maho
 *
 */
@SuppressWarnings("serial")
public class PToolBar extends JToolBar {


	private JButton newButton = null;
	private JButton openButton = null;
	private JButton saveButton = null;
	private JButton saveAllButton = null;
	

	private JButton cutButton = null;
	private JButton copyButton = null;
	private JButton pasteButton = null;
	private JButton undoButton = null;
	private JButton redoButton = null;
	private JButton findButton = null;
	private JButton replaceButton = null;
	private JButton stepButton = null;
	
	private JToggleButton workspaceToggleButton = null;
	private JToggleButton consoleToggleButton = null;
	private JToggleButton registerToggleButton = null;
	private JToggleButton watchpointToggleButton = null;
	
	private JButton buildButton = null;
	private JButton startSimulateButton = null;
	private JButton stopSimulateButton = null;
	private EditorPane editorPane;
	private GuiController guiController;
	private PFrame callerFrame;

	public ConsolePane consolePane;
	public SimulatorFrame simulatorFrame=null;
	
	/**
	 * The constructor of toolbar class.
	 * 
	 */
	public PToolBar(EditorPane ePane, GuiController gController, PFrame pFrame) {
		// TODO Auto-generated constructor stub
		this.initialize();
		this.editorPane = ePane;
		this.guiController = gController;
		this.callerFrame = pFrame;
		consolePane = pFrame.consolePane;
	}
		

	/**
	 * This method initializes this
	 * 
	 */
	private void initialize() {
        this.setSize(new Dimension(649, 23));
        this.setFloatable(false);
        this.setName("PIDEToolBar");
        this.setComponentOrientation(ComponentOrientation.LEFT_TO_RIGHT);
        //create buttons
        //file ->
        this.add(this.getNewButton());
        this.add(this.getOpenButton());
        this.add(this.getSaveButton());
        this.add(this.getSaveAllButton()); 
        
        this.addSeparator();
        //edit -> 
        this.add(this.getCutButton());
        this.add(this.getCopyButton());
        this.add(this.getPasteButton());
       
        this.addSeparator();
        this.add(this.getUndoButton());
        this.add(this.getRedoButton());
        //this.add(this.getFindButton());
        this.add(this.getReplaceButton());
        
        
        this.addSeparator();
        //view toggle -> 
        this.add(this.getWorkspaceToggleButton());
        this.add(this.getConsoleToggleButton());
        this.add(this.getRegisterToggleButton());
        this.add(this.getWatchpointToggleButton());
        
        this.getWorkspaceToggleButton().setVisible(false);
        this.getConsoleToggleButton().setVisible(false);
        this.getRegisterToggleButton().setVisible(false);
        this.getWatchpointToggleButton().setVisible(false);
        
        this.addSeparator();
        //compile and simulate
        this.add(this.getBuildButton());
        this.add(this.getStartSimulateButton());//ayni zamanda pause button
        this.add(this.getStopSimulateButton());
        this.add(this.getstepButton());
        
        this.getBuildButton().setVisible(true);
        this.getStartSimulateButton().setVisible(true);
        this.getStopSimulateButton().setVisible(true);
     /*   
        this.addSeparator();
        //analysis -> add listener
        
        this.addSeparator();
        //debugger -> debug,step,step into,step out,run to cursor
        
        this.addSeparator();
        //programmer -> program
        
        this.addSeparator();
        //help
		*/	
	}


	/**
	 * This method initializes and returns the new button.	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getNewButton() {
		if (this.newButton == null) {
			this.newButton = new JButton();
			this.newButton.setIcon(new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/general/New16.gif")));
			this.newButton.setToolTipText("New");
			this.newButton.setSize(new Dimension(27, 25));
			this.newButton.setPreferredSize(new Dimension(27, 25));
			this.newButton.setMnemonic(KeyEvent.VK_N);
			this.newButton.addActionListener(new ActionListener(){

				public void actionPerformed(ActionEvent e) {
					// TODO Auto-generated method stub
					NewFileFrame newFileFrame = new NewFileFrame(callerFrame, editorPane);
					newFileFrame.setVisible(true);
				}});
		}
		return this.newButton;
	}


	/**
	 * This method initializes and returns the open button.	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getOpenButton() {
		if (this.openButton == null) {
			this.openButton = new JButton();
			this.openButton.setIcon(new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/general/Open16.gif")));
			this.openButton.setMnemonic(KeyEvent.VK_O);
			this.openButton.setToolTipText("Open");
			this.openButton.setSize(new Dimension(20,20));
			this.openButton.setPreferredSize(new Dimension(20, 20));
			this.openButton.setName("Open");
			this.openButton.addActionListener(new ActionListener(){

				public void actionPerformed(ActionEvent e) {
					// TODO Auto-generated method stub
					guiController.openMenuPressed(callerFrame, editorPane, null);
					
					callerFrame.workspacePane.projectRoot.removeAllChildren();
					
					DefaultMutableTreeNode activeworkspaceNode = new DefaultMutableTreeNode(callerFrame.activeProject.getProjectName());
					String inactiveProjectPath = null;
					int start = 0;
					int end =0;
					
					for (int i = 0; i < callerFrame.activeProjectFile.fileList.size(); i++) {
						inactiveProjectPath = callerFrame.activeProjectFile.fileList.get(i);
						
						start = inactiveProjectPath.lastIndexOf(File.separatorChar)+1;
						end = inactiveProjectPath.length();
						inactiveProjectPath = inactiveProjectPath.substring(start,end);
						
						activeworkspaceNode.add(new DefaultMutableTreeNode(inactiveProjectPath));
					}
					
					callerFrame.workspacePane.projectRoot.add(activeworkspaceNode);
					
					for(int i = 0; i < callerFrame.systemFile.activeWorkspaces.size();i++)
					{
						inactiveProjectPath = (String)callerFrame.systemFile.activeWorkspaces.get(i);
						
						start = inactiveProjectPath.lastIndexOf(File.separatorChar)+1;
						end = inactiveProjectPath.length();
						inactiveProjectPath = inactiveProjectPath.substring(start,end);
						
						callerFrame.workspacePane.projectRoot.add(new DefaultMutableTreeNode(inactiveProjectPath));
						
					}
					
					callerFrame.workspacePane.model.nodeStructureChanged((DefaultMutableTreeNode)callerFrame.workspacePane.model.getRoot());
					
				}});
		}
		return this.openButton;
	}


	/**
	 * This method initializes and returns the save button.	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getSaveButton() {
		if (this.saveButton == null) {
			this.saveButton = new JButton();
			this.saveButton.setIcon(new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/general/Save16.gif")));
			this.saveButton.setName("Save");
			this.saveButton.setToolTipText("Save");
			this.saveButton.setSize(new Dimension(20,20));
			this.saveButton.setPreferredSize(new Dimension(20, 20));
			this.saveButton.setMnemonic(KeyEvent.VK_S);
			this.saveButton.setEnabled(false);
			this.saveButton.addActionListener(new ActionListener(){

				public void actionPerformed(ActionEvent e) {
					// TODO Auto-generated method stub
					guiController.saveMenuPressed(callerFrame, editorPane);
				}});
		}
		return this.saveButton;
	}

	/**
	 * This method initializes and returns the save all button.	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getSaveAllButton() {
		if (this.saveAllButton == null) {
			this.saveAllButton = new JButton();
			this.saveAllButton.setIcon(new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/general/SaveAll16.gif")));
			this.saveAllButton.setName("SaveAll");
			this.saveAllButton.setToolTipText("Save All");
			this.saveAllButton.setSize(new Dimension(20,20));
			this.saveAllButton.setPreferredSize(new Dimension(20, 20));
			this.saveAllButton.setMnemonic(KeyEvent.VK_A);
			this.saveAllButton.setEnabled(false);
			this.saveAllButton.addActionListener(new ActionListener(){

				public void actionPerformed(ActionEvent e) {
					// TODO Auto-generated method stubtt
					int index = editorPane.activeFileIndex;
					int size = editorPane.list.size();
					for (int i = 0; i < size; i++) {
						editorPane.activeFileIndex = i;
						guiController.saveMenuPressed(callerFrame, editorPane);
					}
					editorPane.activeFileIndex = index;
				}});
		}
		return this.saveAllButton;
	}

	/**
	 * This method initializes and returns the cut button.
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getCutButton() {
		if (this.cutButton == null) {
			this.cutButton = new JButton(new DefaultEditorKit.CutAction());
			this.cutButton.addActionListener(new ActionListener(){

				public void actionPerformed(ActionEvent e) {
					// TODO Auto-generated method stub
					//new DefaultEditorKit.CutAction();
					int index = editorPane.activeFileIndex;
					editorPane.list.get(index).changeFlag = 1;
					editorPane.setTitleAt(index, "*"+editorPane.list.get(index).getFileName());
				}});
			this.cutButton.setText("");
			this.cutButton.setIcon(new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/general/Cut16.gif")));
			this.cutButton.setName("cutButton");
			this.cutButton.setToolTipText("Cut");
			this.cutButton.setSize(new Dimension(27, 25));
			this.cutButton.setPreferredSize(new Dimension(27, 25));
			this.cutButton.setMnemonic(KeyEvent.VK_X);
		}
		return this.cutButton;
	}


	/**
	 * This method initializes and returns the copy button.	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getCopyButton() {
		if (this.copyButton == null) {
			this.copyButton = new JButton(new DefaultEditorKit.CopyAction());
			this.copyButton.setText("");
			
			this.copyButton.setToolTipText("Copy");
			this.copyButton.setIcon(new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/general/Copy16.gif")));
			this.copyButton.setName("copyButton");
			this.copyButton.setSize(new Dimension(20,20));
			this.copyButton.setPreferredSize(new Dimension(20,20));
			this.copyButton.setMnemonic(KeyEvent.VK_C);
		}
		return this.copyButton;
	}


	/**
	 * This method initializes and returns the paste button.	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getPasteButton() {
		if (this.pasteButton == null) {
			this.pasteButton = new JButton(new DefaultEditorKit.PasteAction());
			this.pasteButton.setText("");
			this.pasteButton.setIcon(new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/general/Paste16.gif")));
			this.pasteButton.setName("pasteButton");
			this.pasteButton.setToolTipText("Paste");
			this.pasteButton.setMnemonic(KeyEvent.VK_V);
			this.pasteButton.setSize(new Dimension(20,20));
			this.pasteButton.setPreferredSize(new Dimension(20,20));
		}
		return this.pasteButton;
	}



	/**
	 * This method initializes and returns the undo button.	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getUndoButton() {
		if (this.undoButton == null) {
			this.undoButton = new JButton();
			this.undoButton.setIcon(new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/general/Undo16.gif")));
			this.undoButton.setName("undoButton");
			this.undoButton.setToolTipText("Undo");
			this.undoButton.setEnabled(false);
			this.undoButton.setMnemonic(KeyEvent.VK_Z);
			this.undoButton.setSize(new Dimension(20,20));
			this.undoButton.setPreferredSize(new Dimension(20,20));
	
		}
		return this.undoButton;
	}


	/**
	 * This method initializes and returns the redo button.	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getRedoButton() {
		if (this.redoButton == null) {
			this.redoButton = new JButton();
			this.redoButton.setEnabled(false);
			this.redoButton.setMnemonic(KeyEvent.VK_Y);
			this.redoButton.setName("redoButton");
			this.redoButton.setToolTipText("Redo");
			this.redoButton.setIcon(new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/general/Redo16.gif")));
			this.redoButton.setSize(new Dimension(20,20));
			this.redoButton.setPreferredSize(new Dimension(20,20));
		}
		return this.redoButton;
	}


	/**
	 * This method initializes and returns the find button.	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getFindButton() {
		if (this.findButton == null) {
			this.findButton = new JButton();
			this.findButton.setIcon(new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/general/Find16.gif")));
			this.findButton.setName("findButton");
			this.findButton.setToolTipText("Find");
			this.findButton.setMnemonic(KeyEvent.VK_F);
			this.findButton.setSize(new Dimension(20,20));
			this.findButton.setPreferredSize(new Dimension(20,20));
		}
		return this.findButton;
	}


	/**
	 * This method initializes and returns the replace button.	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getReplaceButton() {
		if (this.replaceButton == null) {
			this.replaceButton = new JButton();
			this.replaceButton.setIcon(new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/general/Replace16.gif")));
			this.replaceButton.setToolTipText("Find and Replace");
			this.replaceButton.setName("replaceButton");
			this.replaceButton.setMnemonic(KeyEvent.VK_F);
			this.replaceButton.setSize(new Dimension(20,20));
			this.replaceButton.setPreferredSize(new Dimension(20,20));
			
			replaceButton.addActionListener(new ActionListener(){
				
				public void actionPerformed(ActionEvent arg0) {
					// TODO Auto-generated method stub
					
					int index = editorPane.activeFileIndex;
					System.out.println("index: "+index);
					JTextPane text = null;
					
					if(index > -1)
					{
						text = editorPane.list.get(index).getEditFieldEditorPane();						
						
						Caret selected = new DefaultCaret()
						{
							 public void focusLost(FocusEvent e)
							 {
							 	setVisible(false);
							 }
						};
						selected.setBlinkRate( text.getCaret().getBlinkRate() );
						text.setCaret( selected );
						
						try
						{
							text.read( new FileReader( new File( "FindReplace.java" ) ), "" );
						}
						catch(IOException e)
						{
							System.out.println( e );
						}
						text.getDocument().putProperty( DefaultEditorKit.EndOfLineStringProperty, "\n" );

						//  Create the top-level container and add contents to it.
						
						PFrame frame = callerFrame;//new PFrame();
/*						frame.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
						frame.getContentPane().add( new JScrollPane( text ) );
						frame.setSize(400, 400);
						frame.setVisible( true );
*/
						//  display

						FindReplace.getSharedInstance( frame ).showReplace( text );
					}
				}});
			
		}
		return this.replaceButton;
	}
	/**
	 * This method initializes and returns the workspace toggle button.	
	 * 	
	 * @return javax.swing.JToggleButton	
	 */
	public JToggleButton getWorkspaceToggleButton() {
		if (this.workspaceToggleButton == null) {
			this.workspaceToggleButton = new JToggleButton();
			this.workspaceToggleButton.setMnemonic(KeyEvent.VK_W);
			this.workspaceToggleButton.setToolTipText("Toggle Workspace View");
			this.workspaceToggleButton.setText("WS");
			this.workspaceToggleButton.setName("WorkspaceToggle");
			this.workspaceToggleButton.setFont(new Font("Dialog", Font.BOLD, 10));
		
		}
		return this.workspaceToggleButton;
	}


	/**
	 * This method initializes and returns the console toggle button.	
	 * 	
	 * @return javax.swing.JToggleButton	
	 */
	public JToggleButton getConsoleToggleButton() {
		if (this.consoleToggleButton == null) {
			this.consoleToggleButton = new JToggleButton();
			this.consoleToggleButton.setMnemonic(KeyEvent.VK_C);
			this.consoleToggleButton.setToolTipText("Toggle Console View");
			this.consoleToggleButton.setText("Con");
			this.consoleToggleButton.setName("ConsoleToggleButton");
			this.consoleToggleButton.setFont(new Font("Dialog", Font.BOLD, 10));
			
		}
		return this.consoleToggleButton;
	}

	/**
	 * This method initializes and returns the register toggle button.	
	 * 	
	 * @return javax.swing.JToggleButton	
	 */
	public JToggleButton getRegisterToggleButton() {
		if (this.registerToggleButton == null) {
			this.registerToggleButton = new JToggleButton();
			this.registerToggleButton.setMnemonic(KeyEvent.VK_R);
			this.registerToggleButton.setText("Reg");
			this.registerToggleButton.setToolTipText("Toggle Register View");
			this.registerToggleButton.setName("debuggerToggleButton");
			this.registerToggleButton.setActionCommand("");
			this.registerToggleButton.setEnabled(false);
			this.registerToggleButton.setFont(new Font("Dialog", Font.BOLD, 10));
			
		}
		return this.registerToggleButton;
	}
	

	/**
	 * This method initializes and returns the watchpoint toggle button.	
	 * 	
	 * @return javax.swing.JToggleButton	
	 */
	public JToggleButton getWatchpointToggleButton() {
		if (this.watchpointToggleButton == null) {
			this.watchpointToggleButton = new JToggleButton();
			this.watchpointToggleButton.setMnemonic(KeyEvent.VK_P);
			this.watchpointToggleButton.setToolTipText("Toggle WatchPoint View");
			this.watchpointToggleButton.setText("WPt");
			this.watchpointToggleButton.setActionCommand("WPt");
			this.watchpointToggleButton.setEnabled(false);
			this.watchpointToggleButton.setName("watchpointToggleButton");
			this.registerToggleButton.setFont(new Font("Dialog", Font.BOLD, 10));
		}
		return this.watchpointToggleButton;
	}



	/**
	 * This method initializes and returns the build button.	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getBuildButton() {
		if (this.buildButton == null) {
			this.buildButton = new JButton();
			this.buildButton.setToolTipText("Build");
			this.buildButton.setIcon(new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/general/Properties16.gif")));
			this.buildButton.setName("buildButton");
			this.buildButton.setMnemonic(KeyEvent.VK_F5);
			this.buildButton.setSize(new Dimension(20,20));
			this.buildButton.setPreferredSize(new Dimension(20,20));
			this.buildButton.addActionListener(new ActionListener(){

				public void actionPerformed(ActionEvent e) {
					// TODO Auto-generated method stub
					String s = "", path = "";
					if(!editorPane.list.isEmpty())
					{
						if(editorPane.list.get(editorPane.getSelectedIndex()).changeFlag != 0)
						{
							
				    		guiController.saveMenuPressed(callerFrame, editorPane);
						}
						
						s = editorPane.list.get(editorPane.activeFileIndex).getFileName();
						path = editorPane.list.get(editorPane.activeFileIndex).getFilePath();
						int limit =  0;
						for (int i = 0; i < s.length(); i++) {
							if( s.charAt(i) == '.')
								limit = i;
						}
						if(limit != 0)
							s = s.substring(0, limit);

						/* BURADA BREAKPOINT LISTESI ASSEMBLERA GONDERILCEK*/
							String path2=new String();
							int index = editorPane.activeFileIndex;
							ArrayList bpLnum = editorPane.list.get(index).bpList;
							
							ArrayList bpAddress = new ArrayList();
							//bpLnum.add(21);	bpLnum.add(22);bpLnum.add(23);bpLnum.add(24);

							int result = assembler.asmLexer.assemble(s, path, consolePane, bpLnum, bpAddress);					
							
								
							for (int i = 0; i < bpAddress.size(); i++) {
								System.out.println("ADRESS " + bpAddress.get(i));
								
							}
							
							
							callerFrame.statusBarLabel.setText(" ");
					}
					else callerFrame.statusBarLabel.setText("There is no file to build");
				}				
			});
		}
		return this.buildButton;
	}


	/**
	 * This method initializes and returns the start simulate button.	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getStartSimulateButton() {
		if (this.startSimulateButton == null) {
			this.startSimulateButton = new JButton();
			this.startSimulateButton.setIcon(new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/media/Play16.gif")));
			this.startSimulateButton.setMnemonic(KeyEvent.VK_F6);
			this.startSimulateButton.setToolTipText("Simulate");
			this.startSimulateButton.setName("startSimulateButton");
			this.startSimulateButton.setSize(new Dimension(20,20));
			this.startSimulateButton.setPreferredSize(new Dimension(20,20));
			this.startSimulateButton.addMouseListener(new java.awt.event.MouseListener() {
				public void mousePressed(java.awt.event.MouseEvent e) {
					if(simulatorFrame != null)
					{	
						return;
					}
					String s = "", path = "";
					if(!editorPane.list.isEmpty())
					{
						if(editorPane.list.get(editorPane.getSelectedIndex()).changeFlag != 0)
						{
							System.out.println("HEBELE");
				    		guiController.saveMenuPressed(callerFrame, editorPane);
						}
						
							s = editorPane.list.get(editorPane.activeFileIndex).getFileName();
							path = editorPane.list.get(editorPane.activeFileIndex).getFilePath();
							System.out.println("filename" + s);
							System.out.println("filepath" + path);
/*							
							int limit =  0;
							for (int i = 0; i < s.length(); i++) {
								if( s.charAt(i) == '.')
									limit = i;
							}
							if(limit != 0)
								s = s.substring(0, limit);
*/							
							String path2=new String();
							int limit =  0;
							for (int i = path.length()-1; i>=0; i--) {
								if( path.charAt(i) == '.')
								{
									limit = i;
									break;
								}
								
							}
							if(limit != 0)
								path2 = path.substring(0, limit);
							else
								System.out.println("invalid filename - length=0");
							
							System.out.println("PATH2 = " + path2);
							/* BURADA BREAKPOINT LISTESI ASSEMBLERA GONDERILCEK*/
								
								int index = editorPane.activeFileIndex;
								ArrayList bpLnum = editorPane.list.get(index).bpList;
								
								ArrayList bpAddress = new ArrayList();
								//bpLnum.add(21);	bpLnum.add(22);bpLnum.add(23);bpLnum.add(24);

								int result = assembler.asmLexer.assemble(s, path, consolePane, bpLnum, bpAddress);
								
								for (int i = 0; i < bpAddress.size(); i++) {
									System.out.println("ADRESS " + bpAddress.get(i));
									
								}
								
								if(result == 1)
								{		
									
										simulatorFrame= new SimulatorFrame(path2,consolePane, bpAddress);
										//SimulatorFrame simulatorFrame= new SimulatorFrame(s,consolePane);
										simulatorFrame.setpFrame(callerFrame);
										simulatorFrame.setVisible(true);									
										
										simulatorFrame.setTitle("Simulation - PIDE - " + simulatorFrame.getPic().fileName + ".HEX");
								
								}
								
							callerFrame.statusBarLabel.setText(" ");
							
					}
					else callerFrame.statusBarLabel.setText("There is no file to simulate");
					
				}
				public void mouseClicked(java.awt.event.MouseEvent e) {
				}
				public void mouseReleased(java.awt.event.MouseEvent e) {
				}
				public void mouseEntered(java.awt.event.MouseEvent e) {
				}
				public void mouseExited(java.awt.event.MouseEvent e) {
				}
			});
		}
		return this.startSimulateButton;
	}


	/**
	 * This method initializes and returns the stop simulate button.	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getStopSimulateButton() {
		if (this.stopSimulateButton == null) {
			this.stopSimulateButton = new JButton();
			this.stopSimulateButton.setToolTipText("Stop Simulation");
			this.stopSimulateButton.setMnemonic(KeyEvent.VK_F7);
			this.stopSimulateButton.setIcon(new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/media/Stop16.gif")));
			this.stopSimulateButton.setEnabled(false);
			this.stopSimulateButton.setName("stopSimulateButton");
			this.stopSimulateButton.setSize(new Dimension(20,20));
			this.stopSimulateButton.setPreferredSize(new Dimension(20,20));
		}
		return this.stopSimulateButton;
	}


	/**
	 * This method initializes and returns the replace button.	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getstepButton() {
		if (this.stepButton == null) {
			this.stepButton = new JButton();
			this.stepButton.setIcon(new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/general/StepForward16.gif")));
			this.stepButton.setToolTipText("Step Forward");
			this.stepButton.setName("Step Forward");
			this.stepButton.setMnemonic(KeyEvent.VK_S);
			this.stepButton.setSize(new Dimension(20,20));
			this.stepButton.setPreferredSize(new Dimension(20,20));
			
			stepButton.addActionListener(new ActionListener(){
				
				public void actionPerformed(ActionEvent arg0) {
					// TODO Auto-generated method stub
					if(simulatorFrame != null)
					{
						simulatorFrame.pic.task1.step = true;
					}
						
				}});
			
		}
		return this.stepButton;
	}



}  //  @jve:decl-index=0:visual-constraint="19,10"
