package gui;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import projectManager.Project;
import projectManager.ProjectFile;
/**
 * This class is the workspace panel where all workspaces are shown in a
 * tree structure.
 * 
 */
@SuppressWarnings("serial")
public class WorkspacePane extends JTabbedPane {

	private JScrollPane WorkspaceScrollPane = null;
	public DefaultMutableTreeNode projectRoot = new DefaultMutableTreeNode("Projects");
	public DefaultTreeModel model = new DefaultTreeModel(projectRoot);
	public JTree workspaceTree = null;
	private String[] workspaceList = null;
	private int noOfWorkspaces = 0;
	public ProjectFile activeProjectFile;
	private Project activeProject;
	private JPopupMenu filePopupMenu,projectPopupMenu ;
	private JMenuItem[] fileItems,projectItems;
	private String[] fileItemList = {"Open","Remove from Project"}; 
	private String[] projectItemList = {"Activate","Add New File","Add File..."}; 
	public String selectedFile = "";
	public GuiController guiController = new GuiController();
	public PFrame callerFrame;
	public EditorPane editorPane;

	/**
	 * 
	 * The main constructor of the workspace panel class. It creates the
	 * workspace tree and pop up menus for tree elements.
	 */
	public WorkspacePane(String[] workspaceList, int noOfWorkspaces, ProjectFile activeProjectFile, PFrame cFrame, EditorPane ePane, Project aProject) {
		// TODO Auto-generated constructor stub
		this.workspaceList = workspaceList;
		this.noOfWorkspaces = noOfWorkspaces;
		this.activeProjectFile = activeProjectFile;
		this.callerFrame = cFrame;
		this.editorPane = ePane;
		this.activeProject = aProject;
		this.initialize();
	}
		


	/**
	 * This method initializes panel, workspace tree and popup menus.
	 * 
	 */
	private void initialize() {
        this.setSize(new Dimension(618, 72));
        this.addTab("Workspace", null, this.getWorkspaceScrollPane(), null);
        
        FilePopupMenuHandler filePopupMenuHandler = new FilePopupMenuHandler();
        ProjectPopupMenuHandler projectPopupMenuHandler = new ProjectPopupMenuHandler();
        
        
        
        filePopupMenu = new JPopupMenu();
        fileItems = new JMenuItem[fileItemList.length];
        
        for(int i=0;i<fileItemList.length;i++)
        {
        	fileItems[i] = new JMenuItem(fileItemList[i]);
        	filePopupMenu.add(fileItems[i]);
        	fileItems[i].addActionListener(filePopupMenuHandler);
        }
        
        projectPopupMenu = new JPopupMenu();
        projectItems = new JMenuItem[projectItemList.length];
        
        for(int i=0;i<projectItemList.length;i++)
        {
        	projectItems[i] = new JMenuItem(projectItemList[i]);
        	projectPopupMenu.add(projectItems[i]);
        	projectItems[i].addActionListener(projectPopupMenuHandler);
        }
        
        
			
	}


	/**
	 * This method initializes and returns the workspace scroll pane.	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getWorkspaceScrollPane() {
		if (this.WorkspaceScrollPane == null) {
			this.WorkspaceScrollPane = new JScrollPane();
			this.WorkspaceScrollPane.setViewportView(this.getWorkspaceTree());
		}
		return this.WorkspaceScrollPane;
	}
	
	/**
	 * This method initializes and returns the workspace tree.	
	 * 	
	 * @return javax.swing.JTree	
	 */
	private JTree getWorkspaceTree() {
		if (this.workspaceTree == null) {
			workspaceTree = new JTree(model);
			DefaultMutableTreeNode workspaceDirectory = null;
			DefaultMutableTreeNode dummy=null;
			String inactiveProjectPath = null;
			int start = 0;
			int end =0;
			
			if(noOfWorkspaces>=1)
			{
				workspaceDirectory = new DefaultMutableTreeNode(workspaceList[0]);
				
				for (int i = 0; i < activeProjectFile.fileList.size(); i++) {
					inactiveProjectPath = activeProjectFile.fileList.get(i);
					
					start = inactiveProjectPath.lastIndexOf(File.separatorChar)+1;
					end = inactiveProjectPath.length();
					inactiveProjectPath = inactiveProjectPath.substring(start,end);
					dummy =new DefaultMutableTreeNode(inactiveProjectPath);
					workspaceDirectory.add(dummy);
				}
				
				projectRoot.add(workspaceDirectory);
			}
			for(int i = 1; i < noOfWorkspaces;i++)
			{
				
				workspaceDirectory = new DefaultMutableTreeNode(workspaceList[i]);
				projectRoot.add(new DefaultMutableTreeNode(workspaceList[i]));
				
				
			}
			
			workspaceTree.addMouseListener(new MouseListener(){

				private void processMouseClick(MouseEvent e)
				{
					TreePath selectedItem =  workspaceTree.getPathForLocation(e.getX(), e.getY());

					if(e.isPopupTrigger())
					{	
												
						int limit = 0, count = 0;
						String item = selectedItem.toString();
						limit = 0;
						for (int i = 0; i < item.length(); i++) {
							
							if(item.charAt(i) == ',')
							{
								count++;
								if(count == 2) 
								{
									limit = i;
									break;
								}
							}
						}
						
						item = item.substring(limit+2, item.length()-1);
						
						
						if(activeProjectFile != null)
						selectedFile = activeProjectFile.getFilePath()+item;
					
						
						if(selectedItem != null)
						{
							workspaceTree.setSelectionPath(selectedItem);
							String selected =selectedItem.toString().substring(1, selectedItem.toString().length()-1);
							
							if(selected.lastIndexOf(".")>0)//this is a file
							{
								filePopupMenu.show(e.getComponent(), e.getX(), e.getY());
							}
							else if(selected.compareTo("Projects") != 0)//this is a project.Do not show popup for root
							{
								
								projectPopupMenu.show(e.getComponent(), e.getX(), e.getY());
							}
						}
					}
					else if(e.getClickCount() == 2)
					{
						if(selectedItem.toString().lastIndexOf(".")>0)//this is a file
						{
							//System.out.println("hopp");
							
							int limit = 0, count = 0;
							String item = selectedItem.toString();
							limit = 0;
							for (int i = 0; i < item.length(); i++) {
								
								if(item.charAt(i) == ',')
								{
									count++;
									if(count == 2) 
									{
										limit = i;
										break;
									}
								}
							}
							
							item = item.substring(limit+2, item.length()-1);
							
							selectedFile = activeProjectFile.getFilePath()+item;
						
							guiController.openMenuPressed(callerFrame, editorPane, selectedFile);
						}
					}
				}
				public void mouseClicked(MouseEvent e) {
					// TODO Auto-generated method stub
				
					
				}

				public void mouseEntered(MouseEvent e) {
					// TODO Auto-generated method stub
					
				}

				public void mouseExited(MouseEvent e) {
					// TODO Auto-generated method stub
					
				}

				public void mousePressed(MouseEvent e) {
					// TODO Auto-generated method stub
					processMouseClick(e);
				}

				public void mouseReleased(MouseEvent e) {
					// TODO Auto-generated method stub
					processMouseClick(e);
				}
				
			});

		}
		return this.workspaceTree;
	}
	/**
	 * 
	 * This method is the handler for the popup menu for the files
	 * in the workspace tree. A popup menu is shown when an item
	 * in the workspace tree is right-clicked.
	 *
	 */
	private class FilePopupMenuHandler implements ActionListener{

		public void actionPerformed(ActionEvent arg0) {
			// TODO Auto-generated method stub
			//System.out.println("File menu handler calls "+ arg0.getSource());
			JMenuItem item = (JMenuItem)arg0.getSource();
			if(item.getText() == "Open")
			{
				guiController.openMenuPressed(callerFrame, editorPane, selectedFile);
			}
			else if(item.getText() == "Remove from Project")
			{
				
				File trash = new File(selectedFile);
				trash.delete();
				//filePath = filePath.substring(2, filePath.length());
				callerFrame.activeProject.projectFile.fileList.remove(selectedFile);
				callerFrame.activeProject.projectFile.save();
				
				
				callerFrame.workspacePane.projectRoot.removeAllChildren();
				
				DefaultMutableTreeNode activeworkspaceNode = new DefaultMutableTreeNode(callerFrame.activeProject.getProjectName());
				String inactiveProjectPath = null;
				int start = 0;
				int end =0;
				
				for (int i = 0; i < callerFrame.activeProjectFile.fileList.size(); i++) {
					inactiveProjectPath = callerFrame.activeProjectFile.fileList.get(i);
					
					start = inactiveProjectPath.lastIndexOf(File.separatorChar)+1;
					end = inactiveProjectPath.length();
					inactiveProjectPath = inactiveProjectPath.substring(start,end);
					
					activeworkspaceNode.add(new DefaultMutableTreeNode(inactiveProjectPath));
				}
				
				callerFrame.workspacePane.projectRoot.add(activeworkspaceNode);
				
				for(int i = 0; i < callerFrame.systemFile.activeWorkspaces.size();i++)
				{
					inactiveProjectPath = (String)callerFrame.systemFile.activeWorkspaces.get(i);
					
					start = inactiveProjectPath.lastIndexOf(File.separatorChar)+1;
					end = inactiveProjectPath.length();
					inactiveProjectPath = inactiveProjectPath.substring(start,end);
					
					callerFrame.workspacePane.projectRoot.add(new DefaultMutableTreeNode(inactiveProjectPath));
					
				}
				
				callerFrame.workspacePane.model.nodeStructureChanged((DefaultMutableTreeNode) callerFrame.workspacePane.model.getRoot());
			}
		}
		
	}
	/**
	 * 
	 * This method is the handler for the popup menu for the projects
	 * in the workspace tree. A popup menu is shown when an item
	 * in the workspace tree is right-clicked.
	 *
	 */
	private class ProjectPopupMenuHandler implements ActionListener{

		public void actionPerformed(ActionEvent arg0) {
			// TODO Auto-generated method stub
			//System.out.println("Project menu handler calls "+ arg0.getSource());
			JMenuItem item = (JMenuItem)arg0.getSource();
			if(item.getText() == "Add New File")
			{
					NewFileFrame nff = new NewFileFrame(callerFrame,editorPane);
					nff.setVisible(true);
			}
			else if(item.getText() == "Add File...")
			{
					guiController.openMenuPressed(callerFrame, editorPane, null);
					projectRoot.removeAllChildren();
					
					DefaultMutableTreeNode activeworkspaceNode = new DefaultMutableTreeNode(callerFrame.activeProject.getProjectName());
					String inactiveProjectPath = null;
					int start = 0;
					int end =0;
					
					for (int i = 0; i < callerFrame.activeProjectFile.fileList.size(); i++) {
						inactiveProjectPath = callerFrame.activeProjectFile.fileList.get(i);
						
						start = inactiveProjectPath.lastIndexOf(File.separatorChar)+1;
						end = inactiveProjectPath.length();
						inactiveProjectPath = inactiveProjectPath.substring(start,end);
						
						activeworkspaceNode.add(new DefaultMutableTreeNode(inactiveProjectPath));
					}
					
					projectRoot.add(activeworkspaceNode);
					
					for(int i = 0; i < callerFrame.systemFile.activeWorkspaces.size();i++)
					{
						inactiveProjectPath = (String)callerFrame.systemFile.activeWorkspaces.get(i);
						
						start = inactiveProjectPath.lastIndexOf(File.separatorChar)+1;
						end = inactiveProjectPath.length();
						inactiveProjectPath = inactiveProjectPath.substring(start,end);
						
						callerFrame.workspacePane.projectRoot.add(new DefaultMutableTreeNode(inactiveProjectPath));
						
					}
					
					model.nodeStructureChanged((DefaultMutableTreeNode) model.getRoot());
			}
		}
		
	}

}  //  @jve:decl-index=0:visual-constraint="20,69"
