package peripherals;

import picmicrocontroller.PIC;
import picmicrocontroller.Register;
import simulator.KeyPadPanel;
/**
 * This class is created in order to keep the information of 16 
 * Push Buttons. The class is inherited from the Peripheral class.
 * @author maho
 * 
 */
public class KeyPad  extends Peripheral {

	////////////////ATTRIBUTES/////////////////////
	/***/
	private PushButton keyPad[][];
	private KeyPadPanel keyPadPanel;
	
	/////////////CONSTRUCTORS////////////////////////
	/**
	 * Main constructor which initializes 4*4 PushButton objects with default settings.
	 * @author maho
	 * 
	 */
	public KeyPad(PIC pic)
	{
		super(KEY_PAD, pic);
		this.keyPadPanel = new KeyPadPanel(this);
		this.keyPad =new PushButton[4][4];
		for (int i = 0 ; i < 4 ; i++)
		{
			for (int j = 0 ; j < 4; j++)
			{
				this.keyPad[i][j] = new  PushButton(i*4+j);
			}
		}
	}
	/**
	 * This method sets the state of the button with the given id.
	 * @param buttonID
	 * @param state
	 */
	public void setButtonState(int buttonID, boolean state)
	{
		this.keyPad[buttonID/4][buttonID%4].setButtonState(state);
		//System.out.println("button "+buttonID/4+" "+buttonID%4+" has state "+state+ " now");
	}

	@Override
	public void update() {
		
		Register trisb = this.pic.getDataMemory().getTRISB();
		Register portb = this.pic.getDataMemory().getPORTB();
		
		Register portbtemp = new Register();
		portbtemp.write(portb.read());
		
		// assume all pull ups are active
		portbtemp.write((byte) (portbtemp.read()| trisb.read()));
		
		for(int i=0;i<4;i++)
		{
			for(int j=0;j<4;j++)
			{
				if(this.keyPad[i][j].isButtonState() == PushButton.BUTTON_DOWN)
				{
					if(trisb.getBit(i)==1)
					{
						if(trisb.getBit(j+4)==1)
						{
							// both inputs
							//do nothing
							//System.out.println("i input j input");
						}else
						{
							// i input, (j+4) output
							// drive i from (j+4)
							//System.out.println("i input j output");
							portbtemp.updateBit(i, (byte)portbtemp.getBit(j+4));
						}
					}else
					{
						if(trisb.getBit(j+4)==1)
						{
							//System.out.println("i output j input");
							// i output, (j+4) input
							// drive (j+4) from i
							portbtemp.updateBit(j+4, (byte)portbtemp.getBit(i));
						}else
						{
							//both outputs
							
							//System.out.println("SHORT CIRCUIT ERROR");
							System.exit(1);
						}
						
					}
				}else // button is up 
				{
					// do nothing
				}
				
				portb.write(portbtemp.read());
				
			}
		}
	}
	/**
	 * This method returns the visual represantation of the key pad.
	 * @return KeyPadPanel
	 */
	public KeyPadPanel getKeyPadPanel() {
		// TODO Auto-generated method stub
		return this.keyPadPanel;
	}
}
