/**
 * 
 */
package peripherals;

import picmicrocontroller.PIC;

/**
 * This is a general class representing the peripherals on the CENG336 Board.
 * Specific peripherals are inherited from this class.
 * @author maho
 * @since 19.01.2007
 */
public abstract class Peripheral {
	//////////////////CONSTANTS////////////////////
	/*IDs of peripherals*/
	protected static int SEVEN_SEGMENT_DISPLAY_ARRAY = 0;
	protected static int LED_ARRAY = 1;
	protected static int KEY_PAD = 2;
	protected static int POTENTIOMETER = 3;
	////////////////ATTRIBUTES/////////////////////
	/**Identification number of the peripherals on Ceng 336 Board.
	 * All peripherals have one instance so each peripheral type has a unique ID.*/
	protected int id;
	/**Value representing if the peripheral is activated during simulation.*/
	protected boolean isEnabled;
	/**Reference of microcontroller*/
	protected PIC pic;
	
	
	/////////////CONSTRUCTORS////////////////////////
	/**
	 * Constructor with default value(true) of isEnabled.
	 * Id should be specified
	 * @param id type-unique id
	 * @author maho
	 * @since 19.01.2007
	 */
	public Peripheral(int id, PIC pic) {
		this.id = id;
		this.pic = pic;
		this.isEnabled = true;
	}
	/**
	 * Main constructor of the Peripheral class.
	 * @param id		type-unique id
	 * @param isEnabled status of peripheral
	 * @author maho
	 * @since 19.01.2007
	 */
	public Peripheral(int id, PIC pic, boolean isEnabled) {
		this.id = id;
		this.pic = pic;
		this.isEnabled = isEnabled;
	}
	
	/////////////////METHODS/////////////////////////////
	/**
	 * This abstract method communicates with the virtual card and 
	 * makes necessary changes on it.
	 * @author maho
	 * @since 19.01.2007
	 */
	public abstract void update();	
	
	/////////GETTER SETTER METHODS///////////////////////
	/**
	 * @return the id
	 * @author maho
	 * @since 19.01.2007
	 */
	public int getId() {
		return this.id;
	}
	/**
	 * @param id the id to set
	 * @author maho
	 * @since 19.01.2007
	 */
	public void setId(int id) {
		this.id = id;
	}
	/**
	 * @return the isEnabled
	 * @author maho
	 * @since 19.01.2007
	 */
	public boolean isEnabled() {
		return this.isEnabled;
	}
	/**
	 * @param isEnabled the isEnabled to set
	 * @author maho
	 * @since 19.01.2007
	 */
	public void setEnabled(boolean isEnabled) {
		this.isEnabled = isEnabled;
	}
	/**
	 * @return the pic
	 * @author maho
	 * @since 21.01.2007
	 */
	public PIC getPic() {
		return this.pic;
	}
	/**
	 * @param pic the pic to set
	 * @author maho
	 * @since 21.01.2007
	 */
	public void setPic(PIC pic) {
		this.pic = pic;
	}
	
	
}
