/**
 * 
 */
package peripherals;

import picmicrocontroller.PIC;
import picmicrocontroller.Register;
import simulator.PotentiometerPanel;

/**
 * This class represents the potentiometer on CENG336 Board.
 * @author e1347061
 *
 */
public class Potentiometer extends Peripheral {

	
	////////////////ATTRIBUTES/////////////////////
	/**The analog data that this potentiometer is generating*/
	private double analogData=0;
	
	
	private PotentiometerPanel pPanel = null;
	/**
	 * Constructor that initializes the potentiometer.
	 * @param id
	 * @param pic
	 */
	public Potentiometer( PIC pic) {
		
		super(Peripheral.POTENTIOMETER, pic);
		this.analogData = 0;
		this.pPanel = new PotentiometerPanel(this);
	
	}

	/**
	 * Main constructor that initializes the potentiometer.
	 * @param id
	 * @param pic
	 * @param isEnabled
	 */
	public Potentiometer( PIC pic, boolean isEnabled) {
		super(Peripheral.POTENTIOMETER, pic, isEnabled);
		this.analogData = 0;
		this.pPanel = new PotentiometerPanel(this);
	}

	/* (non-Javadoc)
	 * @see peripherals.Peripheral#update()
	 */
	@Override
	public void update() {
		// TODO Auto-generated method stub
		//System.out.println("current value: "+analogData);
		Register adcon0 = this.pic.getDataMemory().getADCON0();
		if(adcon0.getBit(0) == 1)
		{
			//find the pin that te analog data will be sent
			int analogDataIndex = adcon0.getBit(3) +  2 * adcon0.getBit(4) + 4 * adcon0.getBit(5);
			
			this.pic.analog[analogDataIndex] = analogData;
		}

	}

	/**
	 * @return the analogData
	 */
	public double getAnalogData() {
		return analogData;
	}

	/**
	 * @param analogData the analogData to set
	 */
	public void setAnalogData(double analogData) {
		this.analogData = analogData;
	}

	/**
	 * This method returns the visual representation of the potentiometer.
	 */
	public PotentiometerPanel getPPanel() {
		return pPanel;
	}


}
