/**
 * 
 */
package peripherals;

/**
 * This class represents a single push button display of the CENG336 Board.
 * @author maho
 * @since 20.01.2007
 */
public class PushButton {

	//////////////CONSTANTS//////////////////////
	public static boolean BUTTON_DOWN = true;
	public static boolean BUTTON_UP = false;
	
	//////////////ATTRIBUTES/////////////////////
	/**Button id.This id represents the place of the button in KeyPad*/
	private int buttonId;
	/**Value representing if the button unit is activated during simulation.*/
	private boolean isEnabled;
	/**Push state of the button*/
	private boolean buttonState;
	
	/////////////CONSTRUCTORS////////////////////////
	/**
	 * Constructor with default values.
	 * Id should be specified.
	 * @param buttonId instance-unique id
	 * @author maho
	 * @since 20.01.2007
	 */
	public PushButton(int buttonId) {
		this.buttonId = buttonId;
		this.buttonState = BUTTON_UP;
		this.isEnabled = true;
		
	}	
	
	/**
	 * Main constructor of the PushButton class.
	 * @param ssId		instance-unique id 
	 * @param isEnabled push status of button
	 * @author maho
	 * @since 20.01.2007
	 */
	public PushButton(int buttonId, boolean isEnabled) {
		this.buttonId = buttonId;
		this.buttonState = BUTTON_UP;
		this.isEnabled = isEnabled;
	}
	
	
	/////////GETTER SETTER METHODS///////////////////////
	/**
	 * @return the buttonId
	 */
	public int getButtonId() {
		return this.buttonId;
	}

	/**
	 * @param buttonId the buttonId to set
	 */
	public void setButtonId(int buttonId) {
		this.buttonId = buttonId;
	}

	/**
	 * @return the buttonState
	 */
	public boolean isButtonState() {
		return this.buttonState;
	}

	/**
	 * @param buttonState the buttonState to set
	 */
	public void setButtonState(boolean buttonState) {
		this.buttonState = buttonState;
	}

	/**
	 * @return the isEnabled
	 */
	public boolean isEnabled() {
		return this.isEnabled;
	}

	/**
	 * @param isEnabled the isEnabled to set
	 */
	public void setEnabled(boolean isEnabled) {
		this.isEnabled = isEnabled;
	}
	
}
