/**
 * 
 */
package peripherals;

import java.awt.Graphics;

import javax.swing.JOptionPane;

import picmicrocontroller.PIC;
import picmicrocontroller.Register;
import simulator.SevenSegmentPanel;

/**
 * This class is created in order to keep the information of three 
 * 7-Segment Displays. The class is inherited from the Peripheral class.
 * @author maho
 * @since 19.01.2007
 */
public class SevenSegmentDisplayArray extends Peripheral {
	//////////////////CONSTANTS////////////////////
	/**Number of SSD elements to be simulated*/
	private static int NO_OF_SSD = 3;
	//private static int MAX_COUNTER = 20;
	////////////////ATTRIBUTES/////////////////////
	/**Array of seven segment displays. The board has 3 SSDs.*/
	private SevenSegmentDisplay sevenSegmentDisplayArray[];
	
	private SevenSegmentPanel ssdPanel;

	public boolean sticky;
	//private int counter;
	
	
	/////////////CONSTRUCTORS////////////////////////
	/**
	 * Constructor which initializes 3 SSD objects with default settings.
	 * @author maho
	 * @since 19.01.2007
	 */
	public SevenSegmentDisplayArray(PIC pic) {
		
		super(SEVEN_SEGMENT_DISPLAY_ARRAY, pic, true);
		this.ssdPanel = new SevenSegmentPanel();
		this.sevenSegmentDisplayArray = new SevenSegmentDisplay[NO_OF_SSD];
		//id 0-->rightmost id 1-->middle id 2-->leftmost
		for(int i=0;i<NO_OF_SSD;i++)
		{
			this.sevenSegmentDisplayArray[i] = new SevenSegmentDisplay(i);
		}
	}
	
	/**
	 * Main constructor of SevenSegmentDisplayArray class.
	 * @param isEnabled status of SSDA
	 * @author maho
	 * @since 19.01.2007
	 */
	public SevenSegmentDisplayArray(PIC pic, boolean isEnabled ) {
		super(SEVEN_SEGMENT_DISPLAY_ARRAY, pic, isEnabled);
		this.ssdPanel = new SevenSegmentPanel();
		this.sevenSegmentDisplayArray = new SevenSegmentDisplay[NO_OF_SSD];
		
		for(int i=0;i<NO_OF_SSD;i++)
		{
			this.sevenSegmentDisplayArray[i] = new SevenSegmentDisplay(i);
		}
	}

	/////////////////METHODS/////////////////////////////

	@Override
	public void update() {
		
		//System.out.println("SevenSegmentDisplayArray.update()");
		byte trisD = this.pic.getDataMemory().getTRISD().read();
		//mask with TRISD so that only output pins are reflected
		
		byte output = (byte) (this.pic.getDataMemory().getPORTD().read() & ~trisD);
		Register porte = this.pic.getDataMemory().getPORTE();
		for(int i=0;i<NO_OF_SSD;i++)
		{
			//byte porteBit =(byte) ((1<<i)&(pic.getDataMemory().getPORTE().read())>>i);
			//System.out.print("ssd: " + i + "  portebit " + porte.getBit(i));
			
			if(porte.getBit(i)== 1 )
			{
				this.sevenSegmentDisplayArray[i].setEnabled(true);
				if(output != 0)
					this.sevenSegmentDisplayArray[i].writeData(output);		
				else if(!sticky)
					this.sevenSegmentDisplayArray[i].writeData(output);		
				this.sevenSegmentDisplayArray[i].update();
				//System.out.println("    data: " + this.sevenSegmentDisplayArray[i].readData());
			}
			else
			{
				if(!sticky)
				{
					//System.out.println( "oops"+i);
					this.sevenSegmentDisplayArray[i].setEnabled(false);
					//this.sevenSegmentDisplayArray[i].writeData((byte) 0);
					this.sevenSegmentDisplayArray[i].update();
					//System.out.println("");
				}
				else {
					//System.out.println( "oops");
					this.sevenSegmentDisplayArray[i].setEnabled(true);
					this.sevenSegmentDisplayArray[i].writeData((byte) this.sevenSegmentDisplayArray[i].readData());
					//System.out.println("    data: " + this.sevenSegmentDisplayArray[i].readData());
					this.sevenSegmentDisplayArray[i].update();
				}
				
			}
			
			this.ssdPanel.changeState(i, this.sevenSegmentDisplayArray[i].bits);
		}
		
		
	}
	/**
	 * 
	 * This method returns the visual represantation of the SSD array.
	 */
	public SevenSegmentPanel getSsdPanel() {
		return this.ssdPanel;
	}


	
	
	
}
