/**
 * 
 */
package picmicrocontroller;


import gui.ConsolePane;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.Stack;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import simulator.SimThread;
import simulator.SimulatorFrame;
/**
 * This class represents a PIC 16F877.
 *
 */
public class PIC {
	
	//////////////////CONSTANTS////////////////////
	public static int BANK0 = 0;
	public static int BANK1 = 1;
	public static int BANK2 = 2;
	public static int BANK3 = 3;
	
	public static int _W = 0;
	
	public static final int INDIRECT = 0;
	public static final int TMR0 = 1;
	public static final int PCL = 2;
	public static final int STATUS = 3;
	public static final int FSR = 4;
	public static final int PCLATH = 10;
	public static final int INTCON = 11;
	public static final int ADRESH = 30;
	public static final int ADRESL = 30;
	public static final int ADCON0 = 31;
	public static final int ADCON1 = 31;
	
	public static final int OPTION_REG = 1;
	public static final int PR2 = 12;
	public static final int TXSTA = 18;
	
	public static int _PORTA = 5;
	public static int _PORTB = 6;
	public static int _PORTC = 7;
	public static int _PORTD = 8;
	public static int _PORTE = 9;
	
	public static int _TRISA = 5;
	public static int _TRISB = 6;
	public static int _TRISC = 7;
	public static int _TRISD = 8;
	public static int _TRISE = 9;
	
	////////////////ATTRIBUTES/////////////////////
	/***/
	protected DataMemory dataMemory;
	/***/
	public FlashProgramMemory flashProgramMemory;
	/***/
	protected Register wreg;
	/***/
	protected ProgramCounter programCounter;
	/***/	
	protected Stack ProgramCounterStack;
	/***/
	protected Register WDT;
	/***/
	protected ADConverter adConverter;
	/**analog data
	 * analog[0-3] 	PORTA.0-3
	 * analog[4]	PORTA.5
	 * analog [5-7]	PORTE.0-2*/
	
	public double analog[];
	
	public SimulatorFrame simulatorFrame;
		
	public ExecutorService threadExecutor =null;
	public SimThread task1 = null;
	
	public boolean threadPaused = false;
	public boolean threadFromPause = false;
	
	public String fileName; 
	
	/////////////CONSTRUCTORS////////////////////////
	/**
	 * Main constructor of the PIC class. Initializes memories and
	 * modules of PIC 16F877
	 */
	public PIC(SimulatorFrame frame, String name) {
		this.analog = new double[8];
		this.fileName = name;
		for(int i=0;i<8;i++)
			analog[i] = (double)0;
		this.simulatorFrame=frame;
		this.dataMemory = new DataMemory(this);
		this.flashProgramMemory = new FlashProgramMemory();
		this.programCounter = new ProgramCounter(this);
		this.ProgramCounterStack = new Stack();
		this.wreg = new Register();
		this.WDT = new Register();
		this.adConverter = new ADConverter(this);
	}
	/////////////////METHODS/////////////////////////////
	/**
	 * This method gets a line from the hex file and parses it which gives 
	 * instruction codes. Then it writes the instructions into flash memory.
	 */
	private void writeLine(String line)
	{
		//System.out.println("..................................");
		int numberOfData = 0;
		int type=0;
		int offset=0;
		int [] iarr = null;
		int itempM=0;
		int itempL=0;
		
		int position=0;
		
		String stemp;
		
		stemp = line.substring(1, 3);
			numberOfData = Integer.parseInt(stemp, 16)/2 ;
		
		stemp = line.substring(3, 7);
			offset = Integer.parseInt(stemp, 16) /2;
		
		stemp = line.substring(7, 9);
			type = Integer.parseInt(stemp, 16);
		
		
		iarr = new int[numberOfData];
		
		
		if(type == 0)
		{
			//System.out.print("data ");
			for(int i=0; i<numberOfData; i++)
			{
				stemp = line.substring(9+4*i, 11+4*i);
					itempM = Integer.parseInt(stemp, 16);
				stemp = line.substring(11+4*i, 13+4*i);
					itempL = Integer.parseInt(stemp, 16);
					
				iarr[i] = itempM+ itempL*256;  
				
			}
			
		}else if(type==1)
		{
			// System.out.println("end of file");
		}if(type==2)
		{
			// extended segment base address
		}else if(type==4)
		{
			//System.out.println("extended linear address record");
		}
		
		for(int j=0; j<iarr.length; j++)
		{
			position = offset + j;
			//System.out.print(position + "=" + iarr[j] + " ");
			this.flashProgramMemory.write(position, iarr[j]);

		}
		//System.out.println("");
		
		
	}

	/**
	 * This method gets a HEX file and uploads it to the flash program memory
	 * @param inFileName
	 * @throws IOException
	 */
	public void readHEX(String inFileName) throws IOException
	{
		// TODO Auto-generated method stub

		BufferedReader inputStream = null;

        
            inputStream = new BufferedReader(new FileReader(inFileName));

            String l;
            
            while ((l = inputStream.readLine()) != null) { 
            	
            	this.writeLine(l);	//outputStream.println(l);
            	
            }
             
            if (inputStream != null) {
                inputStream.close();
            }
              		
	}
	/**
	 * This method decodes an instruction code and returns an Instruction
	 * object which may be used for simulation.
	 * @param iCode instruction code
	 * @return instruction object
	 */
	public Instruction decodeInstr(int iCode)
	{
		Instruction instr=null;
		int f=0;
		int d=0;
		int b=0;
		int k=0;
		
		if((iCode & Integer.parseInt("11111100000000",2)) == Integer.parseInt("00011100000000", 2))
		{
			//System.out.println("addwf");	
			f = iCode & Integer.parseInt("00000001111111",2);
			d = (iCode & Integer.parseInt("00000010000000",2))>>7;
			instr = new I_addwf(this, f, d);
			return instr;
			
			
		}else if((iCode & Integer.parseInt("11111100000000",2)) == Integer.parseInt("00010100000000", 2))
		{
			//System.out.println("andwf");
			f = iCode & Integer.parseInt("00000001111111",2);
			d = (iCode & Integer.parseInt("00000010000000",2))>>7;
			instr = new I_andwf(this, f, d);
			return instr;
			
		}else if((iCode & Integer.parseInt("11111110000000",2)) == Integer.parseInt("00000110000000", 2))
		{
			//System.out.println("clrf");
			f = iCode & Integer.parseInt("00000001111111",2);
			instr = new I_clrf(this, f);
			return instr;
			
		}else if((iCode & Integer.parseInt("11111110000000",2)) == Integer.parseInt("00000100000000", 2))
		{
			//System.out.println("clrw");
			instr = new I_clrw(this);
			return instr;
			
		}else if((iCode & Integer.parseInt("11111100000000",2)) == Integer.parseInt("00100100000000", 2))
		{
			//System.out.println("comf");
			f = iCode & Integer.parseInt("00000001111111",2);
			d = (iCode & Integer.parseInt("00000010000000",2))>>7;
			instr = new I_comf(this, f, d);
			return instr;
			
		}else if((iCode & Integer.parseInt("11111100000000",2)) == Integer.parseInt("00001100000000", 2))
		{
			//System.out.println("decf");
			f = iCode & Integer.parseInt("00000001111111",2);
			d = (iCode & Integer.parseInt("00000010000000",2))>>7;
			instr = new I_decf(this, f, d);
			return instr;
			
		}else if((iCode & Integer.parseInt("11111100000000",2)) == Integer.parseInt("00101100000000", 2))
		{
			//System.out.println("decfsz");
			f = iCode & Integer.parseInt("00000001111111",2);
			d = (iCode & Integer.parseInt("00000010000000",2))>>7;
			instr = new I_decfsz(this, f, d);
			return instr;
			
		}else if((iCode & Integer.parseInt("11111100000000",2)) == Integer.parseInt("00101000000000", 2))
		{
			//System.out.println("incf");
			f = iCode & Integer.parseInt("00000001111111",2);
			d = (iCode & Integer.parseInt("00000010000000",2))>>7;
			instr = new I_incf(this, f, d);
			return instr;
			
		}else if((iCode & Integer.parseInt("11111100000000",2)) == Integer.parseInt("00111100000000", 2))
		{
			//System.out.println("incfsz");
			f = iCode & Integer.parseInt("00000001111111",2);
			d = (iCode & Integer.parseInt("00000010000000",2))>>7;
			instr = new I_incfsz(this, f, d);
			return instr;
			
		}else if((iCode & Integer.parseInt("11111100000000",2)) == Integer.parseInt("00010000000000", 2))
		{
			//System.out.println("iorwf");
			f = iCode & Integer.parseInt("00000001111111",2);
			d = (iCode & Integer.parseInt("00000010000000",2))>>7;
			instr = new I_iorwf(this, f, d);
			return instr;
			
		}else if((iCode & Integer.parseInt("11111100000000",2)) == Integer.parseInt("00100000000000", 2))
		{
			//System.out.println("movf");
			f = iCode & Integer.parseInt("00000001111111",2);
			d = (iCode & Integer.parseInt("00000010000000",2))>>7;
			instr = new I_movf(this, f, d);
			return instr;
			
		}else if((iCode & Integer.parseInt("11111110000000",2)) == Integer.parseInt("00000010000000", 2))
		{
			//System.out.println("movwf");
			f = iCode & Integer.parseInt("00000001111111",2);
			instr = new I_movwf(this, f);
			return instr;

		}else if((iCode & Integer.parseInt("11111110011111",2)) == Integer.parseInt("00000000000000", 2))
		{
			//System.out.println("nop");
			instr = new I_nop(this);
			return instr;

		}else if((iCode & Integer.parseInt("11111100000000",2)) == Integer.parseInt("00110100000000", 2))
		{
			//System.out.println("rlf");
			f = iCode & Integer.parseInt("00000001111111",2);
			d = (iCode & Integer.parseInt("00000010000000",2))>>7;
			instr = new I_rlf(this, f, d);
			return instr;
			
		}else if((iCode & Integer.parseInt("11111100000000",2)) == Integer.parseInt("00110000000000", 2))
		{
			//System.out.println("rrf");
			f = iCode & Integer.parseInt("00000001111111",2);
			d = (iCode & Integer.parseInt("00000010000000",2))>>7;
			instr = new I_rrf(this, f, d);
			return instr;
			
		}else if((iCode & Integer.parseInt("11111100000000",2)) == Integer.parseInt("00001000000000", 2))
		{
			//System.out.println("subwf");
			f = iCode & Integer.parseInt("00000001111111",2);
			d = (iCode & Integer.parseInt("00000010000000",2))>>7;
			instr = new I_subwf(this, f, d);
			return instr;
			
		}else if((iCode & Integer.parseInt("11111100000000",2)) == Integer.parseInt("00111000000000", 2))
		{
			//System.out.println("swapf");
			f = iCode & Integer.parseInt("00000001111111",2);
			d = (iCode & Integer.parseInt("00000010000000",2))>>7;
			instr = new I_swapf(this, f, d);
			return instr;
			
		}else if((iCode & Integer.parseInt("11111100000000",2)) == Integer.parseInt("00011000000000", 2))
		{
			//System.out.println("xorwf");
			f = iCode & Integer.parseInt("00000001111111",2);
			d = (iCode & Integer.parseInt("00000010000000",2))>>7;
			instr = new I_xorf(this, f, d);
			return instr;
			
		}else if((iCode & Integer.parseInt("11110000000000",2)) == Integer.parseInt("01000000000000", 2))
		{
			//System.out.println("bcf");
			f = iCode & Integer.parseInt("00000001111111",2);
			b = (iCode & Integer.parseInt("00001110000000",2))>>7;
			instr = new I_bcf(this, f, b);
			return instr;
			
		}else if((iCode & Integer.parseInt("11110000000000",2)) == Integer.parseInt("01010000000000", 2))
		{
			//System.out.println("bsf");
			f = iCode & Integer.parseInt("00000001111111",2);
			b = (iCode & Integer.parseInt("00001110000000",2))>>7;
			instr = new I_bsf(this, f, b);
			return instr;
			
		}else if((iCode & Integer.parseInt("11110000000000",2)) == Integer.parseInt("01100000000000", 2))
		{
			//System.out.println("btfsc");
			f = iCode & Integer.parseInt("00000001111111",2);
			b = (iCode & Integer.parseInt("00001110000000",2))>>7;
			instr = new I_btfsc(this, f, b);
			return instr;
			
		}else if((iCode & Integer.parseInt("11110000000000",2)) == Integer.parseInt("01110000000000", 2))
		{
			//System.out.println("btfss");
			f = iCode & Integer.parseInt("00000001111111",2);
			b = (iCode & Integer.parseInt("00001110000000",2))>>7;
			instr = new I_btfss(this, f, b);
			return instr;
			
		}else if((iCode & Integer.parseInt("11111000000000",2)) == Integer.parseInt("11111000000000", 2))
		{
			//System.out.println("addlw");
			k = iCode & Integer.parseInt("00000011111111",2);
			instr = new I_addlw(this, k);
			return instr;
			
		}else if((iCode & Integer.parseInt("11111100000000",2)) == Integer.parseInt("11100100000000", 2))
		{
			//System.out.println("andlw");
			k = iCode & Integer.parseInt("00000011111111",2);
			instr = new I_andlw(this, k);
			return instr;
			
		}else if((iCode & Integer.parseInt("11100000000000",2)) == Integer.parseInt("10000000000000", 2))
		{
			//System.out.println("call");
			k = iCode & Integer.parseInt("00011111111111",2);
			instr = new I_call(this, k);
			return instr;
			
		}else if((iCode & Integer.parseInt("11111111111111",2)) == Integer.parseInt("00000001100100", 2))
		{
			//System.out.println("clrwdt");
			instr = new I_clrwdt(this);
			return instr;
			
		}else if((iCode & Integer.parseInt("11100000000000",2)) == Integer.parseInt("10100000000000", 2))
		{
			//System.out.println("goto");
			k = iCode & Integer.parseInt("00011111111111",2);
			instr = new I_goto(this, k);
			return instr;
			
		}else if((iCode & Integer.parseInt("11111100000000",2)) == Integer.parseInt("11100000000000", 2))
		{
			//System.out.println("iorlw");
			k = iCode & Integer.parseInt("00000011111111",2);
			instr = new I_iorlw(this, k);
			return instr;
			
		}else if((iCode & Integer.parseInt("11110000000000",2)) == Integer.parseInt("11000000000000", 2))
		{
			//System.out.println("movlw");
			k = iCode & Integer.parseInt("00000011111111",2);
			instr = new I_movlw(this, k);
			return instr;
			
		}else if((iCode & Integer.parseInt("11111111111111",2)) == Integer.parseInt("00000000001001", 2))
		{
			//System.out.println("retfie");
			instr = new I_retfie(this);
			return instr;
			
		}else if((iCode & Integer.parseInt("11110000000000",2)) == Integer.parseInt("11010000000000", 2))
		{
			//System.out.println("retlw");
			k = iCode & Integer.parseInt("00000011111111",2);
			instr = new I_retlw(this, k);
			return instr;
			
		}else if((iCode & Integer.parseInt("11111111111111",2)) == Integer.parseInt("00000000001000", 2))
		{
			//System.out.println("return");
			instr = new I_return(this);
			return instr;

		}else if((iCode & Integer.parseInt("11111111111111",2)) == Integer.parseInt("00000001100011", 2))
		{
			//System.out.println("sleep");
			instr = new I_sleep(this);
			return instr;

		}else if((iCode & Integer.parseInt("11111000000000",2)) == Integer.parseInt("11110000000000", 2))
		{
			//System.out.println("sublw");
			k = iCode & Integer.parseInt("00000011111111",2);
			instr = new I_iorlw(this, k);
			return instr;
			
		}else if((iCode & Integer.parseInt("11111100000000",2)) == Integer.parseInt("11101000000000", 2))
		{
			//System.out.println("xorlw");
			k = iCode & Integer.parseInt("00000011111111",2);
			instr = new I_xorlw(this, k);
			return instr;
			
		}else
		{
			System.out.println("no such instruction!");
			return null;
		}

	}

	/**
	 * Main procedure for testing. Creates a stand alone simulator.
	 * @param args
	 * @throws IOException
	 */
	public static void main(String[] args) throws IOException {
	
		//SimulatorFrame simulatorFrame= new SimulatorFrame("ad_tester",new ConsolePane());
		
		//simulatorFrame.setVisible(true);
		
	}
	
	/**
	 * This method loads the hex file specified by the filename and creates a thread
	 * to simulate PIC cycles.
	 * 
	 */
	public void simulatePIC() throws IOException
	{
		
		//PIC mypic = new PIC(new SimulatorFrame());
		this.dataMemory.reset();
		
		this.readHEX(this.fileName + ".hex");//"prototype.hex");
		//System.out.println(fileName);
		this.task1 = new SimThread(this);
		
		this.threadExecutor = Executors.newFixedThreadPool(1);
		
		this.threadExecutor.execute(this.task1);
		
	
	}
	/**
	 * This method closes the simulator thread.
	 *
	 */
	public void finishSimulation()
	{
		this.task1.kill = true;
		while(task1.sleeping==true);
		this.threadExecutor.shutdown();
	}
	/////////GETTER SETTER METHODS///////////////////////
	/**
	 * @return the dataMemory
	 */
	public DataMemory getDataMemory() {
		return this.dataMemory;
	}
	/**
	 * @param dataMemory the dataMemory to set
	 */
	public void setDataMemory(DataMemory dataMemory) {
		this.dataMemory = dataMemory;
	}
	/**
	 * @return the flashProgramMemory
	 */
	public FlashProgramMemory getFlashProgramMemory() {
		return this.flashProgramMemory;
	}
	/**
	 * @param flashProgramMemory the flashProgramMemory to set
	 */
	public void setFlashProgramMemory(FlashProgramMemory flashProgramMemory) {
		this.flashProgramMemory = flashProgramMemory;
	}
	
	public SimulatorFrame getSimulatorFrame()
	{
		return this.simulatorFrame;
	}
	/**
	 * @return the programCounter
	 */
	public ProgramCounter getProgramCounter() {
		return programCounter;
	}
	/**
	 * @param programCounter the programCounter to set
	 */
	public void setProgramCounter(ProgramCounter programCounter) {
		this.programCounter = programCounter;
	}
	/**
	 * 
	 * @return the adConverter
	 */
	public ADConverter getADConverter() {
		return adConverter;
	}
	/**
	 * This method is used to pause the simulation.
	 *
	 */
	public void sleep() {
		this.threadPaused = true;
	}
}
