/**
 * 
 */
package picmicrocontroller;

/**
 * 
 * This class reresents a 8-bit register of a PIC 16F877
 *
 */
public class Register {

	/////////////////ATTRIBUTES//////////////////////////
	/***/
	private byte data;
	/***/
	private byte prevData;
		
	////////////////CONSTRUCTORS/////////////////////////
	/**
	 * Main constructor of register class. Sets values to 0.
	 */
	public Register() {
		this.data = 0;
		this.prevData = 0;
	}

	/**
	 * Constructor which initializes the fields to the given values.
	 * @param data 
	 * @param prevData
	 */
	public Register(byte data, byte prevData) {
		this.data = data;
		this.prevData = prevData;
	}
	/////////////////METHODS/////////////////////////////
	/**
	 * This method returns the content of a register.
	 */
	public byte read()
	{
		return this.data;
	}
	/**
	 * This method sets the data of the register. It also keeps
	 * the old value.
	 */
	public void write(byte data)
	{
		this.prevData = this.data;
		this.data = data;
	}
	/**
	 * Returns the bit given the index number 0-7
	 * @param index
	 * @return
	 */
	public int getBit(int index)
	{
		return (( this.data >> index ) & (byte)1);
	}
	/**
	 * Sets the bit given the index number 0-7
	 * @param index
	 */
	public void setBit(int index)
	{
		this.prevData = this.data;
		this.data |= (byte)1 << index;
	}
	/**
	 * Clears the bit given the index number 0-7
	 * @param index
	 */
	public void clearBit(int index)
	{
		this.prevData = this.data;
		this.data &= ~((byte)1 << index);
	}
	/**
	 * Updates the bit with a given value and index number 0-7
	 * @param index
	 * @param bit
	 */
	public void updateBit(int index, byte bit)
	{
		this.prevData = this.data;
		if(bit==1)
		{
			this.setBit(index);
		}else
		{
			this.clearBit(index);
		}
	}
	/**
	 * Clears the register. Keeps the old value.
	 */
	public void clear()
	{
		this.prevData = this.data;
		this.data = 0;
	}
	/**
	 * Checks if the content of the register has been changed.
	 */
	public boolean hasChanged()
	{
		return this.data != this.prevData;
	}
	
}
