/**
 * 
 */
package gui;



import java.awt.BorderLayout;
import java.awt.ComponentOrientation;
import java.awt.Dimension;
import java.io.File;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JToolBar;

import projectManager.PIDEFile;
import projectManager.Project;
import projectManager.ProjectFile;
import projectManager.SystemSettingsFile;
import simulator.SimulatorFrame;



/**
 * This class is the main frame of the PIDE. It contains all other modules 
 * and interface elements. The main frame is divided by split panes.
 *
 */
@SuppressWarnings("serial")
public class PFrame extends JFrame {

	
	private JPanel jPanel = null;
	private JSplitPane rootSplitPane = null;
	private JSplitPane topRootSplitPane = null;
	private PToolBar pToolBar = null;
	private PMenuBar pMenuBar = null;
	public ConsolePane consolePane = null;
	public WorkspacePane workspacePane = null;
	private EditorPane editorPane = null;
	private JSplitPane leftTopSplitPane = null;	
	
	
	private JPanel bottomPanel = null;
	private JToolBar statusBar = null;
	public JLabel statusBarLabel = null;
	
	public GuiController guiController = new GuiController();
	
	
	public SystemSettingsFile systemFile = null;
	public Project activeProject = null;
	public ProjectFile activeProjectFile  = null;
	
	/**
	 * The main constructor of the main frame. It does nothing but call the initialize method
	 * which creates and initializes the objects contained in the class.
	 */
	public PFrame() {
		// TODO Auto-generated constructor stub
		this.initialize();
	}
		

	/**
	 * This method initializes the GUI of the program
	 * 
	 */
	private void initialize() {
		
		systemFile = new SystemSettingsFile(PIDEFile.SYSTEM_SETTINGS,"pide",".");
		if(systemFile.existsActiveProject)
		{
			int start = systemFile.activeProject.lastIndexOf(File.separatorChar)+1;
			int end = systemFile.activeProject.length();
			String projectName = systemFile.activeProject.substring(start, end);
			//System.out.println(projectName);
			activeProject = new Project(projectName,systemFile.activeProject);
			activeProjectFile = activeProject.projectFile;
		}
		
		this.editorPane = new EditorPane(this.guiController, this);
		
        this.setSize(new Dimension(946, 715));
        this.setContentPane(this.getJPanel());
        this.setTitle("Pide");
        
        
        this.pToolBar = new PToolBar(this.editorPane, this.guiController, this);
        
        this.getStatusBar();
  
        this.pToolBar.setAutoscrolls(true);
        this.pMenuBar = new PMenuBar(this.pToolBar, this.getStatusBar(), this.consolePane, 
        		this.guiController, this, this.editorPane, this.workspacePane);
        
        this.add(this.pToolBar,BorderLayout.PAGE_START);
        this.add(this.getStatusBar(),BorderLayout.PAGE_END);
        this.setJMenuBar(this.pMenuBar);	
        this.addWindowListener(new java.awt.event.WindowAdapter() {
        	@Override
		public void windowClosing(java.awt.event.WindowEvent e) {
        		//System.out.println("windowClosing()"); 
        		systemFile.save();
        		if(systemFile.existsActiveProject)
        		{
        			activeProjectFile.save();
        		}
        		if(editorPane.list.size() > 0)
        		{
        			int response = -1, shown = 0;
        			for(int i = 0; i<editorPane.list.size();i++)
        			{
 
        				if(editorPane.list.get(i).changeFlag == 1)
        				{
        					if(shown == 0)
        					{
        						response = JOptionPane.showConfirmDialog(PFrame.this, "There are files that are not saved.\n"
            							+"Would you like to save all before exit?",
            							"Unsaved Files", JOptionPane.YES_NO_OPTION);
        						shown = 1;
        					}
        					if(response == JOptionPane.YES_OPTION)
        					{
        						//save all unsaved files
        						guiController.saveMenuPressed(PFrame.this, editorPane);
        						
        					}
        					else 
        					{
        						//simply close but warn the user
        						JOptionPane.showMessageDialog(PFrame.this, "All changes are discarded", 
        								"Discard Changes", JOptionPane.INFORMATION_MESSAGE);
        						break;
        					}
        					
        				}
        			}
        		}
        		if(pToolBar.simulatorFrame != null)
        		{
        			if(pToolBar.simulatorFrame.pic.threadExecutor != null)
        			{
        				
        				pToolBar.simulatorFrame.pic.task1.kill = true;
    					while(pToolBar.simulatorFrame.pic.task1.sleeping==true);
    					pToolBar.simulatorFrame.pic.threadExecutor.shutdown();
        			
        			}
        			
        			pToolBar.simulatorFrame.dispose();
        			pToolBar.simulatorFrame.memoryFrame.dispose();
        			
        		}
        		
        	}
        });
	}


	/**
	 * This method initializes jPanel. Inserts the root split panel
	 * into the main frame.	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanel() {
		if (this.jPanel == null) {
			this.jPanel = new JPanel();
			this.jPanel.setLayout(new BorderLayout());
			this.jPanel.add(this.getRootSplitPane(), BorderLayout.CENTER);
		}
		return this.jPanel;
	}


	/**
	 * This method initializes rootSplitPane. Top component of
	 * root panel contains another split pane. Bottom component
	 * contains the console panel	
	 * 	
	 * @return javax.swing.JSplitPane	
	 */
	public JSplitPane getRootSplitPane() {
		if (this.rootSplitPane == null) {
			this.rootSplitPane = new JSplitPane();
			this.rootSplitPane.setOneTouchExpandable(true);
			this.rootSplitPane.setComponentOrientation(ComponentOrientation.LEFT_TO_RIGHT);
			this.rootSplitPane.setDividerLocation(500);
			this.rootSplitPane.setDividerSize(3);
			this.rootSplitPane.setName("rootSplitPane");
			this.rootSplitPane.setOrientation(JSplitPane.VERTICAL_SPLIT);
			this.rootSplitPane.setContinuousLayout(true);
			this.rootSplitPane.setBottomComponent(this.getbottomPanel());	
			
			this.rootSplitPane.setTopComponent(this.getTopRootSplitPane());			
		}
		return this.rootSplitPane;
	}


	/**
	 * This method initializes topRootSplitPane. Left component of
	 * top panel contains another split pane. Right component
	 * contains nothing for the time being but was intended to contain
	 * debugger panel.	
	 * 	
	 * @return javax.swing.JSplitPane	
	 */
	public JSplitPane getTopRootSplitPane() {
		if (this.topRootSplitPane == null) {
			this.topRootSplitPane = new JSplitPane();
			this.topRootSplitPane.setDividerLocation(800);
			this.topRootSplitPane.setDividerSize(3);
			
			this.topRootSplitPane.setLeftComponent(this.getLeftTopSplitPane());
			this.topRootSplitPane.setRightComponent(null);
			
		}
		return this.topRootSplitPane;
	}


	/**
	 * This method initializes leftTopSplitPane. Left component of
	 * left top panel contains workspace panel. Right component
	 * contains editor panel.	
	 * 	
	 * @return javax.swing.JSplitPane	
	 */
	public JSplitPane getLeftTopSplitPane() {
		if (this.leftTopSplitPane == null) {
			this.leftTopSplitPane = new JSplitPane();
			this.leftTopSplitPane.setContinuousLayout(true);
			this.leftTopSplitPane.setDividerSize(3);
			this.leftTopSplitPane.setName("leftTopSplitPane");
			this.leftTopSplitPane.setDividerLocation(150);
			int index =0;
			String[] workspaceList = null;
			if(systemFile.existsActiveProject)
			{
				//System.out.println(systemFile.activeWorkspaces.size()+1);
				index = systemFile.activeWorkspaces.size()+1;
				String dummy = new String();
				int start = 0;
				int end =0;
				workspaceList = new String[index];
				dummy =systemFile.activeProject;
				start = dummy.lastIndexOf(File.separatorChar)+1;
				end = dummy.length();
				dummy = dummy.substring(start,end);
				workspaceList[0] = new String(dummy);
				for (int i = 1; i < index; i++) {
					dummy =systemFile.activeWorkspaces.get(i-1);
					start = dummy.lastIndexOf(File.separatorChar)+1;
					end = dummy.length();
					dummy = dummy.substring(start,end);
					workspaceList[i] = new String(dummy);
				}
			}
			
			this.workspacePane = new WorkspacePane(workspaceList,index,activeProjectFile, this, editorPane, activeProject);
			this.leftTopSplitPane.setLeftComponent(this.workspacePane);
			this.leftTopSplitPane.setRightComponent(this.editorPane);
		}
		return this.leftTopSplitPane;
	}


	/**
	 * This method initializes bottomPanel. Bottom panel 
	 * contains console panel.	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getbottomPanel() {
		
		if (this.bottomPanel == null) {
			this.bottomPanel = new JPanel();
			this.bottomPanel.setLayout(new BorderLayout());
			
			this.consolePane = new ConsolePane();
			this.bottomPanel.add(this.consolePane, BorderLayout.CENTER);
			this.bottomPanel.setPreferredSize(new Dimension(0,0));
			this.consolePane.setVisible(true);
		}
		
		
		return this.bottomPanel;
	}


	/**
	 * This method initializes statusBar	
	 * 	
	 * @return javax.swing.JToolBar	
	 */
	public JToolBar getStatusBar() {
		if (this.statusBar == null) {
			this.statusBarLabel = new JLabel();
			this.statusBarLabel.setText(" ");
			this.statusBar = new JToolBar();
			this.statusBar.add(this.statusBarLabel);
			this.statusBar.setFloatable(false);
		}
		return this.statusBar;
	}

	/**
	 * This method returns the PMenuBar object associated to the main frame
	 * @return PMenuBar
	 */
	public PMenuBar getPMenuBar() {
		return this.pMenuBar;
	}

	public PToolBar getPToolBar() {
		return this.pToolBar;
	}

}  //  @jve:decl-index=0:visual-constraint="10,-130"
