/**
 * 
 */
package gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;

import javax.sound.sampled.Line;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.text.BadLocationException;
import javax.swing.text.Caret;
import javax.swing.text.DefaultCaret;
import javax.swing.text.DefaultEditorKit;
import javax.swing.text.DefaultHighlighter;
import javax.swing.text.Element;
import javax.swing.text.Highlighter;
import javax.swing.text.JTextComponent;
import javax.swing.tree.DefaultMutableTreeNode;

import simulator.SimulatorFrame;

/**
 * This class is the menu bar object of PIDE. It contains all menu items
 * and binds them to related action listeners.
 * @author OZGUR
 *
 */
@SuppressWarnings("serial")
public class PMenuBar extends JMenuBar {

	private JMenu fileMenu = null;
	private JMenu editMenu = null;
	private JMenu viewMenu = null;
	private JMenu projectMenu = null;
	private JMenu simulateMenu = null;
	private JMenu debugMenu = null;
	private JMenu programmerMenu = null;
	private JMenu analysisMenu = null;
	private JMenu toolsMenu = null;
	private JMenu phelpMenu = null;
	private JMenu newMenu = null;
	private JMenuItem projectMenuItem = null; 
	private JMenuItem asmFileMenuItem = null;  
	private JMenuItem openMenuItem = null;
	private JMenuItem saveMenuItem = null;
	private JMenuItem saveAsMenuItem = null;
	private JMenuItem closeMenuItem = null;
	private JMenu recentFilesMenu = null;
	private JMenu recentProjectsjMenu = null;
	private JMenuItem exitMenuItem = null;
	private JMenuItem undoMenuItem = null;
	private JMenuItem redoMenuItem = null;
	private JMenuItem cutMenuItem = null;
	private JMenuItem copyMenuItem = null;
	private JMenuItem pasteMenuItem = null;
	private JMenuItem selectAllMenuItem = null;
	private JMenuItem findReplaceMenuItem = null;
	private JCheckBoxMenuItem statusBarCheckBoxMenuItem = null;
	private JCheckBoxMenuItem consoleCheckBoxMenuItem = null;
	private JCheckBoxMenuItem debugCheckBoxMenuItem = null;
	private JCheckBoxMenuItem workspaceCheckBoxMenuItem = null;
	private JMenuItem openProjectMenuItem = null;
	private JMenuItem saveProjectMenuItem = null;
	private JMenuItem closeProjectMenuItem = null;
	private JMenuItem buildProjectMenuItem = null;
	private JMenuItem buildOptionsMenuItem = null;
	private JMenuItem addFileToProjectMenu = null;
	private JMenuItem removeFileFromProjectMenu = null;
	private JMenuItem propertiesMenuItem = null;
	private JMenu breakpointMenu = null;
	private JMenu watchpointMenu = null;
	private JMenuItem startMenuItem = null;
	private JMenuItem addBreakpointMenuItem = null;  
	private JMenuItem removeBreakpointMenuItem = null;  
	private JMenuItem addWatchpointMenuItem = null;  
	private JMenuItem removeWatchpointMenuItem = null;  
	private JMenuItem stepMenuItem = null;
	private JMenuItem continueMenuItem = null;
	
	private JMenuItem stepIntoMenuItem = null;
	private JMenuItem stepOutMenuItem = null;
	private JMenuItem stepOverMenuItem = null;
	private JMenuItem stopMenuItem = null;
	private JMenuItem settingsMenuItem = null;
	private JMenuItem writeMenuItem = null;
	private JMenuItem readMenuItem = null;
	private JMenuItem verifyMenuItem = null;
	private JMenuItem eraseMenuItem = null;
	private JMenuItem settingsProgrammerMenuItem = null;
	private JMenuItem addAnalysisMenuItem = null;
	private JMenuItem displayAnalysisMenuItem = null;
	private JMenuItem hideAnalysisMenuItem = null;
	private JMenuItem customizeToolsMenuItem = null;
	private JMenuItem optionsToolsMenuItem = null;
	private JMenuItem aboutHelpMenuItem = null;
	private JMenuItem contentsHelpMenuItem = null;
	private JMenuItem indexHelpMenuItem = null;
	private JMenuItem searchHelpMenuItem = null;
	private JMenu toolbarViewMenu = null;
	private JCheckBoxMenuItem fileToolbarCheckBoxMenuItem = null;
	private JCheckBoxMenuItem editToolbarCheckBoxMenuItem = null;
	private JCheckBoxMenuItem simulateToolbarCheckBoxMenuItem = null;
	private JCheckBoxMenuItem buildToolbarCheckBoxMenuItem = null;
	private JCheckBoxMenuItem debugToolbarCheckBoxMenuItem = null;
	private JMenuItem startSimulateMenuItem = null;
	private JMenuItem pauseSimulateMenuItem = null;
	private JMenuItem runSimulateMenuItem = null;
	private JMenuItem continueSimulateMenuItem = null;
	private JMenuItem stopSimulateMenuItem = null;
	private JMenuItem settingsSimulateMenuItem = null;
	private PToolBar toolBar;
	private JToolBar statusBar;
	public boolean ignore = true;
	private ConsolePane consolePane;
	public GuiController guiController;
	public PFrame callerFrame;
	public EditorPane editorPane;
	public WorkspacePane workspacePane;
	public int konsolHeight;
	public int workspaceWidth;
	/**
	 * The constructor of the menu bar class. 
	 * 
	*/
	public PMenuBar(PToolBar tBar, JToolBar sBar, ConsolePane cPane, GuiController gController, 
			PFrame cFrame, EditorPane ePane, WorkspacePane wPane) {
		// TODO Auto-generated constructor stub
		initialize();
		statusBar = sBar; 
		toolBar = tBar;
		consolePane = cPane;
		guiController = gController;
		callerFrame = cFrame;
		editorPane = ePane;
		workspacePane = wPane;
	}
		

	/**
	 * This method initializes the menu bar properties.
	 * 
	*/
	private void initialize() {
        this.setSize(new Dimension(607, 32));
        this.add(getFileMenu());
        this.add(getEditMenu());
        this.add(getViewMenu());
        this.add(getProjectMenu());
        this.add(getSimulateMenu());
        this.add(getDebugMenu());
        //this.add(getProgrammerMenu());
        //this.add(getAnalysisMenu());
        //this.add(getToolsMenu());
        this.add(getPhelpMenu());
        
	}


	/**
	 * This method initializes and returns the file menu.	
	 * 	
	 * @return javax.swing.JMenu	
	*/
	private JMenu getFileMenu() {
		if (fileMenu == null) {
			fileMenu = new JMenu();
			fileMenu.setText("File");
			fileMenu.setName("fileMenu");
			fileMenu.setMnemonic(KeyEvent.VK_F);
			fileMenu.setRolloverEnabled(true);
			fileMenu.add(getNewMenu());
			fileMenu.add(getOpenMenuItem());
			fileMenu.add(getCloseMenuItem());
			
			fileMenu.addSeparator();
			fileMenu.add(getSaveMenuItem());
			fileMenu.add(getSaveAsMenuItem());
			
			//fileMenu.addSeparator();
			//fileMenu.add(getRecentFilesMenu());
			//fileMenu.add(getRecentProjectsjMenu());
			getRecentFilesMenu().setEnabled(false);
			getRecentProjectsjMenu().setEnabled(false);
			
			fileMenu.addSeparator();
			fileMenu.add(getExitMenuItem());
			
		}
		return fileMenu;
	}


	/**
	 * This method initializes and returns the edit menu.	
	 * 	
	 * @return javax.swing.JMenu	
	*/
	public JMenu getEditMenu() {
		if (editMenu == null) {
			editMenu = new JMenu();
			editMenu.setText("Edit");
			editMenu.setName("editMenu");
			editMenu.setMnemonic(KeyEvent.VK_E);
			editMenu.add(getUndoMenuItem());
			editMenu.add(getRedoMenuItem());
			editMenu.addSeparator();
			editMenu.add(getCutMenuItem());
			editMenu.add(getCopyMenuItem());
			editMenu.add(getPasteMenuItem());
			editMenu.addSeparator();
			editMenu.add(getSelectAllMenuItem());
			editMenu.addSeparator();
			editMenu.add(getFindReplaceMenuItem());
		}
		return editMenu;
	}



	/**
	 * This method initializes and returns the view menu.	
	 * 	
	 * @return javax.swing.JMenu	
	*/
	private JMenu getViewMenu() {
		if (viewMenu == null) {
			viewMenu = new JMenu();
			viewMenu.setText("View");
			viewMenu.add(getToolbarViewMenu());
			viewMenu.add(getStatusBarCheckBoxMenuItem());
			viewMenu.add(getConsoleCheckBoxMenuItem());
			viewMenu.add(getWorkspaceCheckBoxMenuItem());			
		}
		return viewMenu;
	}


	/**
	 * This method initializes and returns the project menu.	
	 * 	
	 * @return javax.swing.JMenu	
	*/
	private JMenu getProjectMenu() {
		if (projectMenu == null) {
			projectMenu = new JMenu();
			projectMenu.setText("Project");
			//projectMenu.add(getOpenProjectMenuItem());
			//projectMenu.add(getSaveProjectMenuItem());
			//projectMenu.add(getCloseProjectMenuItem());
			//projectMenu.addSeparator();
			projectMenu.add(getBuildProjectMenuItem());
			//projectMenu.add(getBuildOptionsMenuItem());
			projectMenu.addSeparator();
			projectMenu.add(getAddFileToProjectMenu());
			projectMenu.add(getRemoveFileFromProjectMenu());
			//projectMenu.addSeparator();
			//projectMenu.add(getPropertiesMenuItem());
		}
		return projectMenu;
	}



	/**
	 * This method initializes and returns the simulate menu.	
	 * 	
	 * @return javax.swing.JMenu	
	*/
	private JMenu getSimulateMenu() {
		if (simulateMenu == null) {
			simulateMenu = new JMenu();
			simulateMenu.setText("Simulate");
			simulateMenu.add(getrunSimulateMenuItem());
			simulateMenu.addSeparator();
			simulateMenu.add(getStartSimulateMenuItem());
			simulateMenu.add(getPauseSimulateMenuItem());
			simulateMenu.add(getContinueSimulateMenuItem());
			simulateMenu.add(getStopSimulateMenuItem());
			
			
			//simulateMenu.addSeparator();
			//simulateMenu.add(getSettingsSimulateMenuItem());
		}
		return simulateMenu;
	}


	/**
	 * This method initializes and returns the debug menu.	
	 * 	
	 * @return javax.swing.JMenu	
	*/
	private JMenu getDebugMenu() {
		if (debugMenu == null) {
			debugMenu = new JMenu();
			debugMenu.setText("Debug");
			debugMenu.add(getBreakpointMenu());
			//debugMenu.add(getWatchpointMenu());
			
			debugMenu.addSeparator();
			debugMenu.add(getStartMenuItem());
			debugMenu.add(getStepMenuItem());
			debugMenu.add(getContinueMenuItem());
			//debugMenu.add(getStepIntoMenuItem());
			//debugMenu.add(getStepOutMenuItem());
			//debugMenu.add(getStepOverMenuItem());
			debugMenu.add(getStopMenuItem());
			
			//debugMenu.addSeparator();
			//debugMenu.add(getSettingsMenuItem());
		}
		return debugMenu;
	}


	/**
	 * This method initializes and returns the programmer menu.	
	 * Not used currently
	 * 	
	 * @return javax.swing.JMenu	
	*/
	@SuppressWarnings("unused")
	private JMenu getProgrammerMenu() {
		if (programmerMenu == null) {
			programmerMenu = new JMenu();
			programmerMenu.setText("Programmer");
			programmerMenu.add(getWriteMenuItem());
			programmerMenu.add(getReadMenuItem());
			programmerMenu.add(getVerifyMenuItem());
			programmerMenu.add(getEraseMenuItem());
			programmerMenu.addSeparator();
			programmerMenu.add(getSettingsProgrammerMenuItem());
		}
		return programmerMenu;
	}


	/**
	 * This method initializes and returns the analysis menu.	
	 * Not used currently	
	 * @return javax.swing.JMenu	
	*/
	private JMenu getAnalysisMenu() {
		if (analysisMenu == null) {
			analysisMenu = new JMenu();
			analysisMenu.setText("Analysis");
			analysisMenu.add(getAddAnalysisMenuItem());
			analysisMenu.add(getDisplayAnalysisMenuItem());
			analysisMenu.add(getHideAnalysisMenuItem());
		}
		return analysisMenu;
	}



	/**
	 * This method initializes and returns the tools menu.	
	 * 	
	 * @return javax.swing.JMenu	
	*/
	private JMenu getToolsMenu() {
		if (toolsMenu == null) {
			toolsMenu = new JMenu();
			toolsMenu.setText("Tools");
			toolsMenu.add(getCustomizeToolsMenuItem());
			toolsMenu.add(getOptionsToolsMenuItem());
		}
		return toolsMenu;
	}


	/**
	 * This method initializes and returns the help menu.	
	 * 	
	 * @return javax.swing.JMenu	
	*/
	private JMenu getPhelpMenu() {
		if (phelpMenu == null) {
			phelpMenu = new JMenu();
			phelpMenu.setText("Help");
			phelpMenu.add(getAboutHelpMenuItem());
		}
		return phelpMenu;
	}


	/**
	 * This method initializes and returns the new menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenu getNewMenu() {
		if (newMenu == null) {
			newMenu = new JMenu();
			newMenu.setText("New");
			newMenu.setMnemonic(KeyEvent.VK_N);
			newMenu.add(getProjectMenuItem());
			newMenu.addSeparator();
			newMenu.add(getFileMenuItem());
		}
		return newMenu;
	}


	/**
	 * This method initializes and returns the project menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getProjectMenuItem() {
		if (projectMenuItem == null) {
			projectMenuItem = new JMenuItem();
			projectMenuItem.setText("Project");
			projectMenuItem.setMnemonic(KeyEvent.VK_P);
			projectMenuItem.addActionListener(new ActionListener(){

				public void actionPerformed(ActionEvent e) {
					// TODO Auto-generated method stub
					NewProjectFrame newProjectFrame = new NewProjectFrame(callerFrame,workspacePane );//!! casting 
					newProjectFrame.setVisible(true);
				}});
		}
		return projectMenuItem;
	}

	/**
	 * This method initializes and returns the file menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getFileMenuItem() {
		if (asmFileMenuItem == null) {
			asmFileMenuItem = new JMenuItem();
			asmFileMenuItem.setText("File");
			asmFileMenuItem.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    		
						NewFileFrame newFileFrame = new NewFileFrame(callerFrame, editorPane);
						newFileFrame.setVisible(true);
						
				}
			});
		}
		return asmFileMenuItem;
	}

	/**
	 * This method initializes and returns the open menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getOpenMenuItem() {
		if (openMenuItem == null) {
			openMenuItem = new JMenuItem();
			openMenuItem.setText("Open...");
			openMenuItem.setMnemonic(KeyEvent.VK_O);
			openMenuItem.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    		
						guiController.openMenuPressed(callerFrame, editorPane, null);
						
						workspacePane.projectRoot.removeAllChildren();
						
						DefaultMutableTreeNode activeworkspaceNode = new DefaultMutableTreeNode(callerFrame.activeProject.getProjectName());
						String inactiveProjectPath = null;
						int start = 0;
						int end =0;
						
						for (int i = 0; i < callerFrame.activeProjectFile.fileList.size(); i++) {
							inactiveProjectPath = callerFrame.activeProjectFile.fileList.get(i);
							
							start = inactiveProjectPath.lastIndexOf(File.separatorChar)+1;
							end = inactiveProjectPath.length();
							inactiveProjectPath = inactiveProjectPath.substring(start,end);
							
							activeworkspaceNode.add(new DefaultMutableTreeNode(inactiveProjectPath));
						}
						
						workspacePane.projectRoot.add(activeworkspaceNode);
						
						for(int i = 0; i < callerFrame.systemFile.activeWorkspaces.size();i++)
						{
							inactiveProjectPath = (String)callerFrame.systemFile.activeWorkspaces.get(i);
							
							start = inactiveProjectPath.lastIndexOf(File.separatorChar)+1;
							end = inactiveProjectPath.length();
							inactiveProjectPath = inactiveProjectPath.substring(start,end);
							
							callerFrame.workspacePane.projectRoot.add(new DefaultMutableTreeNode(inactiveProjectPath));
							
						}
						
						workspacePane.model.nodeStructureChanged((DefaultMutableTreeNode) workspacePane.model.getRoot());
						
				}
			});
		}
		return openMenuItem;
	}


	/**
	 * This method initializes and returns the save menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	public JMenuItem getSaveMenuItem() {
		if (saveMenuItem == null) {
			saveMenuItem = new JMenuItem();
			saveMenuItem.setText("Save");
			saveMenuItem.setEnabled(false);
			saveMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
			saveMenuItem.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    		
						guiController.saveMenuPressed(callerFrame, editorPane);
						
				}
			});
		}
		return saveMenuItem;
	}


	/**
	 * This method initializes and returns the save as menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	public JMenuItem getSaveAsMenuItem() {
		if (saveAsMenuItem == null) {
			saveAsMenuItem = new JMenuItem();
			saveAsMenuItem.setText("Save As...");
			saveAsMenuItem.setEnabled(false);
			saveAsMenuItem.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    		
						guiController.saveAsMenuPressed(callerFrame, editorPane);
						
				}
			});
		}
		return saveAsMenuItem;
	}


	/**
	 * This method initializes and returns the close menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getCloseMenuItem() {
		if (closeMenuItem == null) {
			closeMenuItem = new JMenuItem();
			closeMenuItem.setText("Close");
		}
		return closeMenuItem;
	}


	/**
	 * This method initializes and returns the recent files menu.	
	 * 	
	 * @return javax.swing.JMenu	
	*/
	private JMenu getRecentFilesMenu() {
		if (recentFilesMenu == null) {
			recentFilesMenu = new JMenu();
			recentFilesMenu.setText("Recent Files");
		}
		return recentFilesMenu;
	}

	/**
	 * This method initializes and returns the recent projects menu.	
	 * 	
	 * @return javax.swing.JMenu	
	*/
	private JMenu getRecentProjectsjMenu() {
		if (recentProjectsjMenu == null) {
			recentProjectsjMenu = new JMenu();
			recentProjectsjMenu.setText("Recent Projects");
		}
		return recentProjectsjMenu;
	}


	/**
	 * This method initializes and returns the exit menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getExitMenuItem() {
		if (exitMenuItem == null) {
			exitMenuItem = new JMenuItem();
			exitMenuItem.setText("Exit");
					
			exitMenuItem.addMouseListener(new java.awt.event.MouseListener() {
				public void mouseClicked(java.awt.event.MouseEvent e) {
					
				}
				public void mousePressed(java.awt.event.MouseEvent e) {
					System.exit(0);
				}
				public void mouseReleased(java.awt.event.MouseEvent e) {
				}
				public void mouseEntered(java.awt.event.MouseEvent e) {
				}
				public void mouseExited(java.awt.event.MouseEvent e) {
				}
				
			});
		}
		return exitMenuItem;
	}


	/**
	 * This method initializes and returns the undo menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	public JMenuItem getUndoMenuItem() {
		if (undoMenuItem == null) {
			undoMenuItem = new JMenuItem();
			undoMenuItem.setText("Undo");
			undoMenuItem.setEnabled(false);
		}
		return undoMenuItem;
	}

	/**
	 * This method initializes redoMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	public JMenuItem getRedoMenuItem() {
		if (redoMenuItem == null) {
			redoMenuItem = new JMenuItem();
			redoMenuItem.setText("Redo");
			redoMenuItem.setEnabled(false);
		}
		return redoMenuItem;
	}


	/**
	 * This method initializes and returns the cut menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	public JMenuItem getCutMenuItem() {
		if (cutMenuItem == null) {
			cutMenuItem = new JMenuItem(new DefaultEditorKit.CutAction());
			cutMenuItem.setText("Cut");
			cutMenuItem.setEnabled(false);
		}
		return cutMenuItem;
	}


	/**
	 * This method initializes and returns the copy menu item	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	public JMenuItem getCopyMenuItem() {
		if (copyMenuItem == null) {
			copyMenuItem = new JMenuItem(new DefaultEditorKit.CopyAction());
			copyMenuItem.setText("Copy");
			copyMenuItem.setEnabled(false);
		}
		return copyMenuItem;
	}


	/**
	 * This method initializes and returns the paste menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	public JMenuItem getPasteMenuItem() {
		if (pasteMenuItem == null) {
			pasteMenuItem = new JMenuItem(new DefaultEditorKit.PasteAction());
			pasteMenuItem.setText("Paste");
			pasteMenuItem.setEnabled(false);
		}
		return pasteMenuItem;
	}


	/**
	 * This method initializes and returns the select all menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	public JMenuItem getSelectAllMenuItem() {
		if (selectAllMenuItem == null) {
			selectAllMenuItem = new JMenuItem();
			selectAllMenuItem.setText("Select All");
			selectAllMenuItem.setEnabled(false);
		}
		return selectAllMenuItem;
	}


	/**
	 * This method initializes and returns the replace menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	public JMenuItem getFindReplaceMenuItem() {
		if (findReplaceMenuItem == null) {
			findReplaceMenuItem = new JMenuItem();
			findReplaceMenuItem.setText("Find/Replace");
			findReplaceMenuItem.setEnabled(false);
			findReplaceMenuItem.addActionListener(new ActionListener(){
			
				public void actionPerformed(ActionEvent arg0) {
					// TODO Auto-generated method stub
					
					int index = editorPane.activeFileIndex;
					System.out.println("index: "+index);
					JTextPane text = null;
					if(index > -1)
					{
						text = editorPane.list.get(index).getEditFieldEditorPane();						
						
						Caret selected = new DefaultCaret()
						{
							 public void focusLost(FocusEvent e)
							 {
							 	setVisible(false);
							 }
						};
						selected.setBlinkRate( text.getCaret().getBlinkRate() );
						text.setCaret( selected );

						try
						{
							text.read( new FileReader( new File( "FindReplace.java" ) ), "" );
						}
						catch(IOException e)
						{
							System.out.println( e );
						}
						text.getDocument().putProperty( DefaultEditorKit.EndOfLineStringProperty, "\n" );

						//  Create the top-level container and add contents to it.

						PFrame frame = new PFrame();
						frame.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
						frame.getContentPane().add( new JScrollPane( text ) );
						frame.setSize(400, 400);
						frame.setVisible( true );

						//  display

						FindReplace.getSharedInstance( frame ).showReplace( text );
					}
				}});
		}
		return findReplaceMenuItem;
	}

	/**
	 * This method initializes and returns the status bar check box menu item.	
	 * 	
	 * @return javax.swing.JCheckBoxMenuItem	
	*/
	private JCheckBoxMenuItem getStatusBarCheckBoxMenuItem() {
		if (statusBarCheckBoxMenuItem == null) {
			statusBarCheckBoxMenuItem = new JCheckBoxMenuItem("Status Bar", true);
			statusBarCheckBoxMenuItem.addItemListener(
				    new ItemListener() {
				        public void itemStateChanged(ItemEvent e) {
				            // Set "ignore" whenever box is checked or unchecked.
				            ignore = (e.getStateChange() == ItemEvent.SELECTED);				        
				            if(!ignore)
				            {
				            	statusBar.setVisible(false);
				            }
				            else
				            {
				            	statusBar.setVisible(true);
				            }
				        }
				    }
				);
		}
		return statusBarCheckBoxMenuItem;
	}


	/**
	 * This method initializes and returns the console check box menu item.	
	 * 	
	 * @return javax.swing.JCheckBoxMenuItem	
	*/
	private JCheckBoxMenuItem getConsoleCheckBoxMenuItem() {
		if (consoleCheckBoxMenuItem == null) {
			consoleCheckBoxMenuItem = new JCheckBoxMenuItem("Console", true);
			consoleCheckBoxMenuItem.addItemListener(
				    new ItemListener() {
				        public void itemStateChanged(ItemEvent e) {
				            // Set "ignore" whenever box is checked or unchecked.
				            ignore = (e.getStateChange() == ItemEvent.SELECTED);				        
				            if(!ignore)
				            {
				            	konsolHeight = callerFrame.getRootSplitPane().getDividerLocation();
				            	callerFrame.getRootSplitPane().setDividerLocation(callerFrame.getRootSplitPane().getHeight());
				            }
				            else
				            {
				            	callerFrame.getRootSplitPane().setDividerLocation(konsolHeight);
				            }
				        }
				    }
				);
		}
		return consoleCheckBoxMenuItem;
	}


	/**
	 * This method initializes and returns the debug check box menu item.	
	 * Not used currently.	
	 * @return javax.swing.JCheckBoxMenuItem	
	*/
	@SuppressWarnings("unused")
	private JCheckBoxMenuItem getDebugCheckBoxMenuItem() {
		if (debugCheckBoxMenuItem == null) {
			debugCheckBoxMenuItem = new JCheckBoxMenuItem();
			debugCheckBoxMenuItem.setText("Debug");
		}
		return debugCheckBoxMenuItem;
	}


	/**
	 * This method initializes and returns the workspace check box menu item.	
	 * 	
	 * @return javax.swing.JCheckBoxMenuItem	
	*/
	private JCheckBoxMenuItem getWorkspaceCheckBoxMenuItem() {
		
		if (workspaceCheckBoxMenuItem == null) {
			workspaceCheckBoxMenuItem = new JCheckBoxMenuItem("Workspace", true);
			workspaceCheckBoxMenuItem.addItemListener(
				    new ItemListener() {
				        public void itemStateChanged(ItemEvent e) {
				            // Set "ignore" whenever box is checked or unchecked.
				            ignore = (e.getStateChange() == ItemEvent.SELECTED);				        
				            if(!ignore)
				            {
				            	workspaceWidth = callerFrame.getLeftTopSplitPane().getDividerLocation();
				            	callerFrame.getLeftTopSplitPane().setDividerLocation(0);
				            }
				            else
				            {
				            	callerFrame.getLeftTopSplitPane().setDividerLocation(workspaceWidth);
				            }
				        }
				    }
				);
		}
		return workspaceCheckBoxMenuItem;
	}


	/**
	 * This method initializes and returns the open project menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getOpenProjectMenuItem() {
		if (openProjectMenuItem == null) {
			openProjectMenuItem = new JMenuItem();
			openProjectMenuItem.setText("Open Project");
		}
		return openProjectMenuItem;
	}


	/**
	 * This method initializes and returns the save project menu item.
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getSaveProjectMenuItem() {
		if (saveProjectMenuItem == null) {
			saveProjectMenuItem = new JMenuItem();
			saveProjectMenuItem.setText("Save Project");
		}
		return saveProjectMenuItem;
	}


	/**
	 * This method initializes and returns the close project menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getCloseProjectMenuItem() {
		if (closeProjectMenuItem == null) {
			closeProjectMenuItem = new JMenuItem();
			closeProjectMenuItem.setText("Close Project");
		}
		return closeProjectMenuItem;
	}


	/**
	 * This method initializes and returns the build project menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	public JMenuItem getBuildProjectMenuItem() {
		if (buildProjectMenuItem == null) {
			buildProjectMenuItem = new JMenuItem();
			buildProjectMenuItem.setText("Build");
			buildProjectMenuItem.addActionListener(new ActionListener()
			{

				public void actionPerformed(ActionEvent e) {
					// TODO Auto-generated method stub
					String s = "", path = "";
					if(!editorPane.list.isEmpty())
					{
						if(editorPane.list.get(editorPane.getSelectedIndex()).changeFlag != 0)
						{
							System.out.println("HEBELE");
				    		guiController.saveMenuPressed(callerFrame, editorPane);
						}
						
							s = editorPane.list.get(editorPane.activeFileIndex).getFileName();
							path = editorPane.list.get(editorPane.activeFileIndex).getFilePath();
							int limit =  0;
							for (int i = 0; i < s.length(); i++) {
								if( s.charAt(i) == '.')
									limit = i;
							}
							if(limit != 0)
								s = s.substring(0, limit);


							/* BURADA BREAKPOINT LISTESI ASSEMBLERA GONDERILCEK*/
							ArrayList bpLnum = new ArrayList();
							ArrayList bpAddress = new ArrayList();
							bpLnum.add(21);	bpLnum.add(22);bpLnum.add(23);bpLnum.add(24);

							assembler.asmLexer.assemble(s, path, consolePane, bpLnum, bpAddress);
							for(int i=0;i<bpAddress.size(); i++)
								System.out.println(bpAddress.get(i));
							
							callerFrame.statusBarLabel.setText(" ");
					}
					else callerFrame.statusBarLabel.setText("There is no file to build");
				}
			});
		}
		return buildProjectMenuItem;
	}

	/**
	 * This method initializes and returns the build options menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getBuildOptionsMenuItem() {
		if (buildOptionsMenuItem == null) {
			buildOptionsMenuItem = new JMenuItem();
			buildOptionsMenuItem.setText("Build Options");
		}
		return buildOptionsMenuItem;
	}


	/**
	 * This method initializes and returns the add file to project menu.	
	 * 	
	 * @return javax.swing.JMenu	
	*/
	private JMenuItem getAddFileToProjectMenu() {
		if (addFileToProjectMenu == null) {
			addFileToProjectMenu = new JMenuItem();
			addFileToProjectMenu.setText("Add File(s) to Project");
			addFileToProjectMenu.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    		
						guiController.openMenuPressed(callerFrame, editorPane, null);
						
						workspacePane.projectRoot.removeAllChildren();
						
						DefaultMutableTreeNode activeworkspaceNode = new DefaultMutableTreeNode(callerFrame.activeProject.getProjectName());
						String inactiveProjectPath = null;
						int start = 0;
						int end =0;
						
						for (int i = 0; i < callerFrame.activeProjectFile.fileList.size(); i++) {
							inactiveProjectPath = callerFrame.activeProjectFile.fileList.get(i);
							
							start = inactiveProjectPath.lastIndexOf(File.separatorChar)+1;
							end = inactiveProjectPath.length();
							inactiveProjectPath = inactiveProjectPath.substring(start,end);
							
							activeworkspaceNode.add(new DefaultMutableTreeNode(inactiveProjectPath));
						}
						
						workspacePane.projectRoot.add(activeworkspaceNode);
						
						for(int i = 0; i < callerFrame.systemFile.activeWorkspaces.size();i++)
						{
							inactiveProjectPath = (String)callerFrame.systemFile.activeWorkspaces.get(i);
							
							start = inactiveProjectPath.lastIndexOf(File.separatorChar)+1;
							end = inactiveProjectPath.length();
							inactiveProjectPath = inactiveProjectPath.substring(start,end);
							
							callerFrame.workspacePane.projectRoot.add(new DefaultMutableTreeNode(inactiveProjectPath));
							
						}
						
						workspacePane.model.nodeStructureChanged((DefaultMutableTreeNode) workspacePane.model.getRoot());
						
				}
			});

		}
		return addFileToProjectMenu;
	}


	/**
	 * This method initializes and returns the remove files from project menu.	
	 * 	
	 * @return javax.swing.JMenu	
	*/
	private JMenuItem getRemoveFileFromProjectMenu() {
		if (removeFileFromProjectMenu == null) {
			removeFileFromProjectMenu = new JMenuItem();
			removeFileFromProjectMenu.setText("Remove File(s) from Project");
		}
		return removeFileFromProjectMenu;
	}


	/**
	 * This method initializes and returns the properties menu item	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getPropertiesMenuItem() {
		if (propertiesMenuItem == null) {
			propertiesMenuItem = new JMenuItem();
			propertiesMenuItem.setText("Properties");
		}
		return propertiesMenuItem;
	}


	/**
	 * This method initializes and returns the breakpoint menu.	
	 * 	
	 * @return javax.swing.JMenu	
	*/
	private JMenu getBreakpointMenu() {
		if (breakpointMenu == null) {
			breakpointMenu = new JMenu();
			breakpointMenu.setText("Breakpoint");
			breakpointMenu.add(getAddBreakpointMenuItem());
			breakpointMenu.add(getRemoveBreakpointMenuItem());
		}
		return breakpointMenu;
	}



	/**
	 * This method initializes and returns the add breakpoint menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getAddBreakpointMenuItem() {
		if (addBreakpointMenuItem == null) {
			addBreakpointMenuItem = new JMenuItem();
			addBreakpointMenuItem.setText("Add");
			addBreakpointMenuItem.addActionListener(new ActionListener(){

				public void actionPerformed(ActionEvent e) {
					// TODO Auto-generated method stub
					
					int index = editorPane.activeFileIndex;
					int lineNumber = getLineAtCaret(editorPane.list.get(index).getEditFieldEditorPane());
					
					Highlighter.HighlightPainter cyanPainter;
					cyanPainter = new DefaultHighlighter.DefaultHighlightPainter( Color.cyan );
					try {
						
						Element root = editorPane.list.get(index).getEditFieldEditorPane().getDocument().getDefaultRootElement();
						
						Element line = root.getElement(lineNumber-1);
						Object temp = editorPane.list.get(index).getEditFieldEditorPane().getHighlighter().addHighlight(line.getStartOffset(), line.getEndOffset(), cyanPainter);
						
						editorPane.list.get(index).bpList.add(lineNumber);
						
						editorPane.list.get(index).tempList.add(temp);
						
					} catch (BadLocationException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					}
					//Line l = Line 
				}});
		}
		return addBreakpointMenuItem;
	}


	/**
	 * This method initializes and returns the remove breakpoint menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getRemoveBreakpointMenuItem() {
		if (removeBreakpointMenuItem == null) {
			removeBreakpointMenuItem = new JMenuItem();
			removeBreakpointMenuItem.setText( "Remove");
			removeBreakpointMenuItem.addActionListener(new ActionListener(){

				public void actionPerformed(ActionEvent e) {
					// TODO Auto-generated method stub
					
					System.out.println("remove Pressed:!!");
					int index = editorPane.activeFileIndex;
					int lineNumber = getLineAtCaret(editorPane.list.get(index).getEditFieldEditorPane());
					int bp;
					
					Highlighter.HighlightPainter whitePainter;
					whitePainter = new DefaultHighlighter.DefaultHighlightPainter( Color.white );
					
					Element root = editorPane.list.get(index).getEditFieldEditorPane().getDocument().getDefaultRootElement();
					
					Element line = root.getElement(lineNumber-1);
					
					for (int i = 0; i < editorPane.list.get(index).bpList.size(); i++) {
					
						System.out.println("bps: "+(Integer)editorPane.list.get(index).bpList.get(i));
					}
					
					try {
						
						for (int i = 0; i < editorPane.list.get(index).bpList.size(); i++) {
							
							bp = (Integer)editorPane.list.get(index).bpList.get(i);
							if( lineNumber ==  bp)
							{
								
								System.out.println("trabzon");
								Object temp = editorPane.list.get(index).tempList.get(i);
								editorPane.list.get(index).getEditFieldEditorPane().getHighlighter().removeHighlight(temp);
								editorPane.list.get(index).getEditFieldEditorPane().getHighlighter().addHighlight(line.getStartOffset(), line.getEndOffset(), whitePainter);
								editorPane.list.get(index).bpList.remove(i);
								editorPane.list.get(index).tempList.remove(i);
								break;
							}
						}
						

						
					} catch (BadLocationException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					}
					
				}});
		}
		return removeBreakpointMenuItem;
	}


	/**
	 * This method initializes and returns the watchpoint menu.	
	 * 	
	 * @return javax.swing.JMenu	
	*/
	private JMenu getWatchpointMenu() {
		if (watchpointMenu == null) {
			watchpointMenu = new JMenu();
			watchpointMenu.setText("Watchpoint");
			watchpointMenu.add(getAddWatchpointMenuItem());
			watchpointMenu.add(getRemoveWatchpointMenuItem());
		}
		return watchpointMenu;
	}


	/**
	 * This method initializes and returns the add watchpoint menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getAddWatchpointMenuItem() {
		if (addWatchpointMenuItem == null) {
			addWatchpointMenuItem = new JMenuItem();
			addWatchpointMenuItem.setText("Add");
		}
		return addWatchpointMenuItem;
	}


	/**
	 * This method initializes remove watchpoint menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getRemoveWatchpointMenuItem() {
		if (removeWatchpointMenuItem == null) {
			removeWatchpointMenuItem = new JMenuItem();
			removeWatchpointMenuItem.setText("Remove");
		}
		return removeWatchpointMenuItem;
	}
	
	/**
	 * This method initializes and returns the start menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getStartMenuItem() {
		if (startMenuItem == null) {
			startMenuItem = new JMenuItem();
			startMenuItem.setText("Start");
			
			startMenuItem.addMouseListener(new java.awt.event.MouseListener() {
				public void mousePressed(java.awt.event.MouseEvent e) {
					if(toolBar.simulatorFrame != null)
					{	
						return;
					}
					String s = "", path = "";
					if(!editorPane.list.isEmpty())
					{
						if(editorPane.list.get(editorPane.getSelectedIndex()).changeFlag != 0)
						{
							System.out.println("HEBELE");
				    		guiController.saveMenuPressed(callerFrame, editorPane);
						}
						
							s = editorPane.list.get(editorPane.activeFileIndex).getFileName();
							path = editorPane.list.get(editorPane.activeFileIndex).getFilePath();
							System.out.println("filename" + s);
							System.out.println("filepath" + path);
	/*							
							int limit =  0;
							for (int i = 0; i < s.length(); i++) {
								if( s.charAt(i) == '.')
									limit = i;
							}
							if(limit != 0)
								s = s.substring(0, limit);
	*/							
							String path2=new String();
							int limit =  0;
							for (int i = path.length()-1; i>=0; i--) {
								if( path.charAt(i) == '.')
								{
									limit = i;
									break;
								}
								
							}
							if(limit != 0)
								path2 = path.substring(0, limit);
							else
								System.out.println("invalid filename - length=0");
							
							System.out.println("PATH2 = " + path2);
							/* BURADA BREAKPOINT LISTESI ASSEMBLERA GONDERILCEK*/
								
								int index = editorPane.activeFileIndex;
								ArrayList bpLnum = editorPane.list.get(index).bpList;
								
								ArrayList bpAddress = new ArrayList();
								//bpLnum.add(21);	bpLnum.add(22);bpLnum.add(23);bpLnum.add(24);

								int result = assembler.asmLexer.assemble(s, path, consolePane, bpLnum, bpAddress);
								
								for (int i = 0; i < bpAddress.size(); i++) {
									System.out.println("ADRESS " + bpAddress.get(i));
									
								}
								
								if(result == 1)
								{
									toolBar.simulatorFrame= new SimulatorFrame(path2,consolePane, bpAddress);
									//SimulatorFrame simulatorFrame= new SimulatorFrame(s,consolePane);
									toolBar.simulatorFrame.setpFrame(callerFrame);
									toolBar.simulatorFrame.setVisible(true);									
									
									toolBar.simulatorFrame.setTitle("Simulation - PIDE - " + toolBar.simulatorFrame.getPic().fileName + ".HEX");	
								}
								
							callerFrame.statusBarLabel.setText(" ");
					}
					else callerFrame.statusBarLabel.setText("There is no file to simulate");
					
				}
				public void mouseClicked(java.awt.event.MouseEvent e) {
				}
				public void mouseReleased(java.awt.event.MouseEvent e) {
				}
				public void mouseEntered(java.awt.event.MouseEvent e) {
				}
				public void mouseExited(java.awt.event.MouseEvent e) {
				}
			});
			
			
		}
		return startMenuItem;
	}


	/**
	 * This method initializes and returns the step menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getStepMenuItem() {
		if (stepMenuItem == null) {
			stepMenuItem = new JMenuItem();
			stepMenuItem.setText("Step");
			
			stepMenuItem.addActionListener(new ActionListener(){

				public void actionPerformed(ActionEvent e) {
					// TODO Auto-generated method stub
					if(toolBar.simulatorFrame != null)
					{
						toolBar.simulatorFrame.pic.task1.step = true;
					}
				}});

			
		}
		return stepMenuItem;
	}
	
	/**
	 * This method initializes and returns the step menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getContinueMenuItem() {
		if (continueMenuItem == null) {
			continueMenuItem = new JMenuItem();
			continueMenuItem.setText("Continue");
		}
		
		continueMenuItem.addActionListener(new ActionListener(){

			public void actionPerformed(ActionEvent e) {
				// TODO Auto-generated method stub
				if(toolBar.simulatorFrame != null)
				{
					toolBar.simulatorFrame.pic.task1.cont = true;
				}
			}});

		return continueMenuItem;
	}
	

	/**
	 * This method initializes and returns the step into menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getStepIntoMenuItem() {
		if (stepIntoMenuItem == null) {
			stepIntoMenuItem = new JMenuItem();
			stepIntoMenuItem.setText("Step Into");
		}
		return stepIntoMenuItem;
	}


	/**
	 * This method initializes and returns the step out menu item.
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getStepOutMenuItem() {
		if (stepOutMenuItem == null) {
			stepOutMenuItem = new JMenuItem();
			stepOutMenuItem.setText("Step Out");
		}
		return stepOutMenuItem;
	}



	/**
	 * This method initializes and returns the step over menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getStepOverMenuItem() {
		if (stepOverMenuItem == null) {
			stepOverMenuItem = new JMenuItem();
			stepOverMenuItem.setText("Step Over");
		}
		return stepOverMenuItem;
	}


	/**
	 * This method initializes and returns the stop menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getStopMenuItem() {
		if (stopMenuItem == null) {
			stopMenuItem = new JMenuItem();
			stopMenuItem.setText("Stop");			
			
			stopMenuItem.addActionListener(new ActionListener(){

				public void actionPerformed(ActionEvent e) {
					// TODO Auto-generated method stub
					if(toolBar.simulatorFrame != null)
					{
						toolBar.simulatorFrame.pic.threadFromPause = false;
						if(toolBar.simulatorFrame.pic.threadExecutor!=null)
						{

							//pic.threadExecutor.shutdown();
							//while(toolBar.simulatorFrame.pic.task1.sleeping) System.out.println("sleeping...");
							
							//toolBar.simulatorFrame.pic.task1.notifyAll();
							toolBar.simulatorFrame.pic.threadExecutor.shutdownNow();
							
													
							//pic.finishSimulation();
							System.out.println("SimThread.stop()");						
							//System.out.println("aligel");

							toolBar.simulatorFrame.pic.threadExecutor = null;
							//pic.threadPaused = false;
						}else
						{

						}

						
					}
				}});
		}
		
		return stopMenuItem;
	}


	/**
	 * This method initializes and returns the settings menu item.
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getSettingsMenuItem() {
		if (settingsMenuItem == null) {
			settingsMenuItem = new JMenuItem();
			settingsMenuItem.setText("Settings");
		}
		return settingsMenuItem;
	}



	/**
	 * This method initializes and returns the write menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getWriteMenuItem() {
		if (writeMenuItem == null) {
			writeMenuItem = new JMenuItem();
			writeMenuItem.setText("Write");
		}
		return writeMenuItem;
	}


	/**
	 * This method initializes and returns the read menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getReadMenuItem() {
		if (readMenuItem == null) {
			readMenuItem = new JMenuItem();
			readMenuItem.setText("Read");
		}
		return readMenuItem;
	}


	/**
	 * This method initializes and returns the verify menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getVerifyMenuItem() {
		if (verifyMenuItem == null) {
			verifyMenuItem = new JMenuItem();
			verifyMenuItem.setText("Verify");
		}
		return verifyMenuItem;
	}

	/**
	 * This method initializes and returns the erase menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getEraseMenuItem() {
		if (eraseMenuItem == null) {
			eraseMenuItem = new JMenuItem();
			eraseMenuItem.setText("Erase");
		}
		return eraseMenuItem;
	}

	/**
	 * This method initializes and returns the settings programmer menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/

	private JMenuItem getSettingsProgrammerMenuItem() {
		if (settingsProgrammerMenuItem == null) {
			settingsProgrammerMenuItem = new JMenuItem();
			settingsProgrammerMenuItem.setText("Settings");
		}
		return settingsProgrammerMenuItem;
	}


	/**
	 * This method initializes and returns the add analysis menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getAddAnalysisMenuItem() {
		if (addAnalysisMenuItem == null) {
			addAnalysisMenuItem = new JMenuItem();
			addAnalysisMenuItem.setText("Add");
		}
		return addAnalysisMenuItem;
	}


	/**
	 * This method initializes and returns the display analysis menu item.
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getDisplayAnalysisMenuItem() {
		if (displayAnalysisMenuItem == null) {
			displayAnalysisMenuItem = new JMenuItem();
			displayAnalysisMenuItem.setText("Display");
		}
		return displayAnalysisMenuItem;
	}


	/**
	 * This method initializes and returns the hide analysis menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getHideAnalysisMenuItem() {
		if (hideAnalysisMenuItem == null) {
			hideAnalysisMenuItem = new JMenuItem();
			hideAnalysisMenuItem.setText("Hide");
		}
		return hideAnalysisMenuItem;
	}



	/**
	 * This method initializes and returns the customize tools menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getCustomizeToolsMenuItem() {
		if (customizeToolsMenuItem == null) {
			customizeToolsMenuItem = new JMenuItem();
			customizeToolsMenuItem.setText("Customize");
		}
		return customizeToolsMenuItem;
	}


	/**
	 * This method initializes and returns the options tools menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getOptionsToolsMenuItem() {
		if (optionsToolsMenuItem == null) {
			optionsToolsMenuItem = new JMenuItem();
			optionsToolsMenuItem.setText("Options");
		}
		return optionsToolsMenuItem;
	}


	/**
	 * This method initializes and returns the about help menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getAboutHelpMenuItem() {
		if (aboutHelpMenuItem == null) {
			aboutHelpMenuItem = new JMenuItem();
			aboutHelpMenuItem.setText("About PIDE");
/////////////////			
			aboutHelpMenuItem.addActionListener(new ActionListener(){

				public void actionPerformed(ActionEvent e) {
					// TODO Auto-generated method stub
					AboutFrame helpFrame = new AboutFrame(callerFrame,editorPane);//!! casting 
					helpFrame.setVisible(true);
				}});
///////////////
			
		}
		return aboutHelpMenuItem;
	}


	/**
	 * This method initializes and returns the contents help menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getContentsHelpMenuItem() {
		if (contentsHelpMenuItem == null) {
			contentsHelpMenuItem = new JMenuItem();
			contentsHelpMenuItem.setText("Contents");
		}
		return contentsHelpMenuItem;
	}


	/**
	 * This method initializes and returns the index help menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getIndexHelpMenuItem() {
		if (indexHelpMenuItem == null) {
			indexHelpMenuItem = new JMenuItem();
			indexHelpMenuItem.setText("Index");
		}
		return indexHelpMenuItem;
	}


	/**
	 * This method initializes and returns the search help menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getSearchHelpMenuItem() {
		if (searchHelpMenuItem == null) {
			searchHelpMenuItem = new JMenuItem();
			searchHelpMenuItem.setText("Search");
		}
		return searchHelpMenuItem;
	}
		
	/**
	 * This method initializes toolbar view menu.	
	 * 	
	 * @return javax.swing.JMenu	
	*/
	private JMenu getToolbarViewMenu() {
		if (toolbarViewMenu == null) {
			toolbarViewMenu = new JMenu();
			toolbarViewMenu.setText("Tools");
			
			fileToolbarCheckBoxMenuItem = new JCheckBoxMenuItem("File",true);
			//fileToolbarCheckBoxMenuItem.setText("File");
			//fileToolbarCheckBoxMenuItem.addac
			fileToolbarCheckBoxMenuItem.addItemListener(
				    new ItemListener() {
				        public void itemStateChanged(ItemEvent e) {
				            // Set "ignore" whenever box is checked or unchecked.
				            ignore = (e.getStateChange() == ItemEvent.SELECTED);				        
				            if(!ignore)
				            {
				            	toolBar.getNewButton().setVisible(false);
				            	toolBar.remove(toolBar.getNewButton());
				            	toolBar.getOpenButton().setVisible(false);
				            	toolBar.remove(toolBar.getOpenButton());
				            	toolBar.getSaveButton().setVisible(false);
				            	toolBar.remove(toolBar.getSaveButton());
				            	toolBar.getSaveAllButton().setVisible(false);
				            	toolBar.remove(toolBar.getSaveAllButton());				            	
				            }
				            else
				            {
				            	toolBar.addSeparator();
				            	toolBar.getSaveAllButton().setVisible(true);
				            	toolBar.add(toolBar.getSaveAllButton(),0);	
				            	toolBar.getSaveButton().setVisible(true);
				            	toolBar.add(toolBar.getSaveButton(),0);
				            	toolBar.getOpenButton().setVisible(true);
				            	toolBar.add(toolBar.getOpenButton(),0);
				            	toolBar.getNewButton().setVisible(true);
				            	toolBar.add(toolBar.getNewButton(),0);         	
				            }
				        }
				    }
				);
			
			toolbarViewMenu.add(fileToolbarCheckBoxMenuItem);
			
			
			
			editToolbarCheckBoxMenuItem = new JCheckBoxMenuItem("Edit", true);
			editToolbarCheckBoxMenuItem.addItemListener(
				    new ItemListener() {
				        public void itemStateChanged(ItemEvent e) {
				            // Set "ignore" whenever box is checked or unchecked.
				            ignore = (e.getStateChange() == ItemEvent.SELECTED);				        
				            if(!ignore)
				            {
				            	toolBar.getCutButton().setVisible(false);
				            	toolBar.remove(toolBar.getCutButton());
				            	toolBar.getCopyButton().setVisible(false);
				            	toolBar.remove(toolBar.getCopyButton());
				            	toolBar.getPasteButton().setVisible(false);
				            	toolBar.remove(toolBar.getPasteButton());	
				            	toolBar.getUndoButton().setVisible(false);
				            	toolBar.remove(toolBar.getUndoButton());
				            	toolBar.getRedoButton().setVisible(false);
				            	toolBar.remove(toolBar.getRedoButton());
				            	toolBar.getFindButton().setVisible(false);
				            	toolBar.remove(toolBar.getFindButton());
				            	toolBar.getReplaceButton().setVisible(false);
				            	toolBar.remove(toolBar.getReplaceButton());		
				            }
				            else
				            {
				            	toolBar.getReplaceButton().setVisible(true);
				            	toolBar.add(toolBar.getReplaceButton(),5);	
				            	toolBar.getFindButton().setVisible(true);
				            	toolBar.add(toolBar.getFindButton(),5);
				            	toolBar.getRedoButton().setVisible(true);
				            	toolBar.add(toolBar.getRedoButton(),5);
				            	toolBar.getUndoButton().setVisible(true);
				            	toolBar.add(toolBar.getUndoButton(),5); 
				            	toolBar.getPasteButton().setVisible(true);
				            	toolBar.add(toolBar.getPasteButton(),5);	
				            	toolBar.getCopyButton().setVisible(true);
				            	toolBar.add(toolBar.getCopyButton(),5);
				            	toolBar.getCutButton().setVisible(true);
				            	toolBar.add(toolBar.getCutButton(),5);
				            	
				            }
				        }
				    }
				);
			
			toolbarViewMenu.add(editToolbarCheckBoxMenuItem);
			
			simulateToolbarCheckBoxMenuItem = new JCheckBoxMenuItem("Simulate", true);
			simulateToolbarCheckBoxMenuItem.addItemListener(
				    new ItemListener() {
				        public void itemStateChanged(ItemEvent e) {
				            // Set "ignore" whenever box is checked or unchecked.
				            ignore = (e.getStateChange() == ItemEvent.SELECTED);				        
				            if(!ignore)
				            {
				            	toolBar.getUndoButton().setVisible(false);
				            	toolBar.remove(toolBar.getUndoButton());
				            	toolBar.getRedoButton().setVisible(false);
				            	toolBar.remove(toolBar.getRedoButton());
				            	toolBar.getFindButton().setVisible(false);
				            	toolBar.remove(toolBar.getFindButton());
				            	toolBar.getReplaceButton().setVisible(false);
				            	toolBar.remove(toolBar.getReplaceButton());				            	
				            }
				            else
				            {
				            	toolBar.getReplaceButton().setVisible(true);
				            	toolBar.add(toolBar.getReplaceButton(),9);	
				            	toolBar.getFindButton().setVisible(true);
				            	toolBar.add(toolBar.getFindButton(),9);
				            	toolBar.getRedoButton().setVisible(true);
				            	toolBar.add(toolBar.getRedoButton(),9);
				            	toolBar.getUndoButton().setVisible(true);
				            	toolBar.add(toolBar.getUndoButton(),9);         	
				            }
				        }
				    }
				);
			
			
			buildToolbarCheckBoxMenuItem = new JCheckBoxMenuItem("Build",false);
			buildToolbarCheckBoxMenuItem.addItemListener(
				    new ItemListener() {
				        public void itemStateChanged(ItemEvent e) {
				            // Set "ignore" whenever box is checked or unchecked.
				            ignore = (e.getStateChange() == ItemEvent.SELECTED);				        
				            if(!ignore)
				            {
				            	toolBar.getWorkspaceToggleButton().setVisible(false);
				            	toolBar.remove(toolBar.getWorkspaceToggleButton());
				            	toolBar.getConsoleToggleButton().setVisible(false);
				            	toolBar.remove(toolBar.getConsoleToggleButton());
				            	toolBar.getRegisterToggleButton().setVisible(false);
				            	toolBar.remove(toolBar.getRegisterToggleButton());
				            	toolBar.getWatchpointToggleButton().setVisible(false);
				            	toolBar.remove(toolBar.getWatchpointToggleButton());				            	
				            }
				            else
				            {
				            	toolBar.getWatchpointToggleButton().setVisible(true);
				            	toolBar.add(toolBar.getWatchpointToggleButton(),14);	
				            	toolBar.getRegisterToggleButton().setVisible(true);
				            	toolBar.add(toolBar.getRegisterToggleButton(),14);
				            	toolBar.getConsoleToggleButton().setVisible(true);
				            	toolBar.add(toolBar.getConsoleToggleButton(),14);
				            	toolBar.getWorkspaceToggleButton().setVisible(true);
				            	toolBar.add(toolBar.getWorkspaceToggleButton(),14);         	
				            }
				        }
				    }
				);
			toolbarViewMenu.add(buildToolbarCheckBoxMenuItem);
			
			debugToolbarCheckBoxMenuItem = new JCheckBoxMenuItem("Debug",true);
			debugToolbarCheckBoxMenuItem.addItemListener(
				    new ItemListener() {
				        public void itemStateChanged(ItemEvent e) {
				            // Set "ignore" whenever box is checked or unchecked.
				            ignore = (e.getStateChange() == ItemEvent.SELECTED);				        
				            if(!ignore)
				            {
				            	toolBar.getBuildButton().setVisible(false);
				            	toolBar.remove(toolBar.getBuildButton());
				            	toolBar.getStartSimulateButton().setVisible(false);
				            	toolBar.remove(toolBar.getStartSimulateButton());
				            	toolBar.getStopSimulateButton().setVisible(false);
				            	toolBar.remove(toolBar.getStopSimulateButton());			            	
				            }
				            else
				            {
				            	toolBar.getStopSimulateButton().setVisible(true);
				            	toolBar.add(toolBar.getStopSimulateButton(),21);	
				            	toolBar.getStartSimulateButton().setVisible(true);
				            	toolBar.add(toolBar.getStartSimulateButton(),21);
				            	toolBar.getBuildButton().setVisible(true);
				            	toolBar.add(toolBar.getBuildButton(),21);
				            	
				            }
				        }
				    }
				);
			toolbarViewMenu.add(debugToolbarCheckBoxMenuItem);
			
			
		}
		
		return toolbarViewMenu;
	}


	/**
	 * This method initializes and returns the start simulate menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getStartSimulateMenuItem() {
		if (startSimulateMenuItem == null) {
			startSimulateMenuItem = new JMenuItem();
			startSimulateMenuItem.setText("Start");
			
			startSimulateMenuItem.addMouseListener(new java.awt.event.MouseListener() {
				public void mousePressed(java.awt.event.MouseEvent e) {
					if(toolBar.simulatorFrame != null)
					{
						if(toolBar.simulatorFrame.pic.threadPaused==true)
						{
	
							toolBar.simulatorFrame.pic.threadPaused=false;
							if(toolBar.simulatorFrame.pic.threadFromPause)
								return;
						}
	
						try {
							if(toolBar.simulatorFrame.pic.threadExecutor==null)
								toolBar.simulatorFrame.pic.simulatePIC();
						} catch (IOException e2) {
							// TODO Auto-generated catch block
							e2.printStackTrace();
						}
					}
				}
				public void mouseClicked(java.awt.event.MouseEvent e) {
				}
				public void mouseReleased(java.awt.event.MouseEvent e) {
				}
				public void mouseEntered(java.awt.event.MouseEvent e) {
				}
				public void mouseExited(java.awt.event.MouseEvent e) {
				}
			});
			
			
		}
		return startSimulateMenuItem;
	}


	/**
	 * This method initializes and returns the pause simulate menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getPauseSimulateMenuItem() {
		if (pauseSimulateMenuItem == null) {
			pauseSimulateMenuItem = new JMenuItem();
			pauseSimulateMenuItem.setText("Pause");
			
			pauseSimulateMenuItem.addMouseListener(new java.awt.event.MouseListener() {
				public void mousePressed(java.awt.event.MouseEvent e) {
					if(toolBar.simulatorFrame == null)
					{
						return;						
					}
					//System.out.println("mousePressed()"); // TODO Auto-generated Event stub mousePressed()
					toolBar.simulatorFrame.pic.threadFromPause = true;
					if(toolBar.simulatorFrame.pic.threadExecutor!=null)
					{
						toolBar.simulatorFrame.pic.threadPaused = true;

					}else
					{
						toolBar.simulatorFrame.pic.threadPaused = false;
					}

				}
				public void mouseClicked(java.awt.event.MouseEvent e) {
				}
				public void mouseReleased(java.awt.event.MouseEvent e) {
				}
				public void mouseEntered(java.awt.event.MouseEvent e) {
				}
				public void mouseExited(java.awt.event.MouseEvent e) {
				}
			});
		}
		return pauseSimulateMenuItem;
	}


	/**
	 * This method initializes and returns the continue simulate menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getContinueSimulateMenuItem() {
		if (continueSimulateMenuItem == null) {
			continueSimulateMenuItem = new JMenuItem();
			continueSimulateMenuItem.setText("Continue");
		}
		return continueSimulateMenuItem;
	}


	/**
	 * This method initializes and returns the stop simulate menu item.
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getStopSimulateMenuItem() {
		if (stopSimulateMenuItem == null) {
			stopSimulateMenuItem = new JMenuItem();
			stopSimulateMenuItem.setText("Stop");
			
			stopSimulateMenuItem.addMouseListener(new java.awt.event.MouseListener() {
				public void mousePressed(java.awt.event.MouseEvent e) {
					if(toolBar.simulatorFrame == null)
					{
						return;						
					}
					//System.out.println("mousePressed()"); // TODO Auto-generated Event stub mousePressed()
					toolBar.simulatorFrame.pic.threadFromPause = false;
					if(toolBar.simulatorFrame.pic.threadExecutor!=null)
					{

						//pic.threadExecutor.shutdown();
						toolBar.simulatorFrame.pic.threadPaused = true;
						toolBar.simulatorFrame.pic.task1.kill = true;
						while(toolBar.simulatorFrame.pic.task1.sleeping==true);
						toolBar.simulatorFrame.pic.threadExecutor.shutdownNow();
						//pic.finishSimulation();
						System.out.println("SimThread.stop()");						
						//System.out.println("aligel");

						toolBar.simulatorFrame.pic.threadExecutor = null;
						//pic.threadPaused = false;
					}else
					{

					}

				}
				public void mouseClicked(java.awt.event.MouseEvent e) {
				}
				public void mouseReleased(java.awt.event.MouseEvent e) {
				}
				public void mouseEntered(java.awt.event.MouseEvent e) {
				}
				public void mouseExited(java.awt.event.MouseEvent e) {
				}
			});
			
		}
		return stopSimulateMenuItem;
	}
	/**
	 * This method initializes and returns the stop simulate menu item.
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getrunSimulateMenuItem() {
		if (runSimulateMenuItem == null) {
			runSimulateMenuItem = new JMenuItem();
			runSimulateMenuItem.setText("Run");
		}
		runSimulateMenuItem.addMouseListener(new java.awt.event.MouseListener() {
			public void mousePressed(java.awt.event.MouseEvent e) {
				if(toolBar.simulatorFrame != null)
				{	
					return;
				}
				String s = "", path = "";
				if(!editorPane.list.isEmpty())
				{
					if(editorPane.list.get(editorPane.getSelectedIndex()).changeFlag != 0)
					{
						System.out.println("HEBELE");
			    		guiController.saveMenuPressed(callerFrame, editorPane);
					}
					
						s = editorPane.list.get(editorPane.activeFileIndex).getFileName();
						path = editorPane.list.get(editorPane.activeFileIndex).getFilePath();
						System.out.println("filename" + s);
						System.out.println("filepath" + path);
/*							
						int limit =  0;
						for (int i = 0; i < s.length(); i++) {
							if( s.charAt(i) == '.')
								limit = i;
						}
						if(limit != 0)
							s = s.substring(0, limit);
*/							
						String path2=new String();
						int limit =  0;
						for (int i = path.length()-1; i>=0; i--) {
							if( path.charAt(i) == '.')
							{
								limit = i;
								break;
							}
							
						}
						if(limit != 0)
							path2 = path.substring(0, limit);
						else
							System.out.println("invalid filename - length=0");
						
						System.out.println("PATH2 = " + path2);
						/* BURADA BREAKPOINT LISTESI ASSEMBLERA GONDERILCEK*/
							
							int index = editorPane.activeFileIndex;
							ArrayList bpLnum = editorPane.list.get(index).bpList;
							
							ArrayList bpAddress = new ArrayList();
							//bpLnum.add(21);	bpLnum.add(22);bpLnum.add(23);bpLnum.add(24);

							int result = assembler.asmLexer.assemble(s, path, consolePane, bpLnum, bpAddress);
							
							for (int i = 0; i < bpAddress.size(); i++) {
								System.out.println("ADRESS " + bpAddress.get(i));
								
							}
							
							if(result == 1)
							{
								toolBar.simulatorFrame= new SimulatorFrame(path2,consolePane, bpAddress);
								//SimulatorFrame simulatorFrame= new SimulatorFrame(s,consolePane);
								toolBar.simulatorFrame.setpFrame(callerFrame);
								toolBar.simulatorFrame.setVisible(true);									
								
								toolBar.simulatorFrame.setTitle("Simulation - PIDE - " + toolBar.simulatorFrame.getPic().fileName + ".HEX");	
							}
							
						callerFrame.statusBarLabel.setText(" ");
				}
				else callerFrame.statusBarLabel.setText("There is no file to simulate");
				
			}
			public void mouseClicked(java.awt.event.MouseEvent e) {
			}
			public void mouseReleased(java.awt.event.MouseEvent e) {
			}
			public void mouseEntered(java.awt.event.MouseEvent e) {
			}
			public void mouseExited(java.awt.event.MouseEvent e) {
			}
		});
		return runSimulateMenuItem;
	}

	
	/**
	 * This method initializes and returns the settings simulate menu item.	
	 * 	
	 * @return javax.swing.JMenuItem	
	*/
	private JMenuItem getSettingsSimulateMenuItem() {
		if (settingsSimulateMenuItem == null) {
			settingsSimulateMenuItem = new JMenuItem();
			settingsSimulateMenuItem.setText("Settings");
		}
		return settingsSimulateMenuItem;
	}

	
	/**
	*  Return the current line number at the Caret position.
	*/
	public static int getLineAtCaret(JTextComponent component)
	{
		int caretPosition = component.getCaretPosition();
		Element root = component.getDocument().getDefaultRootElement();
 
		return root.getElementIndex( caretPosition ) + 1;
	}
	
}  //  @jve:decl-index=0:visual-constraint="35,22"
