/**
 * 
 */
package peripherals;

import picmicrocontroller.Register;

/**
 * This class is created in order to keep the information of
 * a single LED.
 * @author maho
 * @since 22.04.2007
 */
public class LED {

//////////////ATTRIBUTES/////////////////////
	
	/**LED id.This id represents the place of the LED unit in  LEDArray */
	private int ledId;
	/**LED data.Data size is 1 byte and each bi represents on/off information of segments*/
	private Register ledDataReg;
	/**Value representing if the LED unit is activated during simulation.*/
	private boolean isEnabled;
	/**LED data*/
	public boolean bit;
	
	/////////////CONSTRUCTORS////////////////////////
	/**
	 * Constructor with default values.
	 * Id should be specified.
	 * @param ledId instance-unique id
	 * @author maho
	 * @since 19.01.2007
	 */
	public LED(int ledId) {
		this.ledId = ledId;
		this.bit = false;
		this.isEnabled = false;
		
		this.ledDataReg = new Register();
		this.ledDataReg.write((byte) 0);
	}	
	
	/**
	 * Main constructor of the LED class.
	 * @param ledId		instance-unique id
	 * @param ledData	initial data to be loaded 
	 * @param isEnabled status of LED
	 * @author maho
	 * @since 19.01.2007
	 */
	public LED(int ledId, Register ledData, boolean isEnabled) {
		this.ledId = ledId;
		this.ledDataReg.write(ledData.read());
		this.isEnabled = isEnabled;
	}
	
	/////////////////METHODS/////////////////////////////
	/**
 	* This methods alters the corresponding LED element in the virtual card.
 	* Setting enable/disable mode is performed by SevenSegmentDisplayArray object.
 	* @author maho
	* @since 19.01.2007
 	*/

	
	public void update()
	{
		if(!this.isEnabled)
		{
			this.bit=false;
		}
		else
		{
			this.bit= (this.ledDataReg.getBit(ledId)==1)?true:false;
		}
	}
	/////////GETTER SETTER METHODS///////////////////////

	/**
	 * @return the isEnabled
	 * @author maho
	 * @since 19.01.2007
	 */
	public boolean isEnabled() {
		return this.isEnabled;
	}

	/**
	 * @param isEnabled the isEnabled to set
	 * @author maho
	 * @since 19.01.2007
	 */
	public void setEnabled(boolean isEnabled) {
		this.isEnabled = isEnabled;
	}

	/**
	 * A LED doesn't have a register to hold its data. It reads the data from PIC 
	 * ports (PORTD) and gives output only if it is enabled.
	 * @return the ssData
	 * @author maho
	 * @since 19.01.2007
	 */
	public byte readData() {
		if(this.isEnabled)
			return this.ledDataReg.read();
		else return 0;
	}

	/**
	 * A LED doesn't have a register to hold its data. It writes the data from PIC 
	 * ports (PORTD) and gives output only if it is enabled.
	 * @param ledData the ssData to set
	 * @author maho
	 * @since 19.01.2007
	 */
	public void writeData(byte ledData) {
		if(this.isEnabled)
			this.ledDataReg.write(ledData);
		else this.ledDataReg.write( (byte) 0) ;
	}

	/**
	 * @return the ledId
	 * @author maho
	 * @since 19.01.2007
	 */
	public int getLedId() {
		return this.ledId;
	}

	/**
	 * @param ledId the ledId to set
	 * @author maho
	 * @since 19.01.2007
	 */
	public void setLedId(int ledId) {
		this.ledId = ledId;
	}

}
