package simulator;

import java.io.IOException;

import picmicrocontroller.Instruction;
import picmicrocontroller.PIC;
/**
 * This class is the (instruction by instruction) simulation of a PIC
 * 16F877. It implements the run method and is called after the simulation 
 * thread is created.
 */
 public class SimThread implements Runnable{

	public PIC pic;
	public boolean step=false;
	public boolean cont=false;
	public boolean kill=false; 
	public boolean sleeping=true;
	
	/**
	 * Main constructor of the simulation thread class.
	 * @param pic
	 */
	public SimThread(PIC pic)
	{
		this.pic = pic;	
	}
	
	public void run() {
		// TODO Auto-generated method stub
		System.out.println("SimThread.run()");

		int instCode;		
		Instruction itemp=null;
		
		
		//Runtime.getRuntime().exec("calc");
		//
		//Runtime.getRuntime().exec("icprog.exe -lprototype.hex -p -i -t98");
		
		int cycle;
		
		for(cycle = 0; true; cycle++)
		{
			if(kill)
				while(true) sleeping = false;
			
			
			if(this.pic.threadPaused == false)
			{
				
				
				this.pic.getSimulatorFrame().getKeyPad().update();
				this.pic.getSimulatorFrame().getPotentiometer().update();
				
				this.pic.getADConverter().simulate();		
				
				//System.out.println(pic.programCounter.getPC());
				instCode = this.pic.getFlashProgramMemory().read(this.pic.getProgramCounter().getPC());
				
				itemp = this.pic.decodeInstr(instCode);	
				
				if(itemp != null)
				{
					if((step==true) || (this.pic.simulatorFrame.breakPointList.contains(pic.getProgramCounter().getPC())))
						itemp.setDetail(true);
					
					this.pic.getProgramCounter().increment();
					itemp.simulate();
				
					
					
					
					// if "step" clicked from previous instruction					
					if(step==true)
					{
						
						step = false;
						/* wait for the user to press step button again*/
						/* or press continue button */ 
						while((cont==false) && (step==false))
						{	
							
							try {	
								if(kill)
									while(true) sleeping = false;
								else
									Thread.sleep(100);
								
							} catch (InterruptedException e) {
								// TODO Auto-generated catch block
								//e.printStackTrace();
							}
							
						}
						
					}else // previous click is not a "step" input from user 
					{
						
						// check if breakpoint
						if(this.pic.simulatorFrame.breakPointList.contains(pic.getProgramCounter().getPC()))
						{
							// there is a breakpoint
								this.pic.simulatorFrame.consolePane.addText("Breakpoint encountered at PC: " + pic.getProgramCounter().getPC()+ "\n");
								this.pic.simulatorFrame.consolePane.addText("Press \"step\" to continue...\n");
								
								//this.pic.simulatorFrame.consolePane.
								/*
								///////////////////////////////////////////
								// wait user to press a key in the console to continue
									try {
										System.out.println("press a key to continue...");
										System.in.read();
									} catch (IOException e1) {
										// TODO Auto-generated catch block
										e1.printStackTrace();
									}
									///////////////////////////////////////////
								*/	
						
								cont = false;
								
									while((cont==false) && (step==false))
									{	
										
										try {
											if(kill)
												while(true) sleeping = false;
											else
												Thread.sleep(100);					
										} catch (InterruptedException e) {
											// TODO Auto-generated catch block
											e.printStackTrace();
										}
										
									}
								
						}else // not a breakpoint
						{
							
								
						}
					}		
										
					try {
						if(kill)
							while(true) sleeping = false;
						else
							Thread.sleep(35);
				
						//Thread.sleep(1135);
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
	
						e.printStackTrace();
						return;
					}
					
	
				}
				
				this.pic.getSimulatorFrame().getSsdArray().update();
				this.pic.getSimulatorFrame().getLedArray().update();
				this.pic.getSimulatorFrame().repaint();
				
			}else // thread is paused
			{
				//System.out.println("paused");
				try {
					if(kill)
						while(true) sleeping = false;
					else
						Thread.sleep(1);
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			
		}
	}

}
