package simulator;
import gui.ConsolePane;
import gui.PFrame;

import javax.swing.JFrame;
import java.awt.Dimension;
import javax.swing.JPanel;
import javax.swing.JButton;

import peripherals.KeyPad;
import peripherals.LEDArray;
import peripherals.Potentiometer;
import peripherals.PushButton;
import peripherals.SevenSegmentDisplayArray;
import picmicrocontroller.DataMemory;
import picmicrocontroller.PIC;

import java.awt.Rectangle;

import java.awt.Font;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.BorderFactory;
import java.awt.Color;
import javax.swing.JLabel;
import java.awt.Graphics;
import java.io.IOException;
import java.util.ArrayList;

import javax.swing.JTextField;
import java.awt.GridBagLayout;
import javax.swing.JCheckBox;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.JToggleButton;

@SuppressWarnings("serial")
public class SimulatorFrame extends JFrame {

	public DataMemory dataObservable;
	private JPanel jPanel = null;
	ArrayList breakPointList = null;
	

	private JPanel jPanel1 = null;
	
	byte b = 1;

	private SevenSegmentPanel sevenSegmentPanel;
	private SevenSegmentDisplayArray ssdArray;
	
	private LEDPanel ledPanel;
	private LEDArray ledArray;
	/***/
	private KeyPadPanel keyPadPanel;
	private KeyPad keyPad;  //  @jve:decl-index=0:
	
	private PotentiometerPanel potPanel;
	private Potentiometer potentiometer;
	public PIC pic;  //  @jve:decl-index=0:

	
	private JButton startButton = null;
	private JButton pauseButton = null;
	private JButton simulateButton = null;
	
	
	private JPanel jPanel3 = null;

	private JCheckBox stickyCheckBox = null;
	public MemoryFrame memoryFrame;
	private JToggleButton memoryFrameToggle = null;
	public ConsolePane consolePane;
	
	private PFrame pFrame = null;
	
	//private JTextField jTextField = null;
	/**
	 * This method initializes 
	 * 
	 */
	public SimulatorFrame(String filename, ConsolePane cPane, ArrayList bpAddresses) {
		super();
		this.breakPointList = bpAddresses;
		this.consolePane = cPane;
		//System.out.println("SIMULATOR FILENAME "  + filename);
		pic = new PIC(this, filename);
		memoryFrame = new MemoryFrame(this,pic);
		
		memoryFrame.setVisible(true);
		memoryFrame.setTitle(this.getTitle()+" Memory");
		this.initialize();
		
	}
	
	public void setpFrame(PFrame pf)
	{
		this.pFrame = pf;
	}
	
	/**
	 * This method initializes this
	 * 
	 */
	private void initialize() {
			
		this.setSize(new Dimension(544, 508));
		this.setResizable(false);
		this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		this.keyPad = new KeyPad(this.pic);
		this.keyPadPanel = keyPad.getKeyPadPanel();
		
		this.potentiometer = new Potentiometer(this.pic);
		this.potPanel = potentiometer.getPPanel();
		//System.out.println(pic.getDataMemory().output(3,11));
		//this.sevenSegmentPanel = new SevenSegmentPanel();

		
		this.ssdArray = new SevenSegmentDisplayArray(this.pic);
		this.sevenSegmentPanel = this.ssdArray.getSsdPanel();
		
		this.ledArray = new LEDArray(this.pic);
		this.ledPanel = ledArray.getLedPanel();
		this.setContentPane(this.getJPanel());
		this.addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent e) {
				//System.out.println("windowClosing()"); // 
				pFrame.getPToolBar().simulatorFrame = null;
				SimulatorFrame.this.pic.threadFromPause = false;
				if(SimulatorFrame.this.pic.threadExecutor!=null)
				{
						
					//pic.threadExecutor.shutdown();
					
					pic.task1.kill = true;
					while(pic.task1.sleeping==true);
					SimulatorFrame.this.pic.threadExecutor.shutdownNow();
					
					//pic.finishSimulation();
					System.out.println("SimThread.stop()");						
					//System.out.println("aligel");

					SimulatorFrame.this.pic.threadExecutor = null;
					//pic.threadPaused = false;
				}
				SimulatorFrame.this.memoryFrame.dispose();
				
			}
		});

		
	}

	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanel() {
		if (this.jPanel == null) {
			this.jPanel = new JPanel();
			//jPanel.setLayout(new GridBagLayout());
			this.jPanel.setLayout(null);
			this.jPanel.setFont(new Font("Dialog", Font.PLAIN, 14));
			this.jPanel.add(this.getJPanel1(), null);
			
			this.jPanel.add(this.keyPadPanel, null);
			this.jPanel.add(this.potPanel, null);
			this.jPanel.add(this.getStartButton(), null);
			this.jPanel.add(this.getPauseButton(), null);
			this.jPanel.add(this.getStopButton(), null);

			jPanel.add(getJPanel3(), null);
			jPanel.add(getStickyCheckBox(), null);
			jPanel.add(getMemoryFrameToggle(), null);
		}
		return this.jPanel;
	}


	/**
	 * This method initializes jPanel1	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanel1() {
		if (this.jPanel1 == null) {
			this.jPanel1 = new JPanel();
			this.jPanel1.setLayout(null);
			this.jPanel1.setBounds(new Rectangle(63, 17, 336, 140));
			this.jPanel1.setBorder(BorderFactory.createMatteBorder(5, 5, 5, 5, Color.black));
		}
		return this.jPanel1;
	}
	/**
	 * This method initializes startButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getStartButton() {
		if (this.startButton == null) {
			this.startButton = new JButton();
			this.startButton.setBounds(new Rectangle(64, 244, 78, 28));
			this.startButton.setText("Start");
			this.startButton.addMouseListener(new MouseListener(){

				public void mouseClicked(MouseEvent arg0) {
					// TODO Auto-generated method stub

				}

				public void mouseEntered(MouseEvent arg0) {
					// TODO Auto-generated method stub

				}

				public void mouseExited(MouseEvent arg0) {
					// TODO Auto-generated method stub

				}

				public void mousePressed(MouseEvent arg0) {
					// TODO Auto-generated method stub
					//updateTables();
					if(SimulatorFrame.this.pic.threadPaused==true)
					{

						SimulatorFrame.this.pic.threadPaused=false;
						if(SimulatorFrame.this.pic.threadFromPause)
							return;
					}

					try {
						if(SimulatorFrame.this.pic.threadExecutor==null)
							SimulatorFrame.this.pic.simulatePIC();
					} catch (IOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}

				public void mouseReleased(MouseEvent arg0) {
					// TODO Auto-generated method stub

				}});
		}
		return this.startButton;
	}

	/**
	 * This method initializes pauseButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getPauseButton() {
		if (this.pauseButton == null) {
			this.pauseButton = new JButton();
			this.pauseButton.setBounds(new Rectangle(144, 244, 78, 28));
			this.pauseButton.setText("Pause");
			this.pauseButton.addMouseListener(new java.awt.event.MouseListener() {
				public void mousePressed(java.awt.event.MouseEvent e) {
					//System.out.println("mousePressed()"); // TODO Auto-generated Event stub mousePressed()
					SimulatorFrame.this.pic.threadFromPause = true;
					if(SimulatorFrame.this.pic.threadExecutor!=null)
					{
						SimulatorFrame.this.pic.threadPaused = true;

					}else
					{
						SimulatorFrame.this.pic.threadPaused = false;
					}

				}
				public void mouseClicked(java.awt.event.MouseEvent e) {
				}
				public void mouseReleased(java.awt.event.MouseEvent e) {
				}
				public void mouseEntered(java.awt.event.MouseEvent e) {
				}
				public void mouseExited(java.awt.event.MouseEvent e) {
				}
			});

		}
		return this.pauseButton;
	}

	/**
	 * This method initializes simulateButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getStopButton() {
		if (this.simulateButton == null) {
			this.simulateButton = new JButton();
			this.simulateButton.setBounds(new Rectangle(224, 244, 78, 28));
			this.simulateButton.setText("Stop");
			this.simulateButton.addMouseListener(new java.awt.event.MouseListener() {
				public void mousePressed(java.awt.event.MouseEvent e) {
					//System.out.println("mousePressed()"); // TODO Auto-generated Event stub mousePressed()
					SimulatorFrame.this.pic.threadFromPause = false;
					if(SimulatorFrame.this.pic.threadExecutor!=null)
					{

						//pic.threadExecutor.shutdown();
						pic.threadPaused = true;
						pic.task1.kill = true;
						while(pic.task1.sleeping==true);
						SimulatorFrame.this.pic.threadExecutor.shutdownNow();
						//pic.finishSimulation();
						System.out.println("SimThread.stop()");						
						//System.out.println("aligel");

						SimulatorFrame.this.pic.threadExecutor = null;
						//pic.threadPaused = false;
					}else
					{

					}

				}
				public void mouseClicked(java.awt.event.MouseEvent e) {
				}
				public void mouseReleased(java.awt.event.MouseEvent e) {
				}
				public void mouseEntered(java.awt.event.MouseEvent e) {
				}
				public void mouseExited(java.awt.event.MouseEvent e) {
				}
			});
		}
		return this.simulateButton;
	}





	public void paint(Graphics g) {
		super.paint(g);
		this.sevenSegmentPanel.paint(g);
		this.ledPanel.paint(g);
	}

	

	public SevenSegmentDisplayArray getSsdArray() {
		return this.ssdArray;
	}

	public void setSsdArray(SevenSegmentDisplayArray ssdArray) {
		this.ssdArray = ssdArray;
	}


	public KeyPad getKeyPad() {
		return this.keyPad;
	}

	public PIC getPic() {
		return this.pic;
	}

	/**
	 * This method initializes jPanel3	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanel3() {
		if (jPanel3 == null) {
			jPanel3 = new JPanel();
			jPanel3.setLayout(null);
			jPanel3.setBounds(new Rectangle(63, 161, 339, 49));
			jPanel3.setBorder(BorderFactory.createMatteBorder(5, 5, 5, 5, Color.black));
		}
		return jPanel3;
	}

	/**
	 * @return the ledArray
	 */
	public LEDArray getLedArray() {
		return ledArray;
	}

	/**
	 * @param ledArray the ledArray to set
	 */
	public void setLedArray(LEDArray ledArray) {
		this.ledArray = ledArray;
	}

	/**
	 * @return the potentiometer
	 */
	public Potentiometer getPotentiometer() {
		return potentiometer;
	}

	/**
	 * @param potentiometer the potentiometer to set
	 */
	public void setPotentiometer(Potentiometer potentiometer) {
		this.potentiometer = potentiometer;
	}

	/**
	 * This method initializes stickyCheckBox	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */
	private JCheckBox getStickyCheckBox() {
		if (stickyCheckBox == null) {
			stickyCheckBox = new JCheckBox();
			stickyCheckBox.setBounds(new Rectangle(61, 211, 95, 21));
			stickyCheckBox.setText("Sticky lights");
			stickyCheckBox.addChangeListener(new ChangeListener(){

				public void stateChanged(ChangeEvent arg0) {
					// TODO Auto-generated method stub
					ssdArray.sticky = SimulatorFrame.this.stickyCheckBox.isSelected();
					//ledArray.sticky = SimulatorFrame.this.stickyCheckBox.isSelected();
				}}
			);
		}
		return stickyCheckBox;
	}

	/**
	 * This method initializes memoryFrameToggle	
	 * 	
	 * @return javax.swing.JToggleButton	
	 */
	public JToggleButton getMemoryFrameToggle() {
		if (memoryFrameToggle == null) {
			memoryFrameToggle = new JToggleButton();
			memoryFrameToggle.setBounds(new Rectangle(67, 279, 111, 24));
			memoryFrameToggle.setText("View Memory");
			memoryFrameToggle.setSelected(true);
			memoryFrameToggle.addChangeListener(new ChangeListener(){

				public void stateChanged(ChangeEvent e) {
					// TODO Auto-generated method stub
					if(SimulatorFrame.this.memoryFrameToggle.isSelected())
					{
						SimulatorFrame.this.memoryFrame.setVisible(true);
					}
					else
					{
						SimulatorFrame.this.memoryFrame.setVisible(false);
					}
				}});
		}
		return memoryFrameToggle;
	}


}  //  @jve:decl-index=0:visual-constraint="39,0"
 
