// $ANTLR : "den.g" -> "TreeWalker.java"$

	package compiler;
	import java.io.*;
	import antlr.*;
	import antlr.collections.AST;
	import antlr.debug.misc.ASTFrame;

import antlr.TreeParser;
import antlr.Token;
import antlr.collections.AST;
import antlr.RecognitionException;
import antlr.ANTLRException;
import antlr.NoViableAltException;
import antlr.MismatchedTokenException;
import antlr.SemanticException;
import antlr.collections.impl.BitSet;
import antlr.ASTPair;
import antlr.collections.impl.ASTArray;

import java.util.ArrayList;
import java.util.Stack;


public class TreeWalker extends antlr.TreeParser       implements asmppLexerTokenTypes
 {

	
	Stack ifStack = new Stack();
	int passNumber=1;
	int addressNumber=0;
	String HEXString = "";
	int hexword=0;
	SymbolTable sTable = new SymbolTable();

	int ifCount = 0;
	

	
	public SymbolTable getSTable() {
		return sTable;
	}
	
	public void setSTable(SymbolTable sT) {
		sTable = sT;
		sTable.printTable();
	}
	
	public void Initialize(int pNum)
	{
		addressNumber=0;
		HEXString = "";
		hexword=0;
		passNumber = pNum;
	}
	
	public int str2int(String str)
	{
			String tempval="";
			int data=0;

			if((str.length() > 1)&& (str.charAt(1)=='x'))
			{	
				//hex
				tempval = str.substring(2);
				data = Integer.parseInt(tempval, 16);
			}else if((str.charAt(0)=='b') | (str.charAt(0)=='B'))
			{ //binary
				tempval = str.substring(2, str.length()-1);
				/*System.out.println("BINARY:"+tempval);*/
				data = Integer.parseInt(tempval, 2);
				
			}else
			{
				if((str.charAt(0)=='d') | (str.charAt(0)=='D'))
				{ //D'10'
					tempval = str.substring(2, str.length()-1);
					data = Integer.parseInt(tempval, 10);
				
				}else //standard decimal
					data = Integer.parseInt(str);
			}
			
			return data;
	}
	public class Symbol {
		private String name;
		private int value;
		// 1:constant, 2:variable, 3:array
		private int type;
		private int startAddress;
		private int arraySize;
		
		Symbol (String nm, int val, int tp, int strAdd, int arrSz) {
			name = nm;
			value = val;
			type = tp;
			startAddress = strAdd;
			arraySize = arrSz;
		}
			
		public void printSymbol() {
			System.out.println("|\t"+name+"\t\t"+value+"\t\t"+type+"\t\t"+startAddress+"\t\t"+arraySize+"\t|");
			System.out.println("+-------------------------------------------------------------------------------+");
		}
	}
	
	public class SymbolTable {
		private ArrayList sTable;
		
		SymbolTable () {
			sTable = new ArrayList();
			
			// Kucuk harf versiyonlari da eklenecek:
			//BANK0
			add("TMR0", "1", 1, -1, 1);
			add("PCL", "2", 1, -1, 1);
			add("STATUS", "3", 1, -1, 1);
				add("RP1", "6", 1, -1, 1);
				add("RP0", "5", 1, -1, 1);
			add("FSR", "4", 1, -1, 1);
			add("PORTA", "5", 1, -1, 1);
			add("PORTB", "6", 1, -1, 1);
			add("PORTC", "7", 1, -1, 1);
			add("PORTD", "8", 1, -1, 1);
			add("PORTE", "9", 1, -1, 1);
			add("PCLATH", "10", 1, -1, 1);
			add("INTCON", "11", 1, -1, 1);
			add("PIR1", "12", 1, -1, 1);
			add("PIR2", "13", 1, -1, 1);
			add("TMR1L", "14", 1, -1, 1);
			add("TMR1H", "15", 1, -1, 1);
			add("T1CON", "16", 1, -1, 1);
			add("TMR2", "17", 1, -1, 1);
			add("T2CON", "18", 1, -1, 1);
			add("SSPBUF", "19", 1, -1, 1);
			add("SSPCON", "20", 1, -1, 1);
			add("CCPR1L", "21", 1, -1, 1);
			add("CCPR1H", "22", 1, -1, 1);
			add("CCP1CON", "23", 1, -1, 1);
			add("RCSTA", "24", 1, -1, 1);
			add("TXREG", "25", 1, -1, 1);
			add("RCREG", "26", 1, -1, 1);
			add("CCPR2L", "27", 1, -1, 1);
			add("CCPR2H", "28", 1, -1, 1);
			add("CCP2CON", "29", 1, -1, 1);
			add("ADRESH", "30", 1, -1, 1);
			add("ADCON0", "31", 1, -1, 1);
			
			//BANK1
			add("OPTION_REG", "1", 1, -1, 1);
			add("TRISA", "5", 1, -1, 1);
			add("TRISB", "6", 1, -1, 1);
			add("TRISC", "7", 1, -1, 1);
			add("TRISD", "8", 1, -1, 1);
			add("TRISE", "9", 1, -1, 1);
			add("PCLATH", "10", 1, -1, 1);
			add("INTCON", "11", 1, -1, 1);
			add("PIE1", "12", 1, -1, 1);
			add("PIE2", "13", 1, -1, 1);
			add("PCON", "14", 1, -1, 1);
			add("SSPCON2", "17", 1, -1, 1);
			add("PR2", "18", 1, -1, 1);
			add("SSPADD", "19", 1, -1, 1);
			add("SSPSTAT", "20", 1, -1, 1);
			add("TXSTA", "24", 1, -1, 1);
			add("SPBRG", "25", 1, -1, 1);
			add("ADRESL", "30", 1, -1, 1);
			add("ADCON1", "31", 1, -1, 1);

			//BANK2
			add("EEDATA", "12", 1, -1, 1);
			add("EEADR", "13", 1, -1, 1);
			add("EEDATH", "14", 1, -1, 1);
			add("EEADRH", "15", 1, -1, 1);

			//BANK3
			add("EECON1", "12", 1, -1, 1);
			add("EECON2", "13", 1, -1, 1);
		}
	
		
		public void add(String name, String value, int type, int startAddress, int arraySize) {
			if(name.charAt(0)=='*')//variable
				name = name.substring(1);
			String tempval="";
			int data=0;

			data = str2int(value);
			
			Symbol sym = new Symbol(name, data, type, startAddress, arraySize);
			sTable.add(sym);
		}
		
		public int getSize() {
			return sTable.size();
		}
		
		public Symbol getSymbol(int i) {
			return (Symbol)sTable.get(i);
		}

		public int getSymbolValue(String s_name) {
			if(s_name.charAt(0)=='*')//variable
				s_name = s_name.substring(1);
			int size = sTable.size();
			Symbol entry = null;
			for (int i = 0; i < size; i++) {
				entry = (Symbol)sTable.get(i);
				if(s_name.compareTo(entry.name) == 0) {
				  return entry.value;
				}
			}
			//Print an error here, there is no such variable 
			System.err.println("No such variable: " + s_name);
			return -1;
			
		} 
		public int getSymbolType(String s_name) {
			int size = sTable.size();
			Symbol entry = null;
			for (int i = 0; i < size; i++) {
				entry = (Symbol)sTable.get(i);
				if(s_name.compareTo(entry.name) == 0) {
				  return entry.type;
				}
			}
			//Print an error here, there is no such variable 
			return -1;
			
		} 
	
		public boolean isDefined (String s_name) {
			int size = sTable.size();
			for (int i = 0; i < size; i++) {
				Symbol entry = (Symbol)sTable.get(i);
				if((s_name.compareTo(entry.name) == 0)) 
				{
				   	return true;
				}
			}
			return false;
		}
				

		public void printTable() {
			System.out.println("+--------------------------------- SYMBOL TABLE  -------------------------------+");
			System.out.println("|\tNAME"+"\t\t"+"VALUE"+"\t\t"+"TYPE"+"\t\t"+"START"+"\t\t"+"SIZE\t|");
			System.out.println("+-------------------------------------------------------------------------------+");
			
			for (int i = 0; i < sTable.size(); i++) {
				getSymbol(i).printSymbol();
			}
		}
	}
public TreeWalker() {
	tokenNames = _tokenNames;
}

	public final String  program(AST _t) throws RecognitionException {
		String s="";
		
		AST program_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST program_AST = null;
		
			s = new String();
			String sub="";
		
		
		try {      // for error handling
			AST __t1864 = _t;
			AST tmp1_AST = null;
			AST tmp1_AST_in = null;
			tmp1_AST = astFactory.create((AST)_t);
			tmp1_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp1_AST);
			ASTPair __currentAST1864 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,PROGRAM);
			_t = _t.getFirstChild();
			sub=subprogramBody(_t);
			_t = _retTree;
			astFactory.addASTChild(currentAST, returnAST);
			currentAST = __currentAST1864;
			_t = __t1864;
			_t = _t.getNextSibling();
			if ( inputState.guessing==0 ) {
				
					//s += "PROGRAM STARTS:\n";
					s+=sub;
				
			}
			program_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = program_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  subprogramBody(AST _t) throws RecognitionException {
		String s="";
		
		AST subprogramBody_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST subprogramBody_AST = null;
		
			String dec=null;
			String stat=null;	
			
		
		
		try {      // for error handling
			AST __t1866 = _t;
			AST tmp2_AST = null;
			AST tmp2_AST_in = null;
			tmp2_AST = astFactory.create((AST)_t);
			tmp2_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp2_AST);
			ASTPair __currentAST1866 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,SUBPROGRAMBODY);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				/*s += " subprogrambody\n";*/
			}
			{
			{
			_loop1872:
			do {
				if (_t==null) _t=ASTNULL;
				if ((_t.getType()==BASICDECL||_t.getType()==FUNCTIONDECLARATION)) {
					{
					if (_t==null) _t=ASTNULL;
					switch ( _t.getType()) {
					case BASICDECL:
					{
						{
						dec=basicDecl(_t);
						_t = _retTree;
						astFactory.addASTChild(currentAST, returnAST);
						}
						break;
					}
					case FUNCTIONDECLARATION:
					{
						{
						dec=funcDeclaration(_t);
						_t = _retTree;
						astFactory.addASTChild(currentAST, returnAST);
						}
						break;
					}
					default:
					{
						throw new NoViableAltException(_t);
					}
					}
					}
					if ( inputState.guessing==0 ) {
						s += dec + "\n";
					}
				}
				else {
					break _loop1872;
				}
				
			} while (true);
			}
			{
			_loop1874:
			do {
				if (_t==null) _t=ASTNULL;
				if ((_t.getType()==STATEMENT)) {
					stat=statement(_t);
					_t = _retTree;
					astFactory.addASTChild(currentAST, returnAST);
					if ( inputState.guessing==0 ) {
						s += stat;
					}
				}
				else {
					break _loop1874;
				}
				
			} while (true);
			}
			}
			currentAST = __currentAST1866;
			_t = __t1866;
			_t = _t.getNextSibling();
			if ( inputState.guessing==0 ) {
				
						
				
			}
			subprogramBody_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = subprogramBody_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  basicDecl(AST _t) throws RecognitionException {
		String s="";
		
		AST basicDecl_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST basicDecl_AST = null;
		
			String dec=null;
		
		
		try {      // for error handling
			AST __t1876 = _t;
			AST tmp3_AST = null;
			AST tmp3_AST_in = null;
			tmp3_AST = astFactory.create((AST)_t);
			tmp3_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp3_AST);
			ASTPair __currentAST1876 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,BASICDECL);
			_t = _t.getFirstChild();
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case ARRAYDECL:
			{
				{
				dec=arrayDecl(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				}
				break;
			}
			case VARDECL:
			{
				{
				dec=varDecl(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				}
				break;
			}
			case CONSTDECL:
			{
				{
				dec=constDecl(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST1876;
			_t = __t1876;
			_t = _t.getNextSibling();
			if ( inputState.guessing==0 ) {
				
						//s += " basicdecl ";
						s += dec;
						
					
			}
			basicDecl_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = basicDecl_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  funcDeclaration(AST _t) throws RecognitionException {
		String s="";
		
		AST funcDeclaration_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST funcDeclaration_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		
			String s1 = "";
		
		
		try {      // for error handling
			AST __t1902 = _t;
			AST tmp4_AST = null;
			AST tmp4_AST_in = null;
			tmp4_AST = astFactory.create((AST)_t);
			tmp4_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp4_AST);
			ASTPair __currentAST1902 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,FUNCTIONDECLARATION);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " functionDecl ";
			}
			{
			a = (AST)_t;
			AST a_AST_in = null;
			a_AST = astFactory.create(a);
			astFactory.addASTChild(currentAST, a_AST);
			match(_t,IDENT);
			_t = _t.getNextSibling();
			}
			if ( inputState.guessing==0 ) {
				s += a.getText() + " ";
			}
			{
			_loop1905:
			do {
				if (_t==null) _t=ASTNULL;
				if ((_t.getType()==IDENT)) {
					b = (AST)_t;
					AST b_AST_in = null;
					b_AST = astFactory.create(b);
					astFactory.addASTChild(currentAST, b_AST);
					match(_t,IDENT);
					_t = _t.getNextSibling();
					if ( inputState.guessing==0 ) {
						s += b.getText() + " \n";
					}
				}
				else {
					break _loop1905;
				}
				
			} while (true);
			}
			{
			_loop1907:
			do {
				if (_t==null) _t=ASTNULL;
				if ((_t.getType()==STATEMENT)) {
					s1=statement(_t);
					_t = _retTree;
					astFactory.addASTChild(currentAST, returnAST);
					if ( inputState.guessing==0 ) {
						s += "\t" + s1;
					}
				}
				else {
					break _loop1907;
				}
				
			} while (true);
			}
			currentAST = __currentAST1902;
			_t = __t1902;
			_t = _t.getNextSibling();
			funcDeclaration_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = funcDeclaration_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  statement(AST _t) throws RecognitionException {
		String s="";
		
		AST statement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST statement_AST = null;
		
			String s1="";
		
		
		try {      // for error handling
			AST __t1909 = _t;
			AST tmp5_AST = null;
			AST tmp5_AST_in = null;
			tmp5_AST = astFactory.create((AST)_t);
			tmp5_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp5_AST);
			ASTPair __currentAST1909 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,STATEMENT);
			_t = _t.getFirstChild();
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case INSTRUCTIONSTATEMENT:
			{
				s1=instructionStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case ORGSTATEMENT:
			{
				s1=orgStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case IFSTATEMENT:
			{
				s1=ifStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case WHILESTATEMENT:
			{
				s1=whileStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case FORSTATEMENT:
			{
				s1=forStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case DOWHILESTATEMENT:
			{
				s1=dowhileStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case ASSIGNMENTSTATEMENT:
			{
				s1=assignmentStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case FUNCTIONCALLSTATEMENT:
			{
				s1=functioncallStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case RETURNSTATEMENT:
			{
				s1=returnStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case LABELSTATEMENT:
			{
				s1=labelStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST1909;
			_t = __t1909;
			_t = _t.getNextSibling();
			if ( inputState.guessing==0 ) {
				
							s += s1 + "\n";
						
			}
			statement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = statement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  arrayDecl(AST _t) throws RecognitionException {
		String s="";
		
		AST arrayDecl_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST arrayDecl_AST = null;
		AST id = null;
		AST id_AST = null;
		AST num1 = null;
		AST num1_AST = null;
		AST num2 = null;
		AST num2_AST = null;
		AST id2 = null;
		AST id2_AST = null;
		AST num3 = null;
		AST num3_AST = null;
		
		
		
		
		try {      // for error handling
			boolean synPredMatched1886 = false;
			if (_t==null) _t=ASTNULL;
			if (((_t.getType()==ARRAYDECL))) {
				AST __t1886 = _t;
				synPredMatched1886 = true;
				inputState.guessing++;
				try {
					{
					AST __t1885 = _t;
					ASTPair __currentAST1885 = currentAST.copy();
					currentAST.root = currentAST.child;
					currentAST.child = null;
					match(_t,ARRAYDECL);
					_t = _t.getFirstChild();
					match(_t,IDENT);
					_t = _t.getNextSibling();
					match(_t,NUMBER);
					_t = _t.getNextSibling();
					match(_t,NUMBER);
					_t = _t.getNextSibling();
					currentAST = __currentAST1885;
					_t = __t1885;
					_t = _t.getNextSibling();
					}
				}
				catch (RecognitionException pe) {
					synPredMatched1886 = false;
				}
				_t = __t1886;
inputState.guessing--;
			}
			if ( synPredMatched1886 ) {
				AST __t1887 = _t;
				AST tmp6_AST = null;
				AST tmp6_AST_in = null;
				tmp6_AST = astFactory.create((AST)_t);
				tmp6_AST_in = (AST)_t;
				astFactory.addASTChild(currentAST, tmp6_AST);
				ASTPair __currentAST1887 = currentAST.copy();
				currentAST.root = currentAST.child;
				currentAST.child = null;
				match(_t,ARRAYDECL);
				_t = _t.getFirstChild();
				id = (AST)_t;
				AST id_AST_in = null;
				id_AST = astFactory.create(id);
				astFactory.addASTChild(currentAST, id_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				num1 = (AST)_t;
				AST num1_AST_in = null;
				num1_AST = astFactory.create(num1);
				astFactory.addASTChild(currentAST, num1_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				num2 = (AST)_t;
				AST num2_AST_in = null;
				num2_AST = astFactory.create(num2);
				astFactory.addASTChild(currentAST, num2_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				currentAST = __currentAST1887;
				_t = __t1887;
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
						// array F(5,221)
						 	/*
						 	s+=" arraydecl ";
							s += Integer.parseInt(num1.getText());
							s += " ";
							s += Integer.parseInt(num2.getText());
							*/
							
							sTable.add(id.getText(), "-1", 3, str2int(num2.getText()), str2int(num1.getText()));
							sTable.printTable();
							addressNumber--;
						
				}
				arrayDecl_AST = (AST)currentAST.root;
			}
			else {
				boolean synPredMatched1890 = false;
				if (_t==null) _t=ASTNULL;
				if (((_t.getType()==ARRAYDECL))) {
					AST __t1890 = _t;
					synPredMatched1890 = true;
					inputState.guessing++;
					try {
						{
						AST __t1889 = _t;
						ASTPair __currentAST1889 = currentAST.copy();
						currentAST.root = currentAST.child;
						currentAST.child = null;
						match(_t,ARRAYDECL);
						_t = _t.getFirstChild();
						match(_t,IDENT);
						_t = _t.getNextSibling();
						match(_t,NUMBER);
						_t = _t.getNextSibling();
						currentAST = __currentAST1889;
						_t = __t1889;
						_t = _t.getNextSibling();
						}
					}
					catch (RecognitionException pe) {
						synPredMatched1890 = false;
					}
					_t = __t1890;
inputState.guessing--;
				}
				if ( synPredMatched1890 ) {
					AST __t1891 = _t;
					AST tmp7_AST = null;
					AST tmp7_AST_in = null;
					tmp7_AST = astFactory.create((AST)_t);
					tmp7_AST_in = (AST)_t;
					astFactory.addASTChild(currentAST, tmp7_AST);
					ASTPair __currentAST1891 = currentAST.copy();
					currentAST.root = currentAST.child;
					currentAST.child = null;
					match(_t,ARRAYDECL);
					_t = _t.getFirstChild();
					id2 = (AST)_t;
					AST id2_AST_in = null;
					id2_AST = astFactory.create(id2);
					astFactory.addASTChild(currentAST, id2_AST);
					match(_t,IDENT);
					_t = _t.getNextSibling();
					num3 = (AST)_t;
					AST num3_AST_in = null;
					num3_AST = astFactory.create(num3);
					astFactory.addASTChild(currentAST, num3_AST);
					match(_t,NUMBER);
					_t = _t.getNextSibling();
					currentAST = __currentAST1891;
					_t = __t1891;
					_t = _t.getNextSibling();
					if ( inputState.guessing==0 ) {
						
							 	/*
								s+=" arraydecl ";
								s += Integer.parseInt(num3.getText());
								*/
								sTable.add(id2.getText(), "-1", 3, -1, str2int(num3.getText()));
								sTable.printTable();
								addressNumber--;
							
					}
					arrayDecl_AST = (AST)currentAST.root;
				}
				else {
					throw new NoViableAltException(_t);
				}
				}
			}
			catch (RecognitionException ex) {
				if (inputState.guessing==0) {
					reportError(ex);
					if (_t!=null) {_t = _t.getNextSibling();}
				} else {
				  throw ex;
				}
			}
			returnAST = arrayDecl_AST;
			_retTree = _t;
			return s;
		}
		
	public final String  varDecl(AST _t) throws RecognitionException {
		String s="";
		
		AST varDecl_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST varDecl_AST = null;
		AST id = null;
		AST id_AST = null;
		AST num = null;
		AST num_AST = null;
		AST id2 = null;
		AST id2_AST = null;
		
		
		
		
		try {      // for error handling
			boolean synPredMatched1895 = false;
			if (_t==null) _t=ASTNULL;
			if (((_t.getType()==VARDECL))) {
				AST __t1895 = _t;
				synPredMatched1895 = true;
				inputState.guessing++;
				try {
					{
					AST __t1894 = _t;
					ASTPair __currentAST1894 = currentAST.copy();
					currentAST.root = currentAST.child;
					currentAST.child = null;
					match(_t,VARDECL);
					_t = _t.getFirstChild();
					match(_t,IDENT);
					_t = _t.getNextSibling();
					match(_t,NUMBER);
					_t = _t.getNextSibling();
					currentAST = __currentAST1894;
					_t = __t1894;
					_t = _t.getNextSibling();
					}
				}
				catch (RecognitionException pe) {
					synPredMatched1895 = false;
				}
				_t = __t1895;
inputState.guessing--;
			}
			if ( synPredMatched1895 ) {
				AST __t1896 = _t;
				AST tmp8_AST = null;
				AST tmp8_AST_in = null;
				tmp8_AST = astFactory.create((AST)_t);
				tmp8_AST_in = (AST)_t;
				astFactory.addASTChild(currentAST, tmp8_AST);
				ASTPair __currentAST1896 = currentAST.copy();
				currentAST.root = currentAST.child;
				currentAST.child = null;
				match(_t,VARDECL);
				_t = _t.getFirstChild();
				id = (AST)_t;
				AST id_AST_in = null;
				id_AST = astFactory.create(id);
				astFactory.addASTChild(currentAST, id_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				num = (AST)_t;
				AST num_AST_in = null;
				num_AST = astFactory.create(num);
				astFactory.addASTChild(currentAST, num_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				currentAST = __currentAST1896;
				_t = __t1896;
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
						 	/*
						 	s+=" vardecl ";
							s += Integer.parseInt(num.getText());
							*/
							
							sTable.add(id.getText(), num.getText(), 2, -1, 1);
							sTable.printTable();
							addressNumber--;
						
				}
				varDecl_AST = (AST)currentAST.root;
			}
			else {
				boolean synPredMatched1899 = false;
				if (_t==null) _t=ASTNULL;
				if (((_t.getType()==VARDECL))) {
					AST __t1899 = _t;
					synPredMatched1899 = true;
					inputState.guessing++;
					try {
						{
						AST __t1898 = _t;
						ASTPair __currentAST1898 = currentAST.copy();
						currentAST.root = currentAST.child;
						currentAST.child = null;
						match(_t,VARDECL);
						_t = _t.getFirstChild();
						match(_t,IDENT);
						_t = _t.getNextSibling();
						currentAST = __currentAST1898;
						_t = __t1898;
						_t = _t.getNextSibling();
						}
					}
					catch (RecognitionException pe) {
						synPredMatched1899 = false;
					}
					_t = __t1899;
inputState.guessing--;
				}
				if ( synPredMatched1899 ) {
					AST __t1900 = _t;
					AST tmp9_AST = null;
					AST tmp9_AST_in = null;
					tmp9_AST = astFactory.create((AST)_t);
					tmp9_AST_in = (AST)_t;
					astFactory.addASTChild(currentAST, tmp9_AST);
					ASTPair __currentAST1900 = currentAST.copy();
					currentAST.root = currentAST.child;
					currentAST.child = null;
					match(_t,VARDECL);
					_t = _t.getFirstChild();
					id2 = (AST)_t;
					AST id2_AST_in = null;
					id2_AST = astFactory.create(id2);
					astFactory.addASTChild(currentAST, id2_AST);
					match(_t,IDENT);
					_t = _t.getNextSibling();
					currentAST = __currentAST1900;
					_t = __t1900;
					_t = _t.getNextSibling();
					if ( inputState.guessing==0 ) {
						
							 	/*
								s+=" vardecl ";
								*/
								sTable.add(id2.getText(), "-1", 2, -1, 1);
								sTable.printTable();
								addressNumber--;
							
					}
					varDecl_AST = (AST)currentAST.root;
				}
				else {
					throw new NoViableAltException(_t);
				}
				}
			}
			catch (RecognitionException ex) {
				if (inputState.guessing==0) {
					reportError(ex);
					if (_t!=null) {_t = _t.getNextSibling();}
				} else {
				  throw ex;
				}
			}
			returnAST = varDecl_AST;
			_retTree = _t;
			return s;
		}
		
	public final String  constDecl(AST _t) throws RecognitionException {
		String s="";
		
		AST constDecl_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST constDecl_AST = null;
		AST id = null;
		AST id_AST = null;
		AST num = null;
		AST num_AST = null;
		
		
		
		
		try {      // for error handling
			AST __t1882 = _t;
			AST tmp10_AST = null;
			AST tmp10_AST_in = null;
			tmp10_AST = astFactory.create((AST)_t);
			tmp10_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp10_AST);
			ASTPair __currentAST1882 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,CONSTDECL);
			_t = _t.getFirstChild();
			id = (AST)_t;
			AST id_AST_in = null;
			id_AST = astFactory.create(id);
			astFactory.addASTChild(currentAST, id_AST);
			match(_t,IDENT);
			_t = _t.getNextSibling();
			num = (AST)_t;
			AST num_AST_in = null;
			num_AST = astFactory.create(num);
			astFactory.addASTChild(currentAST, num_AST);
			match(_t,NUMBER);
			_t = _t.getNextSibling();
			currentAST = __currentAST1882;
			_t = __t1882;
			_t = _t.getNextSibling();
			if ( inputState.guessing==0 ) {
				
						sTable.add(id.getText(), num.getText(), 1, -1, 1);
						sTable.printTable();
						addressNumber--;
					
			}
			constDecl_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = constDecl_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  instructionStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST instructionStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST instructionStatement_AST = null;
		
			String s1="";
		
		
		try {      // for error handling
			AST __t1912 = _t;
			AST tmp11_AST = null;
			AST tmp11_AST_in = null;
			tmp11_AST = astFactory.create((AST)_t);
			tmp11_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp11_AST);
			ASTPair __currentAST1912 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,INSTRUCTIONSTATEMENT);
			_t = _t.getFirstChild();
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case ASMPPSTATEMENT:
			{
				s1=asmppStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case ASMSTATEMENT:
			{
				s1=asmStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST1912;
			_t = __t1912;
			_t = _t.getNextSibling();
			if ( inputState.guessing==0 ) {
				
						s += s1;
					
			}
			instructionStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = instructionStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  orgStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST orgStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST orgStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		
		
		
		try {      // for error handling
			AST __t2069 = _t;
			AST tmp12_AST = null;
			AST tmp12_AST_in = null;
			tmp12_AST = astFactory.create((AST)_t);
			tmp12_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp12_AST);
			ASTPair __currentAST2069 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,ORGSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				/*s += " orgStatement ";*/ s+=" org ";
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									// only constant values are allowed in an "org" statement
									s += sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += b.getText() + " ";
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST2069;
			_t = __t2069;
			_t = _t.getNextSibling();
			orgStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = orgStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  ifStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST ifStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST ifStatement_AST = null;
		
			String s1="";
			String s2="";
			String s3="";
		
		
		try {      // for error handling
			AST __t2072 = _t;
			AST tmp13_AST = null;
			AST tmp13_AST_in = null;
			tmp13_AST = astFactory.create((AST)_t);
			tmp13_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp13_AST);
			ASTPair __currentAST2072 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,IFSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				ifCount++; ifStack.push(ifCount);
			}
			{
			s1=expr(_t);
			_t = _retTree;
			astFactory.addASTChild(currentAST, returnAST);
			if ( inputState.guessing==0 ) {
				s += s1; s+= "IF" + ifStack.peek() + ":\n";
			}
			{
			_loop2075:
			do {
				if (_t==null) _t=ASTNULL;
				if ((_t.getType()==STATEMENT)) {
					s2=statement(_t);
					_t = _retTree;
					astFactory.addASTChild(currentAST, returnAST);
					if ( inputState.guessing==0 ) {
						s += s2;
					}
				}
				else {
					break _loop2075;
				}
				
			} while (true);
			}
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case ELSE:
			{
				AST tmp14_AST = null;
				AST tmp14_AST_in = null;
				tmp14_AST = astFactory.create((AST)_t);
				tmp14_AST_in = (AST)_t;
				astFactory.addASTChild(currentAST, tmp14_AST);
				match(_t,ELSE);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s+= "goto ENDIF" + ifStack.peek() + "\n";s+= "ELSE" + ifStack.peek() + ":\n";
				}
				{
				_loop2078:
				do {
					if (_t==null) _t=ASTNULL;
					if ((_t.getType()==STATEMENT)) {
						s3=statement(_t);
						_t = _retTree;
						astFactory.addASTChild(currentAST, returnAST);
						if ( inputState.guessing==0 ) {
							s += s3;
						}
					}
					else {
						break _loop2078;
					}
					
				} while (true);
				}
				break;
			}
			case 3:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST2072;
			_t = __t2072;
			_t = _t.getNextSibling();
			if ( inputState.guessing==0 ) {
				
					 	s+= "ENDIF" + ifStack.peek() + ":\n";
					 	ifStack.pop();	
					
			}
			ifStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = ifStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  whileStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST whileStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST whileStatement_AST = null;
		
			String s1="";
			String s2="";
				
		
		
		try {      // for error handling
			AST __t2099 = _t;
			AST tmp15_AST = null;
			AST tmp15_AST_in = null;
			tmp15_AST = astFactory.create((AST)_t);
			tmp15_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp15_AST);
			ASTPair __currentAST2099 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,WHILESTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				ifCount++; ifStack.push(ifCount);
			}
			{
			s1=expr(_t);
			_t = _retTree;
			astFactory.addASTChild(currentAST, returnAST);
			if ( inputState.guessing==0 ) {
				s += s1; s+= "WHILE" + ifStack.peek() + ":\n";
			}
			{
			_loop2102:
			do {
				if (_t==null) _t=ASTNULL;
				if ((_t.getType()==STATEMENT)) {
					s2=statement(_t);
					_t = _retTree;
					astFactory.addASTChild(currentAST, returnAST);
					if ( inputState.guessing==0 ) {
						s += s2;
					}
				}
				else {
					break _loop2102;
				}
				
			} while (true);
			}
			}
			currentAST = __currentAST2099;
			_t = __t2099;
			_t = _t.getNextSibling();
			if ( inputState.guessing==0 ) {
				
					 	s+= "goto WHILE" + ifStack.peek() + "\n";
						s+= "ELSE " + ifStack.peek() + ":\n";
						ifStack.pop();	
					
			}
			whileStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = whileStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  forStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST forStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST forStatement_AST = null;
		
			String s1="";
			String s2="";
			String s3="";
			String s4="";		
		
		
		try {      // for error handling
			AST __t2104 = _t;
			AST tmp16_AST = null;
			AST tmp16_AST_in = null;
			tmp16_AST = astFactory.create((AST)_t);
			tmp16_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp16_AST);
			ASTPair __currentAST2104 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,FORSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				ifCount++; ifStack.push(ifCount);
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case FORSUBSTATEMENT:
			{
				s1=forSubStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				if ( inputState.guessing==0 ) {
					s += s1+"\n";s+= "FOR" + ifStack.peek() + ":\n";
				}
				break;
			}
			case EXPR:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			{
			s2=expr(_t);
			_t = _retTree;
			astFactory.addASTChild(currentAST, returnAST);
			if ( inputState.guessing==0 ) {
				s += s2;
			}
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case FORSUBSTATEMENT:
			{
				s3=forSubStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case 3:
			case STATEMENT:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			{
			_loop2109:
			do {
				if (_t==null) _t=ASTNULL;
				if ((_t.getType()==STATEMENT)) {
					s4=statement(_t);
					_t = _retTree;
					astFactory.addASTChild(currentAST, returnAST);
					if ( inputState.guessing==0 ) {
						s += s4;
					}
				}
				else {
					break _loop2109;
				}
				
			} while (true);
			}
			if ( inputState.guessing==0 ) {
				s += s3;
			}
			currentAST = __currentAST2104;
			_t = __t2104;
			_t = _t.getNextSibling();
			if ( inputState.guessing==0 ) {
				
					 	s+= "\ngoto FOR" + ifStack.peek() + "\n";
						s+= "ELSE " + ifStack.peek() + ":\n";
						ifStack.pop();	
					
			}
			forStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = forStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  dowhileStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST dowhileStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST dowhileStatement_AST = null;
		
			String s1="";
			String s2="";	
		
		
		try {      // for error handling
			AST __t2114 = _t;
			AST tmp17_AST = null;
			AST tmp17_AST_in = null;
			tmp17_AST = astFactory.create((AST)_t);
			tmp17_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp17_AST);
			ASTPair __currentAST2114 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,DOWHILESTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				ifCount++; ifStack.push(ifCount);s+= "DOWHILE" + ifStack.peek() + ":\n";
			}
			{
			{
			_loop2117:
			do {
				if (_t==null) _t=ASTNULL;
				if ((_t.getType()==STATEMENT)) {
					s2=statement(_t);
					_t = _retTree;
					astFactory.addASTChild(currentAST, returnAST);
					if ( inputState.guessing==0 ) {
						s += s2;
					}
				}
				else {
					break _loop2117;
				}
				
			} while (true);
			}
			{
			s1=expr(_t);
			_t = _retTree;
			astFactory.addASTChild(currentAST, returnAST);
			}
			if ( inputState.guessing==0 ) {
				s += s1;
			}
			}
			currentAST = __currentAST2114;
			_t = __t2114;
			_t = _t.getNextSibling();
			if ( inputState.guessing==0 ) {
				
					 	s+= "\ngoto DOWHILE" + ifStack.peek() + "\n";
						s+= "ELSE " + ifStack.peek() + ":\n";
						ifStack.pop();	
					
			}
			dowhileStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = dowhileStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  assignmentStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST assignmentStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST assignmentStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		AST c = null;
		AST c_AST = null;
		AST d = null;
		AST d_AST = null;
		AST e = null;
		AST e_AST = null;
		
			int op1type=0;
			int op2type=0;
			String s1="";
			String s2="";	
		
		
		try {      // for error handling
			AST __t2120 = _t;
			AST tmp18_AST = null;
			AST tmp18_AST_in = null;
			tmp18_AST = astFactory.create((AST)_t);
			tmp18_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp18_AST);
			ASTPair __currentAST2120 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,ASSIGNMENTSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				/*s += " assignmentStatement ";*/
			}
			{
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case ADDRESS:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,ADDRESS);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					op1type=1;
				}
				break;
			}
			case IDENT:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					op1type=2;
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case ADDRESS:
			{
				c = (AST)_t;
				AST c_AST_in = null;
				c_AST = astFactory.create(c);
				astFactory.addASTChild(currentAST, c_AST);
				match(_t,ADDRESS);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					op2type=1;
				}
				break;
			}
			case IDENT:
			{
				d = (AST)_t;
				AST d_AST_in = null;
				d_AST = astFactory.create(d);
				astFactory.addASTChild(currentAST, d_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					op2type=2;
				}
				break;
			}
			case NUMBER:
			{
				e = (AST)_t;
				AST e_AST_in = null;
				e_AST = astFactory.create(e);
				astFactory.addASTChild(currentAST, e_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					op2type=3;
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			}
			currentAST = __currentAST2120;
			_t = __t2120;
			_t = _t.getNextSibling();
			if ( inputState.guessing==0 ) {
				
					 	s += "op1:" + op1type + " op2:" + op2type + "\n";
					 	
					 	switch(op2type)//RHS
					 	{
					 		case 1: //address
					 			s += "movf " + sTable.getSymbolValue(c.getText()) + "\n";
					 			switch(op1type)//LHS
							 	{
							 		case 1: //address 
						 				System.err.println("Invalid assignment LHS=address");
							 			break;
							 		case 2: //ident
							 			if(sTable.getSymbolType(b.getText()) == 1)
							 				System.err.println("Invalid assignment 1");
							 			else
							 				s += "movwf " + sTable.getSymbolValue(b.getText()) + "\n";			 		
							 			break;
							 	} 
					 			break;
					 		case 2: //ident
					 			s += "movlw " + sTable.getSymbolValue(d.getText()) + "\n";
					 			switch(op1type)
							 	{
							 		case 1:
							 			System.err.println("Invalid assignment LHS=address");
							 			break;
							 		case 2:
							 			if(sTable.getSymbolType(b.getText()) == 1)
							 				System.err.println("Invalid assignment 1");
							 			else
							 				s += "movwf " + sTable.getSymbolValue(b.getText()) + "\n";		
							 			break;
							 	} 
					 			break;
					 		case 3: //number
					 			s += "movlw " + e.getText() + "\n";
					 			switch(op1type)
							 	{
							 		case 1:
										System.err.println("Invalid assignment LHS=address");
							 			break;
							 		case 2:
							 			if(sTable.getSymbolType(b.getText()) == 1)
							 				System.err.println("Invalid assignment 1");
							 			else
							 				s += "movwf " + sTable.getSymbolValue(b.getText()) + "\n";		
							 			break;
							 	} 
					 			break;
					 	} 
					
			}
			assignmentStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = assignmentStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  functioncallStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST functioncallStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST functioncallStatement_AST = null;
		
				
		
		
		try {      // for error handling
			AST __t2125 = _t;
			AST tmp19_AST = null;
			AST tmp19_AST_in = null;
			tmp19_AST = astFactory.create((AST)_t);
			tmp19_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp19_AST);
			ASTPair __currentAST2125 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,FUNCTIONCALLSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " functionCallStatement ";
			}
			{
			AST tmp20_AST = null;
			AST tmp20_AST_in = null;
			tmp20_AST = astFactory.create((AST)_t);
			tmp20_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp20_AST);
			match(_t,IDENT);
			_t = _t.getNextSibling();
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			case NUMBER:
			case ADDRESS:
			{
				{
				if (_t==null) _t=ASTNULL;
				switch ( _t.getType()) {
				case ADDRESS:
				{
					AST tmp21_AST = null;
					AST tmp21_AST_in = null;
					tmp21_AST = astFactory.create((AST)_t);
					tmp21_AST_in = (AST)_t;
					astFactory.addASTChild(currentAST, tmp21_AST);
					match(_t,ADDRESS);
					_t = _t.getNextSibling();
					break;
				}
				case IDENT:
				{
					AST tmp22_AST = null;
					AST tmp22_AST_in = null;
					tmp22_AST = astFactory.create((AST)_t);
					tmp22_AST_in = (AST)_t;
					astFactory.addASTChild(currentAST, tmp22_AST);
					match(_t,IDENT);
					_t = _t.getNextSibling();
					break;
				}
				case NUMBER:
				{
					AST tmp23_AST = null;
					AST tmp23_AST_in = null;
					tmp23_AST = astFactory.create((AST)_t);
					tmp23_AST_in = (AST)_t;
					astFactory.addASTChild(currentAST, tmp23_AST);
					match(_t,NUMBER);
					_t = _t.getNextSibling();
					break;
				}
				default:
				{
					throw new NoViableAltException(_t);
				}
				}
				}
				{
				_loop2130:
				do {
					if (_t==null) _t=ASTNULL;
					switch ( _t.getType()) {
					case ADDRESS:
					{
						AST tmp24_AST = null;
						AST tmp24_AST_in = null;
						tmp24_AST = astFactory.create((AST)_t);
						tmp24_AST_in = (AST)_t;
						astFactory.addASTChild(currentAST, tmp24_AST);
						match(_t,ADDRESS);
						_t = _t.getNextSibling();
						break;
					}
					case IDENT:
					{
						AST tmp25_AST = null;
						AST tmp25_AST_in = null;
						tmp25_AST = astFactory.create((AST)_t);
						tmp25_AST_in = (AST)_t;
						astFactory.addASTChild(currentAST, tmp25_AST);
						match(_t,IDENT);
						_t = _t.getNextSibling();
						break;
					}
					case NUMBER:
					{
						AST tmp26_AST = null;
						AST tmp26_AST_in = null;
						tmp26_AST = astFactory.create((AST)_t);
						tmp26_AST_in = (AST)_t;
						astFactory.addASTChild(currentAST, tmp26_AST);
						match(_t,NUMBER);
						_t = _t.getNextSibling();
						break;
					}
					default:
					{
						break _loop2130;
					}
					}
				} while (true);
				}
				break;
			}
			case 3:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			}
			currentAST = __currentAST2125;
			_t = __t2125;
			_t = _t.getNextSibling();
			functioncallStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = functioncallStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  returnStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST returnStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST returnStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		AST c = null;
		AST c_AST = null;
		
			int optype=0;		
		
		
		try {      // for error handling
			AST __t2132 = _t;
			AST tmp27_AST = null;
			AST tmp27_AST_in = null;
			tmp27_AST = astFactory.create((AST)_t);
			tmp27_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp27_AST);
			ASTPair __currentAST2132 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,RETURNSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case ADDRESS:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,ADDRESS);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					optype=1;
				}
				break;
			}
			case IDENT:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					optype=2;
				}
				break;
			}
			case NUMBER:
			{
				c = (AST)_t;
				AST c_AST_in = null;
				c_AST = astFactory.create(c);
				astFactory.addASTChild(currentAST, c_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					optype=3;
				}
				break;
			}
			case 3:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST2132;
			_t = __t2132;
			_t = _t.getNextSibling();
			if ( inputState.guessing==0 ) {
				
								switch(optype)
								{
									case 0:
										s += "return";
										break;
									case 1:
										s += "movf " + sTable.getSymbolValue(a.getText()) + ", 0\n";
										s += "retlw";
										break;
									case 2:
										s += "movlw " + sTable.getSymbolValue(b.getText()) + "\n";
										s += "retlw";
										break;
									case 3:
										s += "movlw " + c.getText() + "\n";
										s += "retlw";
										break;
								}
							
			}
			returnStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = returnStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  labelStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST labelStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST labelStatement_AST = null;
		AST id = null;
		AST id_AST = null;
		
				
		
		
		try {      // for error handling
			AST __t2135 = _t;
			AST tmp28_AST = null;
			AST tmp28_AST_in = null;
			tmp28_AST = astFactory.create((AST)_t);
			tmp28_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp28_AST);
			ASTPair __currentAST2135 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,LABELSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
			}
			id = (AST)_t;
			AST id_AST_in = null;
			id_AST = astFactory.create(id);
			astFactory.addASTChild(currentAST, id_AST);
			match(_t,IDENT);
			_t = _t.getNextSibling();
			currentAST = __currentAST2135;
			_t = __t2135;
			_t = _t.getNextSibling();
			if ( inputState.guessing==0 ) {
				
					 	s += id.getText() + ":" ;
					
			}
			labelStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = labelStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  asmppStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST asmppStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST asmppStatement_AST = null;
		
			String s1="";
		
		
		try {      // for error handling
			AST __t1915 = _t;
			AST tmp29_AST = null;
			AST tmp29_AST_in = null;
			tmp29_AST = astFactory.create((AST)_t);
			tmp29_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp29_AST);
			ASTPair __currentAST1915 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,ASMPPSTATEMENT);
			_t = _t.getFirstChild();
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case ADDFFSTATEMENT:
			{
				s1=addffStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case SUBFFSTATEMENT:
			{
				s1=subffStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case ADDWFFSTATEMENT:
			{
				s1=addwffStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case SUBWFFSTATEMENT:
			{
				s1=subwffStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case SWAPFFSTATEMENT:
			{
				s1=swapffStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case IORWFFSTATEMENT:
			{
				s1=iorwffStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case ANDWFFSTATEMENT:
			{
				s1=andwffStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case XORWFFSTATEMENT:
			{
				s1=xorwffStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case MOVFFSTATEMENT:
			{
				s1=movffStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST1915;
			_t = __t1915;
			_t = _t.getNextSibling();
			if ( inputState.guessing==0 ) {
				
						s += s1;
					
			}
			asmppStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = asmppStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  asmStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST asmStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST asmStatement_AST = null;
			
			String s1 = "";
		
		
		try {      // for error handling
			AST __t1954 = _t;
			AST tmp30_AST = null;
			AST tmp30_AST_in = null;
			tmp30_AST = astFactory.create((AST)_t);
			tmp30_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp30_AST);
			ASTPair __currentAST1954 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,ASMSTATEMENT);
			_t = _t.getFirstChild();
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case ADDWFSTATEMENT:
			{
				s1=addwfStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case ANDWFSTATEMENT:
			{
				s1=andwfStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case CLRFSTATEMENT:
			{
				s1=clrfStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case CLRWSTATEMENT:
			{
				s1=clrwStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case COMFSTATEMENT:
			{
				s1=comfStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case DECFSTATEMENT:
			{
				s1=decfStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case DECFSZSTATEMENT:
			{
				s1=decfszStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case INCFSTATEMENT:
			{
				s1=incfStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case INCFSZSTATEMENT:
			{
				s1=incfszStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case IORWFSTATEMENT:
			{
				s1=iorwfStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case MOVFSTATEMENT:
			{
				s1=movfStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case MOVWFSTATEMENT:
			{
				s1=movwfStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case NOPSTATEMENT:
			{
				s1=nopStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case RLFSTATEMENT:
			{
				s1=rlfStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case RRFSTATEMENT:
			{
				s1=rrfStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case SUBWFSTATEMENT:
			{
				s1=subwfStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case SWAPFSTATEMENT:
			{
				s1=swapfStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case XORWFSTATEMENT:
			{
				s1=xorwfStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case BCFSTATEMENT:
			{
				s1=bcfStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case BSFSTATEMENT:
			{
				s1=bsfStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case BTFSCSTATEMENT:
			{
				s1=btfscStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case BTFSSSTATEMENT:
			{
				s1=btfssStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case ADDLWSTATEMENT:
			{
				s1=addlwStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case ANDLWSTATEMENT:
			{
				s1=andlwStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case CALLSTATEMENT:
			{
				s1=callStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case CLRWDTSTATEMENT:
			{
				s1=clrwdtStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case GOTOSTATEMENT:
			{
				s1=gotoStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case IORLWSTATEMENT:
			{
				s1=iorlwStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case MOVLWSTATEMENT:
			{
				s1=movlwStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case RETFIESTATEMENT:
			{
				s1=retfieStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case RETLWSTATEMENT:
			{
				s1=retlwStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case SLEEPSTATEMENT:
			{
				s1=sleepStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case SUBLWSTATEMENT:
			{
				s1=sublwStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case XORLWSTATEMENT:
			{
				s1=xorlwStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST1954;
			_t = __t1954;
			_t = _t.getNextSibling();
			if ( inputState.guessing==0 ) {
				
						s += s1;
					
			}
			asmStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = asmStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  addffStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST addffStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST addffStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		AST c = null;
		AST c_AST = null;
		AST d = null;
		AST d_AST = null;
		
			int v1=0;
			int v2=0;
		
		
		try {      // for error handling
			AST __t1918 = _t;
			AST tmp31_AST = null;
			AST tmp31_AST_in = null;
			tmp31_AST = astFactory.create((AST)_t);
			tmp31_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp31_AST);
			ASTPair __currentAST1918 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,ADDFFSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									v1 = sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					v1 = str2int( b.getText());
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				c = (AST)_t;
				AST c_AST_in = null;
				c_AST = astFactory.create(c);
				astFactory.addASTChild(currentAST, c_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
										v2 = sTable.getSymbolValue(c.getText());
									
				}
				break;
			}
			case NUMBER:
			{
				d = (AST)_t;
				AST d_AST_in = null;
				d_AST = astFactory.create(d);
				astFactory.addASTChild(currentAST, d_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					v2 = str2int(d.getText());
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST1918;
			_t = __t1918;
			_t = _t.getNextSibling();
			if ( inputState.guessing==0 ) {
				
					 	s += " movf "  + v2 + ", 0 " + "\n";
					 	s += " addwf " + v1 + ", 1 ";
					
			}
			addffStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = addffStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  subffStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST subffStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST subffStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		AST c = null;
		AST c_AST = null;
		AST d = null;
		AST d_AST = null;
		
			int v1=0;
			int v2=0;
		
		
		try {      // for error handling
			AST __t1922 = _t;
			AST tmp32_AST = null;
			AST tmp32_AST_in = null;
			tmp32_AST = astFactory.create((AST)_t);
			tmp32_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp32_AST);
			ASTPair __currentAST1922 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,SUBFFSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									v1 = sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					v1 = str2int( b.getText());
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				c = (AST)_t;
				AST c_AST_in = null;
				c_AST = astFactory.create(c);
				astFactory.addASTChild(currentAST, c_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
										v2 = sTable.getSymbolValue(c.getText());
									
				}
				break;
			}
			case NUMBER:
			{
				d = (AST)_t;
				AST d_AST_in = null;
				d_AST = astFactory.create(d);
				astFactory.addASTChild(currentAST, d_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					v2 = str2int(d.getText());
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST1922;
			_t = __t1922;
			_t = _t.getNextSibling();
			if ( inputState.guessing==0 ) {
				
					 	s += " movf "  + v2 + ", 0 " + "\n";
					 	s += " subwf " + v1 + ", 1 ";
					
			}
			subffStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = subffStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  addwffStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST addwffStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST addwffStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		AST c = null;
		AST c_AST = null;
		AST d = null;
		AST d_AST = null;
		
			int v1=0;
			int v2=0;
		
		
		try {      // for error handling
			AST __t1926 = _t;
			AST tmp33_AST = null;
			AST tmp33_AST_in = null;
			tmp33_AST = astFactory.create((AST)_t);
			tmp33_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp33_AST);
			ASTPair __currentAST1926 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,ADDWFFSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									v1 = sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					v1 = str2int( b.getText());
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				c = (AST)_t;
				AST c_AST_in = null;
				c_AST = astFactory.create(c);
				astFactory.addASTChild(currentAST, c_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
										v2 = sTable.getSymbolValue(c.getText());
									
				}
				break;
			}
			case NUMBER:
			{
				d = (AST)_t;
				AST d_AST_in = null;
				d_AST = astFactory.create(d);
				astFactory.addASTChild(currentAST, d_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					v2 = str2int(d.getText());
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST1926;
			_t = __t1926;
			_t = _t.getNextSibling();
			if ( inputState.guessing==0 ) {
				
					 	s += " movf "  + v2 + ", 0 " + "\n";
					 	s += " addwf " + v1 + ", 0 ";
					
			}
			addwffStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = addwffStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  subwffStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST subwffStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST subwffStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		AST c = null;
		AST c_AST = null;
		AST d = null;
		AST d_AST = null;
		
			int v1=0;
			int v2=0;
		
		
		try {      // for error handling
			AST __t1930 = _t;
			AST tmp34_AST = null;
			AST tmp34_AST_in = null;
			tmp34_AST = astFactory.create((AST)_t);
			tmp34_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp34_AST);
			ASTPair __currentAST1930 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,SUBWFFSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									v1 = sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					v1 = str2int( b.getText());
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				c = (AST)_t;
				AST c_AST_in = null;
				c_AST = astFactory.create(c);
				astFactory.addASTChild(currentAST, c_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
										v2 = sTable.getSymbolValue(c.getText());
									
				}
				break;
			}
			case NUMBER:
			{
				d = (AST)_t;
				AST d_AST_in = null;
				d_AST = astFactory.create(d);
				astFactory.addASTChild(currentAST, d_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					v2 = str2int(d.getText());
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST1930;
			_t = __t1930;
			_t = _t.getNextSibling();
			if ( inputState.guessing==0 ) {
				
					 	s += " movf "  + v2 + ", 0 " + "\n";
					 	s += " subwf " + v1 + ", 0 ";
					
			}
			subwffStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = subwffStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  swapffStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST swapffStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST swapffStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		AST c = null;
		AST c_AST = null;
		AST d = null;
		AST d_AST = null;
		
			int v1=0;
			int v2=0;	
			int temp=1; // temp yerine ne koyulacak?
						// temp diye bir adres ayirmak lazim
		
		
		try {      // for error handling
			AST __t1934 = _t;
			AST tmp35_AST = null;
			AST tmp35_AST_in = null;
			tmp35_AST = astFactory.create((AST)_t);
			tmp35_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp35_AST);
			ASTPair __currentAST1934 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,SWAPFFSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									v1 = sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					v1 = str2int( b.getText());
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				c = (AST)_t;
				AST c_AST_in = null;
				c_AST = astFactory.create(c);
				astFactory.addASTChild(currentAST, c_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
										v2 = sTable.getSymbolValue(c.getText());
									
				}
				break;
			}
			case NUMBER:
			{
				d = (AST)_t;
				AST d_AST_in = null;
				d_AST = astFactory.create(d);
				astFactory.addASTChild(currentAST, d_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					v2 = str2int(d.getText());
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST1934;
			_t = __t1934;
			_t = _t.getNextSibling();
			if ( inputState.guessing==0 ) {
				
					 	s += " movf "  + v1 + ", 0 " + "\n";
					 	s += " movwf " +temp + "\n";
					 	s += " movf "  + v2 + ", 0 " + "\n";
					 	s += " movwf " + v1 + "\n";
					 	s += " movf "  +temp+ ", 0" + "\n"; 
					 	s += " movwf " + v2;
					
			}
			swapffStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = swapffStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  iorwffStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST iorwffStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST iorwffStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		AST c = null;
		AST c_AST = null;
		AST d = null;
		AST d_AST = null;
		
			int v1=0;
			int v2=0;	
		
		
		try {      // for error handling
			AST __t1938 = _t;
			AST tmp36_AST = null;
			AST tmp36_AST_in = null;
			tmp36_AST = astFactory.create((AST)_t);
			tmp36_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp36_AST);
			ASTPair __currentAST1938 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,IORWFFSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									v1 = sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					v1 = str2int( b.getText());
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				c = (AST)_t;
				AST c_AST_in = null;
				c_AST = astFactory.create(c);
				astFactory.addASTChild(currentAST, c_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
										v2 = sTable.getSymbolValue(c.getText());
									
				}
				break;
			}
			case NUMBER:
			{
				d = (AST)_t;
				AST d_AST_in = null;
				d_AST = astFactory.create(d);
				astFactory.addASTChild(currentAST, d_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					v2 = str2int(d.getText());
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST1938;
			_t = __t1938;
			_t = _t.getNextSibling();
			if ( inputState.guessing==0 ) {
				
					 	s += " movf "  + v2 + ", 0 " + "\n";
					 	s += " iorwf " + v1 + ", 1 ";
					
			}
			iorwffStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = iorwffStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  andwffStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST andwffStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST andwffStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		AST c = null;
		AST c_AST = null;
		AST d = null;
		AST d_AST = null;
		
			int v1=0;
			int v2=0;
		
		
		try {      // for error handling
			AST __t1942 = _t;
			AST tmp37_AST = null;
			AST tmp37_AST_in = null;
			tmp37_AST = astFactory.create((AST)_t);
			tmp37_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp37_AST);
			ASTPair __currentAST1942 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,ANDWFFSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									v1 = sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					v1 = str2int( b.getText());
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				c = (AST)_t;
				AST c_AST_in = null;
				c_AST = astFactory.create(c);
				astFactory.addASTChild(currentAST, c_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
										v2 = sTable.getSymbolValue(c.getText());
									
				}
				break;
			}
			case NUMBER:
			{
				d = (AST)_t;
				AST d_AST_in = null;
				d_AST = astFactory.create(d);
				astFactory.addASTChild(currentAST, d_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					v2 = str2int(d.getText());
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST1942;
			_t = __t1942;
			_t = _t.getNextSibling();
			if ( inputState.guessing==0 ) {
				
					 	s += " movf "  + v2 + ", 0 " + "\n";
					 	s += " andwf " + v1 + ", 1 ";
					
			}
			andwffStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = andwffStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  xorwffStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST xorwffStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST xorwffStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		AST c = null;
		AST c_AST = null;
		AST d = null;
		AST d_AST = null;
		
			int v1=0;
			int v2=0;
		
		
		try {      // for error handling
			AST __t1946 = _t;
			AST tmp38_AST = null;
			AST tmp38_AST_in = null;
			tmp38_AST = astFactory.create((AST)_t);
			tmp38_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp38_AST);
			ASTPair __currentAST1946 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,XORWFFSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									v1 = sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					v1 = str2int( b.getText());
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				c = (AST)_t;
				AST c_AST_in = null;
				c_AST = astFactory.create(c);
				astFactory.addASTChild(currentAST, c_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
										v2 = sTable.getSymbolValue(c.getText());
									
				}
				break;
			}
			case NUMBER:
			{
				d = (AST)_t;
				AST d_AST_in = null;
				d_AST = astFactory.create(d);
				astFactory.addASTChild(currentAST, d_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					v2 = str2int(d.getText());
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST1946;
			_t = __t1946;
			_t = _t.getNextSibling();
			if ( inputState.guessing==0 ) {
				
					 	s += " movf "  + v2 + ", 0 " + "\n";
					 	s += " xorwf " + v1 + ", 1 ";
					
			}
			xorwffStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = xorwffStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  movffStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST movffStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST movffStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		AST c = null;
		AST c_AST = null;
		AST d = null;
		AST d_AST = null;
		
			int v1=0;
			int v2=0;
		
		
		try {      // for error handling
			AST __t1950 = _t;
			AST tmp39_AST = null;
			AST tmp39_AST_in = null;
			tmp39_AST = astFactory.create((AST)_t);
			tmp39_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp39_AST);
			ASTPair __currentAST1950 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,MOVFFSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									v1 = sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					v1 = str2int( b.getText());
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				c = (AST)_t;
				AST c_AST_in = null;
				c_AST = astFactory.create(c);
				astFactory.addASTChild(currentAST, c_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
										v2 = sTable.getSymbolValue(c.getText());
									
				}
				break;
			}
			case NUMBER:
			{
				d = (AST)_t;
				AST d_AST_in = null;
				d_AST = astFactory.create(d);
				astFactory.addASTChild(currentAST, d_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					v2 = str2int(d.getText());
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST1950;
			_t = __t1950;
			_t = _t.getNextSibling();
			if ( inputState.guessing==0 ) {
				
					 	s += " movf "  + v1 + ", 0 " + "\n";
					 	s += " movwf " + v2;
					
			}
			movffStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = movffStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  addwfStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST addwfStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST addwfStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		AST c = null;
		AST c_AST = null;
		AST d = null;
		AST d_AST = null;
		
		
		
		try {      // for error handling
			AST __t1957 = _t;
			AST tmp40_AST = null;
			AST tmp40_AST_in = null;
			tmp40_AST = astFactory.create((AST)_t);
			tmp40_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp40_AST);
			ASTPair __currentAST1957 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,ADDWFSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " addwf ";
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
						// ident=(constant or variable), doesn't matter => both mean an address
									s += sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += b.getText() + " ";
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				c = (AST)_t;
				AST c_AST_in = null;
				c_AST = astFactory.create(c);
				astFactory.addASTChild(currentAST, c_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					// ident=(constant only, 0 or 1)
										if(sTable.getSymbolType(c.getText()) == 1)
											s += ", " + sTable.getSymbolValue(c.getText());
										else //variable
										{
											System.err.println("Error: Only constants are allowed as destination identifier");
											return "ERROR";
										}						
									
				}
				break;
			}
			case NUMBER:
			{
				d = (AST)_t;
				AST d_AST_in = null;
				d_AST = astFactory.create(d);
				astFactory.addASTChild(currentAST, d_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += ", " + d.getText() + " ";
				}
				break;
			}
			case 3:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST1957;
			_t = __t1957;
			_t = _t.getNextSibling();
			addwfStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = addwfStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  andwfStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST andwfStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST andwfStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		AST c = null;
		AST c_AST = null;
		AST d = null;
		AST d_AST = null;
		
		
		
		try {      // for error handling
			AST __t1961 = _t;
			AST tmp41_AST = null;
			AST tmp41_AST_in = null;
			tmp41_AST = astFactory.create((AST)_t);
			tmp41_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp41_AST);
			ASTPair __currentAST1961 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,ANDWFSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " andwf ";
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									s += sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += b.getText() + " ";
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				c = (AST)_t;
				AST c_AST_in = null;
				c_AST = astFactory.create(c);
				astFactory.addASTChild(currentAST, c_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
										// ident=(constant only, 0 or 1)
										if(sTable.getSymbolType(c.getText()) == 1)
											s += ", " + sTable.getSymbolValue(c.getText());
										else //variable
										{
											System.err.println("Error: Only constants are allowed as destination identifier");
											return "ERROR";
										}
									
				}
				break;
			}
			case NUMBER:
			{
				d = (AST)_t;
				AST d_AST_in = null;
				d_AST = astFactory.create(d);
				astFactory.addASTChild(currentAST, d_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += ", " + d.getText() + " ";
				}
				break;
			}
			case 3:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST1961;
			_t = __t1961;
			_t = _t.getNextSibling();
			andwfStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = andwfStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  clrfStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST clrfStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST clrfStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		
		
		
		try {      // for error handling
			AST __t1965 = _t;
			AST tmp42_AST = null;
			AST tmp42_AST_in = null;
			tmp42_AST = astFactory.create((AST)_t);
			tmp42_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp42_AST);
			ASTPair __currentAST1965 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,CLRFSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " clrf ";
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									// constant only
									s += sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += b.getText() + " ";
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST1965;
			_t = __t1965;
			_t = _t.getNextSibling();
			clrfStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = clrfStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  clrwStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST clrwStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST clrwStatement_AST = null;
		
		
		
		try {      // for error handling
			AST __t1968 = _t;
			AST tmp43_AST = null;
			AST tmp43_AST_in = null;
			tmp43_AST = astFactory.create((AST)_t);
			tmp43_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp43_AST);
			ASTPair __currentAST1968 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,CLRWSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " clrw ";
			}
			currentAST = __currentAST1968;
			_t = __t1968;
			_t = _t.getNextSibling();
			clrwStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = clrwStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  comfStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST comfStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST comfStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		
		
		
		try {      // for error handling
			AST __t1970 = _t;
			AST tmp44_AST = null;
			AST tmp44_AST_in = null;
			tmp44_AST = astFactory.create((AST)_t);
			tmp44_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp44_AST);
			ASTPair __currentAST1970 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,COMFSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " comf ";
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									s += sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += b.getText() + " ";
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST1970;
			_t = __t1970;
			_t = _t.getNextSibling();
			comfStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = comfStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  decfStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST decfStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST decfStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		
		
		
		try {      // for error handling
			AST __t1973 = _t;
			AST tmp45_AST = null;
			AST tmp45_AST_in = null;
			tmp45_AST = astFactory.create((AST)_t);
			tmp45_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp45_AST);
			ASTPair __currentAST1973 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,DECFSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " decf ";
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									s += sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += b.getText() + " ";
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST1973;
			_t = __t1973;
			_t = _t.getNextSibling();
			decfStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = decfStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  decfszStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST decfszStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST decfszStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		AST c = null;
		AST c_AST = null;
		AST d = null;
		AST d_AST = null;
		
		
		
		try {      // for error handling
			AST __t1976 = _t;
			AST tmp46_AST = null;
			AST tmp46_AST_in = null;
			tmp46_AST = astFactory.create((AST)_t);
			tmp46_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp46_AST);
			ASTPair __currentAST1976 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,DECFSZSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " decfsz ";
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									s += sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += b.getText() + " ";
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				c = (AST)_t;
				AST c_AST_in = null;
				c_AST = astFactory.create(c);
				astFactory.addASTChild(currentAST, c_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
										// ident=(constant only, 0 or 1)
										if(sTable.getSymbolType(c.getText()) == 1)
											s += ", " + sTable.getSymbolValue(c.getText());
										else //variable
										{
											System.err.println("Error: Only constants are allowed as destination identifier");
											return "ERROR";
										}
									
				}
				break;
			}
			case NUMBER:
			{
				d = (AST)_t;
				AST d_AST_in = null;
				d_AST = astFactory.create(d);
				astFactory.addASTChild(currentAST, d_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += ", " + d.getText() + " ";
				}
				break;
			}
			case 3:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST1976;
			_t = __t1976;
			_t = _t.getNextSibling();
			decfszStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = decfszStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  incfStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST incfStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST incfStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		
		
		
		try {      // for error handling
			AST __t1980 = _t;
			AST tmp47_AST = null;
			AST tmp47_AST_in = null;
			tmp47_AST = astFactory.create((AST)_t);
			tmp47_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp47_AST);
			ASTPair __currentAST1980 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,INCFSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " incf ";
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									s += sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += b.getText() + " ";
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST1980;
			_t = __t1980;
			_t = _t.getNextSibling();
			incfStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = incfStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  incfszStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST incfszStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST incfszStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		AST c = null;
		AST c_AST = null;
		AST d = null;
		AST d_AST = null;
		
		
		
		try {      // for error handling
			AST __t1983 = _t;
			AST tmp48_AST = null;
			AST tmp48_AST_in = null;
			tmp48_AST = astFactory.create((AST)_t);
			tmp48_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp48_AST);
			ASTPair __currentAST1983 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,INCFSZSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " incfsz ";
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									s += sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += b.getText() + " ";
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				c = (AST)_t;
				AST c_AST_in = null;
				c_AST = astFactory.create(c);
				astFactory.addASTChild(currentAST, c_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
										// ident=(constant only, 0 or 1)
										if(sTable.getSymbolType(c.getText()) == 1)
											s += ", " + sTable.getSymbolValue(c.getText());
										else //variable
										{
											System.err.println("Error: Only constants are allowed as destination identifier");
											return "ERROR";
										}
									
				}
				break;
			}
			case NUMBER:
			{
				d = (AST)_t;
				AST d_AST_in = null;
				d_AST = astFactory.create(d);
				astFactory.addASTChild(currentAST, d_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += ", " + d.getText() + " ";
				}
				break;
			}
			case 3:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST1983;
			_t = __t1983;
			_t = _t.getNextSibling();
			incfszStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = incfszStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  iorwfStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST iorwfStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST iorwfStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		AST c = null;
		AST c_AST = null;
		AST d = null;
		AST d_AST = null;
		
		
		
		try {      // for error handling
			AST __t1987 = _t;
			AST tmp49_AST = null;
			AST tmp49_AST_in = null;
			tmp49_AST = astFactory.create((AST)_t);
			tmp49_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp49_AST);
			ASTPair __currentAST1987 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,IORWFSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " iorwf ";
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									s += sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += b.getText() + " ";
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				c = (AST)_t;
				AST c_AST_in = null;
				c_AST = astFactory.create(c);
				astFactory.addASTChild(currentAST, c_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
										// ident=(constant only, 0 or 1)
										if(sTable.getSymbolType(c.getText()) == 1)
											s += ", " + sTable.getSymbolValue(c.getText());
										else //variable
										{
											System.err.println("Error: Only constants are allowed as destination identifier");
											return "ERROR";
										}
									
				}
				break;
			}
			case NUMBER:
			{
				d = (AST)_t;
				AST d_AST_in = null;
				d_AST = astFactory.create(d);
				astFactory.addASTChild(currentAST, d_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += ", " + d.getText() + " ";
				}
				break;
			}
			case 3:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST1987;
			_t = __t1987;
			_t = _t.getNextSibling();
			iorwfStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = iorwfStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  movfStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST movfStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST movfStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		AST c = null;
		AST c_AST = null;
		AST d = null;
		AST d_AST = null;
		
		
		
		try {      // for error handling
			AST __t1991 = _t;
			AST tmp50_AST = null;
			AST tmp50_AST_in = null;
			tmp50_AST = astFactory.create((AST)_t);
			tmp50_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp50_AST);
			ASTPair __currentAST1991 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,MOVFSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " movf ";
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									s += sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += b.getText() + " ";
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				c = (AST)_t;
				AST c_AST_in = null;
				c_AST = astFactory.create(c);
				astFactory.addASTChild(currentAST, c_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
										// ident=(constant only, 0 or 1)
										if(sTable.getSymbolType(c.getText()) == 1)
											s += ", " + sTable.getSymbolValue(c.getText());
										else //variable
										{
											System.err.println("Error: Only constants are allowed as destination identifier");
											return "ERROR";
										}
									
				}
				break;
			}
			case NUMBER:
			{
				d = (AST)_t;
				AST d_AST_in = null;
				d_AST = astFactory.create(d);
				astFactory.addASTChild(currentAST, d_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += ", " + d.getText() + " ";
				}
				break;
			}
			case 3:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST1991;
			_t = __t1991;
			_t = _t.getNextSibling();
			movfStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = movfStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  movwfStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST movwfStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST movwfStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		
		
		
		try {      // for error handling
			AST __t1995 = _t;
			AST tmp51_AST = null;
			AST tmp51_AST_in = null;
			tmp51_AST = astFactory.create((AST)_t);
			tmp51_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp51_AST);
			ASTPair __currentAST1995 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,MOVWFSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " movwf ";
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									s += sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += b.getText() + " ";
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST1995;
			_t = __t1995;
			_t = _t.getNextSibling();
			movwfStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = movwfStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  nopStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST nopStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST nopStatement_AST = null;
		
		
		
		try {      // for error handling
			AST __t1998 = _t;
			AST tmp52_AST = null;
			AST tmp52_AST_in = null;
			tmp52_AST = astFactory.create((AST)_t);
			tmp52_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp52_AST);
			ASTPair __currentAST1998 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,NOPSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " nop ";
			}
			currentAST = __currentAST1998;
			_t = __t1998;
			_t = _t.getNextSibling();
			nopStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = nopStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  rlfStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST rlfStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST rlfStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		AST c = null;
		AST c_AST = null;
		AST d = null;
		AST d_AST = null;
		
		
		
		try {      // for error handling
			AST __t2000 = _t;
			AST tmp53_AST = null;
			AST tmp53_AST_in = null;
			tmp53_AST = astFactory.create((AST)_t);
			tmp53_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp53_AST);
			ASTPair __currentAST2000 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,RLFSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " rlf ";
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									s += sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += b.getText() + " ";
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				c = (AST)_t;
				AST c_AST_in = null;
				c_AST = astFactory.create(c);
				astFactory.addASTChild(currentAST, c_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
										// ident=(constant only, 0 or 1)
										if(sTable.getSymbolType(c.getText()) == 1)
											s += ", " + sTable.getSymbolValue(c.getText());
										else //variable
										{
											System.err.println("Error: Only constants are allowed as destination identifier");
											return "ERROR";
										}
									
				}
				break;
			}
			case NUMBER:
			{
				d = (AST)_t;
				AST d_AST_in = null;
				d_AST = astFactory.create(d);
				astFactory.addASTChild(currentAST, d_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += ", " + d.getText() + " ";
				}
				break;
			}
			case 3:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST2000;
			_t = __t2000;
			_t = _t.getNextSibling();
			rlfStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = rlfStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  rrfStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST rrfStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST rrfStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		AST c = null;
		AST c_AST = null;
		AST d = null;
		AST d_AST = null;
		
		
		
		try {      // for error handling
			AST __t2004 = _t;
			AST tmp54_AST = null;
			AST tmp54_AST_in = null;
			tmp54_AST = astFactory.create((AST)_t);
			tmp54_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp54_AST);
			ASTPair __currentAST2004 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,RRFSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " rrf ";
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									s += sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += b.getText() + " ";
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				c = (AST)_t;
				AST c_AST_in = null;
				c_AST = astFactory.create(c);
				astFactory.addASTChild(currentAST, c_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
										// ident=(constant only, 0 or 1)
										if(sTable.getSymbolType(c.getText()) == 1)
											s += ", " + sTable.getSymbolValue(c.getText());
										else //variable
										{
											System.err.println("Error: Only constants are allowed as destination identifier");
											return "ERROR";
										}
									
				}
				break;
			}
			case NUMBER:
			{
				d = (AST)_t;
				AST d_AST_in = null;
				d_AST = astFactory.create(d);
				astFactory.addASTChild(currentAST, d_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += ", " + d.getText() + " ";
				}
				break;
			}
			case 3:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST2004;
			_t = __t2004;
			_t = _t.getNextSibling();
			rrfStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = rrfStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  subwfStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST subwfStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST subwfStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		AST c = null;
		AST c_AST = null;
		AST d = null;
		AST d_AST = null;
		
		
		
		try {      // for error handling
			AST __t2008 = _t;
			AST tmp55_AST = null;
			AST tmp55_AST_in = null;
			tmp55_AST = astFactory.create((AST)_t);
			tmp55_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp55_AST);
			ASTPair __currentAST2008 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,SUBWFSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " subwf ";
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									s += sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += b.getText() + " ";
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				c = (AST)_t;
				AST c_AST_in = null;
				c_AST = astFactory.create(c);
				astFactory.addASTChild(currentAST, c_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
										// ident=(constant only, 0 or 1)
										if(sTable.getSymbolType(c.getText()) == 1)
											s += ", " + sTable.getSymbolValue(c.getText());
										else //variable
										{
											System.err.println("Error: Only constants are allowed as destination identifier");
											return "ERROR";
										}
									
				}
				break;
			}
			case NUMBER:
			{
				d = (AST)_t;
				AST d_AST_in = null;
				d_AST = astFactory.create(d);
				astFactory.addASTChild(currentAST, d_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += ", " + d.getText() + " ";
				}
				break;
			}
			case 3:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST2008;
			_t = __t2008;
			_t = _t.getNextSibling();
			subwfStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = subwfStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  swapfStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST swapfStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST swapfStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		AST c = null;
		AST c_AST = null;
		AST d = null;
		AST d_AST = null;
		
		
		
		try {      // for error handling
			AST __t2012 = _t;
			AST tmp56_AST = null;
			AST tmp56_AST_in = null;
			tmp56_AST = astFactory.create((AST)_t);
			tmp56_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp56_AST);
			ASTPair __currentAST2012 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,SWAPFSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " swapf ";
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									s += sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += b.getText() + " ";
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				c = (AST)_t;
				AST c_AST_in = null;
				c_AST = astFactory.create(c);
				astFactory.addASTChild(currentAST, c_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
										// ident=(constant only, 0 or 1)
										if(sTable.getSymbolType(c.getText()) == 1)
											s += ", " + sTable.getSymbolValue(c.getText());
										else //variable
										{
											System.err.println("Error: Only constants are allowed as destination identifier");
											return "ERROR";
										}
									
				}
				break;
			}
			case NUMBER:
			{
				d = (AST)_t;
				AST d_AST_in = null;
				d_AST = astFactory.create(d);
				astFactory.addASTChild(currentAST, d_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += ", " + d.getText() + " ";
				}
				break;
			}
			case 3:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST2012;
			_t = __t2012;
			_t = _t.getNextSibling();
			swapfStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = swapfStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  xorwfStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST xorwfStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST xorwfStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		AST c = null;
		AST c_AST = null;
		AST d = null;
		AST d_AST = null;
		
		
		
		try {      // for error handling
			AST __t2016 = _t;
			AST tmp57_AST = null;
			AST tmp57_AST_in = null;
			tmp57_AST = astFactory.create((AST)_t);
			tmp57_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp57_AST);
			ASTPair __currentAST2016 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,XORWFSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " xorwf ";
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									s += sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += b.getText() + " ";
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				c = (AST)_t;
				AST c_AST_in = null;
				c_AST = astFactory.create(c);
				astFactory.addASTChild(currentAST, c_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
										// ident=(constant only, 0 or 1)
										if(sTable.getSymbolType(c.getText()) == 1)
											s += ", " + sTable.getSymbolValue(c.getText());
										else //variable
										{
											System.err.println("Error: Only constants are allowed as destination identifier");
											return "ERROR";
										}
									
				}
				break;
			}
			case NUMBER:
			{
				d = (AST)_t;
				AST d_AST_in = null;
				d_AST = astFactory.create(d);
				astFactory.addASTChild(currentAST, d_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += ", " + d.getText() + " ";
				}
				break;
			}
			case 3:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST2016;
			_t = __t2016;
			_t = _t.getNextSibling();
			xorwfStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = xorwfStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  bcfStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST bcfStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST bcfStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		AST c = null;
		AST c_AST = null;
		AST d = null;
		AST d_AST = null;
		
		
		
		try {      // for error handling
			AST __t2020 = _t;
			AST tmp58_AST = null;
			AST tmp58_AST_in = null;
			tmp58_AST = astFactory.create((AST)_t);
			tmp58_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp58_AST);
			ASTPair __currentAST2020 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,BCFSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " bcf ";
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									s += sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += b.getText() + " ";
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				c = (AST)_t;
				AST c_AST_in = null;
				c_AST = astFactory.create(c);
				astFactory.addASTChild(currentAST, c_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
										// ident=(constant only, 0 or 1)
										if(sTable.getSymbolType(c.getText()) == 1)
											s += ", " + sTable.getSymbolValue(c.getText());
										else //variable
										{
											System.err.println("Error: Only constants are allowed as destination identifier");
											return "ERROR";
										}
									
				}
				break;
			}
			case NUMBER:
			{
				d = (AST)_t;
				AST d_AST_in = null;
				d_AST = astFactory.create(d);
				astFactory.addASTChild(currentAST, d_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += ", " + d.getText() + " ";
				}
				break;
			}
			case 3:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST2020;
			_t = __t2020;
			_t = _t.getNextSibling();
			bcfStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = bcfStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  bsfStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST bsfStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST bsfStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		AST c = null;
		AST c_AST = null;
		AST d = null;
		AST d_AST = null;
		
		
		
		try {      // for error handling
			AST __t2024 = _t;
			AST tmp59_AST = null;
			AST tmp59_AST_in = null;
			tmp59_AST = astFactory.create((AST)_t);
			tmp59_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp59_AST);
			ASTPair __currentAST2024 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,BSFSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " bsf ";
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									s += sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += b.getText() + " ";
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				c = (AST)_t;
				AST c_AST_in = null;
				c_AST = astFactory.create(c);
				astFactory.addASTChild(currentAST, c_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
										// ident=(constant only, 0 or 1)
										if(sTable.getSymbolType(c.getText()) == 1)
											s += ", " + sTable.getSymbolValue(c.getText());
										else //variable
										{
											System.err.println("Error: Only constants are allowed as destination identifier");
											return "ERROR";
										}
									
				}
				break;
			}
			case NUMBER:
			{
				d = (AST)_t;
				AST d_AST_in = null;
				d_AST = astFactory.create(d);
				astFactory.addASTChild(currentAST, d_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += ", " + d.getText() + " ";
				}
				break;
			}
			case 3:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST2024;
			_t = __t2024;
			_t = _t.getNextSibling();
			bsfStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = bsfStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  btfscStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST btfscStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST btfscStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		AST c = null;
		AST c_AST = null;
		AST d = null;
		AST d_AST = null;
		
		
		
		try {      // for error handling
			AST __t2028 = _t;
			AST tmp60_AST = null;
			AST tmp60_AST_in = null;
			tmp60_AST = astFactory.create((AST)_t);
			tmp60_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp60_AST);
			ASTPair __currentAST2028 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,BTFSCSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " btfsc ";
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									s += sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += b.getText() + " ";
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				c = (AST)_t;
				AST c_AST_in = null;
				c_AST = astFactory.create(c);
				astFactory.addASTChild(currentAST, c_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
										// ident=(constant only, 0 or 1)
										if(sTable.getSymbolType(c.getText()) == 1)
											s += ", " + sTable.getSymbolValue(c.getText());
										else //variable
										{
											System.err.println("Error: Only constants are allowed as destination identifier");
											return "ERROR";
										}
									
				}
				break;
			}
			case NUMBER:
			{
				d = (AST)_t;
				AST d_AST_in = null;
				d_AST = astFactory.create(d);
				astFactory.addASTChild(currentAST, d_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += ", " + d.getText() + " ";
				}
				break;
			}
			case 3:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST2028;
			_t = __t2028;
			_t = _t.getNextSibling();
			btfscStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = btfscStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  btfssStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST btfssStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST btfssStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		AST c = null;
		AST c_AST = null;
		AST d = null;
		AST d_AST = null;
		
		
		
		try {      // for error handling
			AST __t2032 = _t;
			AST tmp61_AST = null;
			AST tmp61_AST_in = null;
			tmp61_AST = astFactory.create((AST)_t);
			tmp61_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp61_AST);
			ASTPair __currentAST2032 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,BTFSSSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " btfss ";
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									s += sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += b.getText() + " ";
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				c = (AST)_t;
				AST c_AST_in = null;
				c_AST = astFactory.create(c);
				astFactory.addASTChild(currentAST, c_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
										// ident=(constant only, 0 or 1)
										if(sTable.getSymbolType(c.getText()) == 1)
											s += ", " + sTable.getSymbolValue(c.getText());
										else //variable
										{
											System.err.println("Error: Only constants are allowed as destination identifier");
											return "ERROR";
										}
									
				}
				break;
			}
			case NUMBER:
			{
				d = (AST)_t;
				AST d_AST_in = null;
				d_AST = astFactory.create(d);
				astFactory.addASTChild(currentAST, d_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += ", " + d.getText() + " ";
				}
				break;
			}
			case 3:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST2032;
			_t = __t2032;
			_t = _t.getNextSibling();
			btfssStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = btfssStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  addlwStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST addlwStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST addlwStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		
		
		
		try {      // for error handling
			AST __t2036 = _t;
			AST tmp62_AST = null;
			AST tmp62_AST_in = null;
			tmp62_AST = astFactory.create((AST)_t);
			tmp62_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp62_AST);
			ASTPair __currentAST2036 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,ADDLWSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " addlw ";
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									s += sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += b.getText() + " ";
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST2036;
			_t = __t2036;
			_t = _t.getNextSibling();
			addlwStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = addlwStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  andlwStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST andlwStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST andlwStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		
		
		
		try {      // for error handling
			AST __t2039 = _t;
			AST tmp63_AST = null;
			AST tmp63_AST_in = null;
			tmp63_AST = astFactory.create((AST)_t);
			tmp63_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp63_AST);
			ASTPair __currentAST2039 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,ANDLWSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " andlw ";
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									s += sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += b.getText() + " ";
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST2039;
			_t = __t2039;
			_t = _t.getNextSibling();
			andlwStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = andlwStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  callStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST callStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST callStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		
		
		
		try {      // for error handling
			AST __t2042 = _t;
			AST tmp64_AST = null;
			AST tmp64_AST_in = null;
			tmp64_AST = astFactory.create((AST)_t);
			tmp64_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp64_AST);
			ASTPair __currentAST2042 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,CALLSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " call ";
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									s += sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += b.getText() + " ";
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST2042;
			_t = __t2042;
			_t = _t.getNextSibling();
			callStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = callStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  clrwdtStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST clrwdtStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST clrwdtStatement_AST = null;
		
		
		
		try {      // for error handling
			AST __t2045 = _t;
			AST tmp65_AST = null;
			AST tmp65_AST_in = null;
			tmp65_AST = astFactory.create((AST)_t);
			tmp65_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp65_AST);
			ASTPair __currentAST2045 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,CLRWDTSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " clrwdt ";
			}
			currentAST = __currentAST2045;
			_t = __t2045;
			_t = _t.getNextSibling();
			clrwdtStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = clrwdtStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  gotoStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST gotoStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST gotoStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		
		
		
		try {      // for error handling
			AST __t2047 = _t;
			AST tmp66_AST = null;
			AST tmp66_AST_in = null;
			tmp66_AST = astFactory.create((AST)_t);
			tmp66_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp66_AST);
			ASTPair __currentAST2047 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,GOTOSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " goto ";
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									s += sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += b.getText() + " ";
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST2047;
			_t = __t2047;
			_t = _t.getNextSibling();
			gotoStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = gotoStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  iorlwStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST iorlwStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST iorlwStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		
		
		
		try {      // for error handling
			AST __t2050 = _t;
			AST tmp67_AST = null;
			AST tmp67_AST_in = null;
			tmp67_AST = astFactory.create((AST)_t);
			tmp67_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp67_AST);
			ASTPair __currentAST2050 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,IORLWSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " iorlw ";
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									s += sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += b.getText() + " ";
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST2050;
			_t = __t2050;
			_t = _t.getNextSibling();
			iorlwStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = iorlwStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  movlwStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST movlwStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST movlwStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		
		
		
		try {      // for error handling
			AST __t2053 = _t;
			AST tmp68_AST = null;
			AST tmp68_AST_in = null;
			tmp68_AST = astFactory.create((AST)_t);
			tmp68_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp68_AST);
			ASTPair __currentAST2053 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,MOVLWSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " movlw ";
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									s += sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += b.getText() + " ";
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST2053;
			_t = __t2053;
			_t = _t.getNextSibling();
			movlwStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = movlwStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  retfieStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST retfieStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST retfieStatement_AST = null;
		
		
		
		try {      // for error handling
			AST __t2056 = _t;
			AST tmp69_AST = null;
			AST tmp69_AST_in = null;
			tmp69_AST = astFactory.create((AST)_t);
			tmp69_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp69_AST);
			ASTPair __currentAST2056 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,RETFIESTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " retfie ";
			}
			currentAST = __currentAST2056;
			_t = __t2056;
			_t = _t.getNextSibling();
			retfieStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = retfieStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  retlwStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST retlwStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST retlwStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		
		
		
		try {      // for error handling
			AST __t2058 = _t;
			AST tmp70_AST = null;
			AST tmp70_AST_in = null;
			tmp70_AST = astFactory.create((AST)_t);
			tmp70_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp70_AST);
			ASTPair __currentAST2058 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,RETLWSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " retlw ";
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									s += sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += b.getText() + " ";
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST2058;
			_t = __t2058;
			_t = _t.getNextSibling();
			retlwStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = retlwStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  sleepStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST sleepStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST sleepStatement_AST = null;
		
		
		
		try {      // for error handling
			AST __t2061 = _t;
			AST tmp71_AST = null;
			AST tmp71_AST_in = null;
			tmp71_AST = astFactory.create((AST)_t);
			tmp71_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp71_AST);
			ASTPair __currentAST2061 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,SLEEPSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " sleep ";
			}
			currentAST = __currentAST2061;
			_t = __t2061;
			_t = _t.getNextSibling();
			sleepStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = sleepStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  sublwStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST sublwStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST sublwStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		
		
		
		try {      // for error handling
			AST __t2063 = _t;
			AST tmp72_AST = null;
			AST tmp72_AST_in = null;
			tmp72_AST = astFactory.create((AST)_t);
			tmp72_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp72_AST);
			ASTPair __currentAST2063 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,SUBLWSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " sublw ";
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									s += sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += b.getText() + " ";
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST2063;
			_t = __t2063;
			_t = _t.getNextSibling();
			sublwStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = sublwStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  xorlwStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST xorlwStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST xorlwStatement_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		
		
		
		try {      // for error handling
			AST __t2066 = _t;
			AST tmp73_AST = null;
			AST tmp73_AST_in = null;
			tmp73_AST = astFactory.create((AST)_t);
			tmp73_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp73_AST);
			ASTPair __currentAST2066 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,XORLWSTATEMENT);
			_t = _t.getFirstChild();
			if ( inputState.guessing==0 ) {
				s += " xorlw ";
			}
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case IDENT:
			{
				a = (AST)_t;
				AST a_AST_in = null;
				a_AST = astFactory.create(a);
				astFactory.addASTChild(currentAST, a_AST);
				match(_t,IDENT);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					
									s += sTable.getSymbolValue(a.getText());
								
				}
				break;
			}
			case NUMBER:
			{
				b = (AST)_t;
				AST b_AST_in = null;
				b_AST = astFactory.create(b);
				astFactory.addASTChild(currentAST, b_AST);
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				if ( inputState.guessing==0 ) {
					s += b.getText() + " ";
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST2066;
			_t = __t2066;
			_t = _t.getNextSibling();
			xorlwStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = xorlwStatement_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  expr(AST _t) throws RecognitionException {
		String s="";
		
		AST expr_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST expr_AST = null;
		
			String s1="";
			String s2="";	
			
		
		
		try {      // for error handling
			AST __t2080 = _t;
			AST tmp74_AST = null;
			AST tmp74_AST_in = null;
			tmp74_AST = astFactory.create((AST)_t);
			tmp74_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp74_AST);
			ASTPair __currentAST2080 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,EXPR);
			_t = _t.getFirstChild();
			{
			{
			{
			s1=subexpr(_t);
			_t = _retTree;
			astFactory.addASTChild(currentAST, returnAST);
			}
			if ( inputState.guessing==0 ) {
				s += s1;				
			}
			}
			}
			currentAST = __currentAST2080;
			_t = __t2080;
			_t = _t.getNextSibling();
			expr_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = expr_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  subexpr(AST _t) throws RecognitionException {
		String s="";
		
		AST subexpr_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST subexpr_AST = null;
		AST a = null;
		AST a_AST = null;
		AST b = null;
		AST b_AST = null;
		AST c = null;
		AST c_AST = null;
		AST d = null;
		AST d_AST = null;
		AST e = null;
		AST e_AST = null;
		AST f = null;
		AST f_AST = null;
		AST k = null;
		AST k_AST = null;
		AST l = null;
		AST l_AST = null;
		AST m = null;
		AST m_AST = null;
		
			int opType = 0;
			String v1 = "";
			String v2 = "";
			int op1type = 0; //1=address 2=ident 3=number
			int op2type = 0;
		
		
		try {      // for error handling
			AST __t2085 = _t;
			AST tmp75_AST = null;
			AST tmp75_AST_in = null;
			tmp75_AST = astFactory.create((AST)_t);
			tmp75_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp75_AST);
			ASTPair __currentAST2085 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,SUBEXPR);
			_t = _t.getFirstChild();
			{
			boolean synPredMatched2091 = false;
			if (_t==null) _t=ASTNULL;
			if ((((_t.getType() >= IDENT && _t.getType() <= ADDRESS)))) {
				AST __t2091 = _t;
				synPredMatched2091 = true;
				inputState.guessing++;
				try {
					{
					{
					if (_t==null) _t=ASTNULL;
					switch ( _t.getType()) {
					case ADDRESS:
					{
						match(_t,ADDRESS);
						_t = _t.getNextSibling();
						break;
					}
					case NUMBER:
					{
						match(_t,NUMBER);
						_t = _t.getNextSibling();
						break;
					}
					case IDENT:
					{
						match(_t,IDENT);
						_t = _t.getNextSibling();
						break;
					}
					default:
					{
						throw new NoViableAltException(_t);
					}
					}
					}
					{
					if (_t==null) _t=ASTNULL;
					switch ( _t.getType()) {
					case EQ:
					{
						match(_t,EQ);
						_t = _t.getNextSibling();
						break;
					}
					case NEQ:
					{
						match(_t,NEQ);
						_t = _t.getNextSibling();
						break;
					}
					case LT:
					{
						match(_t,LT);
						_t = _t.getNextSibling();
						break;
					}
					case GT:
					{
						match(_t,GT);
						_t = _t.getNextSibling();
						break;
					}
					case LTE:
					{
						match(_t,LTE);
						_t = _t.getNextSibling();
						break;
					}
					case GTE:
					{
						match(_t,GTE);
						_t = _t.getNextSibling();
						break;
					}
					default:
					{
						throw new NoViableAltException(_t);
					}
					}
					}
					{
					if (_t==null) _t=ASTNULL;
					switch ( _t.getType()) {
					case ADDRESS:
					{
						match(_t,ADDRESS);
						_t = _t.getNextSibling();
						break;
					}
					case NUMBER:
					{
						match(_t,NUMBER);
						_t = _t.getNextSibling();
						break;
					}
					case IDENT:
					{
						match(_t,IDENT);
						_t = _t.getNextSibling();
						break;
					}
					default:
					{
						throw new NoViableAltException(_t);
					}
					}
					}
					}
				}
				catch (RecognitionException pe) {
					synPredMatched2091 = false;
				}
				_t = __t2091;
inputState.guessing--;
			}
			if ( synPredMatched2091 ) {
				{
				{
				if (_t==null) _t=ASTNULL;
				switch ( _t.getType()) {
				case ADDRESS:
				{
					a = (AST)_t;
					AST a_AST_in = null;
					a_AST = astFactory.create(a);
					astFactory.addASTChild(currentAST, a_AST);
					match(_t,ADDRESS);
					_t = _t.getNextSibling();
					if ( inputState.guessing==0 ) {
						op1type =1;
					}
					break;
				}
				case NUMBER:
				{
					b = (AST)_t;
					AST b_AST_in = null;
					b_AST = astFactory.create(b);
					astFactory.addASTChild(currentAST, b_AST);
					match(_t,NUMBER);
					_t = _t.getNextSibling();
					if ( inputState.guessing==0 ) {
						op1type =3;
					}
					break;
				}
				case IDENT:
				{
					c = (AST)_t;
					AST c_AST_in = null;
					c_AST = astFactory.create(c);
					astFactory.addASTChild(currentAST, c_AST);
					match(_t,IDENT);
					_t = _t.getNextSibling();
					if ( inputState.guessing==0 ) {
						op1type =2;
					}
					break;
				}
				default:
				{
					throw new NoViableAltException(_t);
				}
				}
				}
				{
				if (_t==null) _t=ASTNULL;
				switch ( _t.getType()) {
				case EQ:
				{
					AST tmp76_AST = null;
					AST tmp76_AST_in = null;
					tmp76_AST = astFactory.create((AST)_t);
					tmp76_AST_in = (AST)_t;
					astFactory.addASTChild(currentAST, tmp76_AST);
					match(_t,EQ);
					_t = _t.getNextSibling();
					if ( inputState.guessing==0 ) {
						opType=1;
					}
					break;
				}
				case NEQ:
				{
					AST tmp77_AST = null;
					AST tmp77_AST_in = null;
					tmp77_AST = astFactory.create((AST)_t);
					tmp77_AST_in = (AST)_t;
					astFactory.addASTChild(currentAST, tmp77_AST);
					match(_t,NEQ);
					_t = _t.getNextSibling();
					if ( inputState.guessing==0 ) {
						opType=2;
					}
					break;
				}
				case LT:
				{
					AST tmp78_AST = null;
					AST tmp78_AST_in = null;
					tmp78_AST = astFactory.create((AST)_t);
					tmp78_AST_in = (AST)_t;
					astFactory.addASTChild(currentAST, tmp78_AST);
					match(_t,LT);
					_t = _t.getNextSibling();
					if ( inputState.guessing==0 ) {
						opType=3;
					}
					break;
				}
				case GT:
				{
					AST tmp79_AST = null;
					AST tmp79_AST_in = null;
					tmp79_AST = astFactory.create((AST)_t);
					tmp79_AST_in = (AST)_t;
					astFactory.addASTChild(currentAST, tmp79_AST);
					match(_t,GT);
					_t = _t.getNextSibling();
					if ( inputState.guessing==0 ) {
						opType=4;
					}
					break;
				}
				case LTE:
				{
					AST tmp80_AST = null;
					AST tmp80_AST_in = null;
					tmp80_AST = astFactory.create((AST)_t);
					tmp80_AST_in = (AST)_t;
					astFactory.addASTChild(currentAST, tmp80_AST);
					match(_t,LTE);
					_t = _t.getNextSibling();
					if ( inputState.guessing==0 ) {
						opType=5;
					}
					break;
				}
				case GTE:
				{
					AST tmp81_AST = null;
					AST tmp81_AST_in = null;
					tmp81_AST = astFactory.create((AST)_t);
					tmp81_AST_in = (AST)_t;
					astFactory.addASTChild(currentAST, tmp81_AST);
					match(_t,GTE);
					_t = _t.getNextSibling();
					if ( inputState.guessing==0 ) {
						opType=6;
					}
					break;
				}
				default:
				{
					throw new NoViableAltException(_t);
				}
				}
				}
				{
				if (_t==null) _t=ASTNULL;
				switch ( _t.getType()) {
				case ADDRESS:
				{
					d = (AST)_t;
					AST d_AST_in = null;
					d_AST = astFactory.create(d);
					astFactory.addASTChild(currentAST, d_AST);
					match(_t,ADDRESS);
					_t = _t.getNextSibling();
					if ( inputState.guessing==0 ) {
						op2type =1;
					}
					break;
				}
				case NUMBER:
				{
					e = (AST)_t;
					AST e_AST_in = null;
					e_AST = astFactory.create(e);
					astFactory.addASTChild(currentAST, e_AST);
					match(_t,NUMBER);
					_t = _t.getNextSibling();
					if ( inputState.guessing==0 ) {
						op2type =3;
					}
					break;
				}
				case IDENT:
				{
					f = (AST)_t;
					AST f_AST_in = null;
					f_AST = astFactory.create(f);
					astFactory.addASTChild(currentAST, f_AST);
					match(_t,IDENT);
					_t = _t.getNextSibling();
					if ( inputState.guessing==0 ) {
						op2type =2;
					}
					break;
				}
				default:
				{
					throw new NoViableAltException(_t);
				}
				}
				}
				}
				if ( inputState.guessing==0 ) {
					
									 				switch(opType)
									 				{
									 				case 1: //EQ
									 					//s += "EQUAL op1:" + op1type +" op2:" + op2type + "\n";
									 					switch(op1type)
									 					{
									 					case 1: //address
									 						switch(op2type)
										 					{
										 					case 1: //address
										 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
										 						break;
										 					case 2: //ident
										 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
										 						break;
										 					case 3: //number
										 						s += " movlw "  + e.getText() + "\n";
										 						break;
										 					}
										 					s += " subwf " + sTable.getSymbolValue(a.getText()) + ", 0\n";	
									 						break;
									 					case 2: //ident
									 						switch(op2type)
										 					{
										 					case 1: //address
										 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
										 						break;
										 					case 2: //ident
										 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
										 						break;
										 					case 3: //number
										 						s += " movlw "  + e.getText() + "\n";
										 						break;
										 					}
										 					s += " sublw " + sTable.getSymbolValue(c.getText()) + "\n";	
									 						break;
									 					case 3: //number
									 						switch(op2type)
										 					{
										 					case 1: //address
										 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
										 						break;
										 					case 2: //ident
										 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
										 						break;
										 					case 3: //number
										 						s += " movlw "  + e.getText() + "\n";
										 						break;
										 					}
										 					s += " sublw " + b.getText() + "\n";	
									 						break;
									 					}
									 					
									 								 					
									 					s += " btfss STATUS, 2\n";
									 					s += " goto ELSE" + ifStack.peek() + "\n";
									 					break;
									 				case 2: //NEQ
									 					//s += "NOT EQUAL op1:" + op1type +" op2:" + op2type + "\n";
									 					switch(op1type)
									 					{
									 					case 1: //address
									 						switch(op2type)
										 					{
										 					case 1: //address
										 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
										 						break;
										 					case 2: //ident
										 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
										 						break;
										 					case 3: //number
										 						s += " movlw "  + e.getText() + "\n";
										 						break;
										 					}
										 					s += " subwf " + sTable.getSymbolValue(a.getText()) + ", 0\n";	
									 						break;
									 					case 2: //ident
									 						switch(op2type)
										 					{
										 					case 1: //address
										 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
										 						break;
										 					case 2: //ident
										 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
										 						break;
										 					case 3: //number
										 						s += " movlw "  + e.getText() + "\n";
										 						break;
										 					}
										 					s += " sublw " + sTable.getSymbolValue(c.getText()) + "\n";	
									 						break;
									 					case 3: //number
									 						switch(op2type)
										 					{
										 					case 1: //address
										 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
										 						break;
										 					case 2: //ident
										 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
										 						break;
										 					case 3: //number
										 						s += " movlw "  + e.getText() + "\n";
										 						break;
										 					}
										 					s += " sublw " + b.getText() + "\n";	
									 						break;
									 					}
									 					
									 								 					
									 					s += " btfsc STATUS, 2\n";
									 					s += " goto ELSE" + ifStack.peek() + "\n";
									 					break;
									 					
									 				case 3: //LT
									 					//s += "LESS THAN op1:" + op1type +" op2:" + op2type + "\n";
									 					switch(op1type)
									 					{
									 					case 1: //address
									 						switch(op2type)
										 					{
										 					case 1: //address
										 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
										 						break;
										 					case 2: //ident
										 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
										 						break;
										 					case 3: //number
										 						s += " movlw "  + e.getText() + "\n";
										 						break;
										 					}
										 					s += " subwf " + sTable.getSymbolValue(a.getText()) + ", 0\n";	
									 						break;
									 					case 2: //ident
									 						switch(op2type)
										 					{
										 					case 1: //address
										 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
										 						break;
										 					case 2: //ident
										 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
										 						break;
										 					case 3: //number
										 						s += " movlw "  + e.getText() + "\n";
										 						break;
										 					}
										 					s += " sublw " + sTable.getSymbolValue(c.getText()) + "\n";	
									 						break;
									 					case 3: //number
									 						switch(op2type)
										 					{
										 					case 1: //address
										 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
										 						break;
										 					case 2: //ident
										 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
										 						break;
										 					case 3: //number
										 						s += " movlw "  + e.getText() + "\n";
										 						break;
										 					}
										 					s += " sublw " + b.getText() + "\n";	
									 						break;
									 					}
									 					
									 								 					
									 					s += " btfsc STATUS, 0\n";
									 					s += " goto ELSE" + ifStack.peek() + "\n";
									 					break;
									 				case 4: //GT
									 					//s += "GREATER THAN op1:" + op1type +" op2:" + op2type + "\n";
									 					switch(op1type)
									 					{
									 					case 1: //address
									 						switch(op2type)
										 					{
										 					case 1: //address
										 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
										 						break;
										 					case 2: //ident
										 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
										 						break;
										 					case 3: //number
										 						s += " movlw "  + e.getText() + "\n";
										 						break;
										 					}
										 					s += " subwf " + sTable.getSymbolValue(a.getText()) + ", 0\n";	
									 						break;
									 					case 2: //ident
									 						switch(op2type)
										 					{
										 					case 1: //address
										 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
										 						break;
										 					case 2: //ident
										 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
										 						break;
										 					case 3: //number
										 						s += " movlw "  + e.getText() + "\n";
										 						break;
										 					}
										 					s += " sublw " + sTable.getSymbolValue(c.getText()) + "\n";	
									 						break;
									 					case 3: //number
									 						switch(op2type)
										 					{
										 					case 1: //address
										 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
										 						break;
										 					case 2: //ident
										 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
										 						break;
										 					case 3: //number
										 						s += " movlw "  + e.getText() + "\n";
										 						break;
										 					}
										 					s += " sublw " + b.getText() + "\n";	
									 						break;
									 					}
									 					
									 								 					
									 					s += " btfss STATUS, 0\n";
									 					s += " goto ELSE" + ifStack.peek() + "\n";
									 					break;
									 				case 5://LTE
									 					//s += "LESS THAN OR EQUAL op1:" + op1type +" op2:" + op2type + "\n";
									 					switch(op1type)
									 					{
									 					case 1: //address
									 						switch(op2type)
										 					{
										 					case 1: //address
										 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
										 						break;
										 					case 2: //ident
										 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
										 						break;
										 					case 3: //number
										 						s += " movlw "  + e.getText() + "\n";
										 						break;
										 					}
										 					s += " subwf " + sTable.getSymbolValue(a.getText()) + ", 0\n";	
									 						break;
									 					case 2: //ident
									 						switch(op2type)
										 					{
										 					case 1: //address
										 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
										 						break;
										 					case 2: //ident
										 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
										 						break;
										 					case 3: //number
										 						s += " movlw "  + e.getText() + "\n";
										 						break;
										 					}
										 					s += " sublw " + sTable.getSymbolValue(c.getText()) + "\n";	
									 						break;
									 					case 3: //number
									 						switch(op2type)
										 					{
										 					case 1: //address
										 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
										 						break;
										 					case 2: //ident
										 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
										 						break;
										 					case 3: //number
										 						s += " movlw "  + e.getText() + "\n";
										 						break;
										 					}
										 					s += " sublw " + b.getText() + "\n";	
									 						break;
									 					}
									 					
									 								 					
									 					s += " btfss STATUS, 2\n";
									 					s += " goto ELSE" + ifStack.peek() + "\n";
									 					s += " btfsc STATUS, 0\n";
									 					s += " goto ELSE" + ifStack.peek() + "\n";
									 					break;
									 				case 6: //GTE
														//s += "GREATER THAN OR EQUAL op1:" + op1type +" op2:" + op2type + "\n";
									 					switch(op1type)
									 					{
									 					case 1: //address
									 						switch(op2type)
										 					{
										 					case 1: //address
										 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
										 						break;
										 					case 2: //ident
										 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
										 						break;
										 					case 3: //number
										 						s += " movlw "  + e.getText() + "\n";
										 						break;
										 					}
										 					s += " subwf " + sTable.getSymbolValue(a.getText()) + ", 0\n";	
									 						break;
									 					case 2: //ident
									 						switch(op2type)
										 					{
										 					case 1: //address
										 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
										 						break;
										 					case 2: //ident
										 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
										 						break;
										 					case 3: //number
										 						s += " movlw "  + e.getText() + "\n";
										 						break;
										 					}
										 					s += " sublw " + sTable.getSymbolValue(c.getText()) + "\n";	
									 						break;
									 					case 3: //number
									 						switch(op2type)
										 					{
										 					case 1: //address
										 						s += " movf "  + sTable.getSymbolValue(d.getText()) + ", 0\n";
										 						break;
										 					case 2: //ident
										 						s += " movlw "  + sTable.getSymbolValue(f.getText()) + "\n";
										 						break;
										 					case 3: //number
										 						s += " movlw "  + e.getText() + "\n";
										 						break;
										 					}
										 					s += " sublw " + b.getText() + "\n";	
									 						break;
									 					}
									 					
									 								 					
									 					s += " btfss STATUS, 2\n";
									 					s += " goto ELSE" + ifStack.peek() + "\n";
									 					s += " btfss STATUS, 0\n";
									 					s += " goto ELSE" + ifStack.peek() + "\n";
									 				default:
									 					System.err.println("Error in sub-expression");
									 					break;				 				
									 				}
									 			
				}
			}
			else if (((_t.getType() >= IDENT && _t.getType() <= ADDRESS))) {
				{
				{
				if (_t==null) _t=ASTNULL;
				switch ( _t.getType()) {
				case ADDRESS:
				{
					k = (AST)_t;
					AST k_AST_in = null;
					k_AST = astFactory.create(k);
					astFactory.addASTChild(currentAST, k_AST);
					match(_t,ADDRESS);
					_t = _t.getNextSibling();
					if ( inputState.guessing==0 ) {
						op1type=1;
					}
					break;
				}
				case NUMBER:
				{
					l = (AST)_t;
					AST l_AST_in = null;
					l_AST = astFactory.create(l);
					astFactory.addASTChild(currentAST, l_AST);
					match(_t,NUMBER);
					_t = _t.getNextSibling();
					if ( inputState.guessing==0 ) {
						op1type=3;
					}
					break;
				}
				case IDENT:
				{
					m = (AST)_t;
					AST m_AST_in = null;
					m_AST = astFactory.create(m);
					astFactory.addASTChild(currentAST, m_AST);
					match(_t,IDENT);
					_t = _t.getNextSibling();
					if ( inputState.guessing==0 ) {
						op1type=2;
					}
					break;
				}
				default:
				{
					throw new NoViableAltException(_t);
				}
				}
				}
				if ( inputState.guessing==0 ) {
					
											//s += "SINGLE ELEMENT SUBEXPR op1:" + op1type + "\n";
											switch(op1type)
										 	{
										 		case 1: //address
										 			s += " movf "  + sTable.getSymbolValue(k.getText()) + ", 0\n";
										 			break;
										 		case 2: //ident
										 			s += " movlw "  + sTable.getSymbolValue(m.getText()) + "\n";
										 			break;
										 		case 3: //number
										 			s += " movlw "  + l.getText() + "\n";
										 			break;
										 	}
										 	s += " btfsc STATUS, 2\n";
									 		s += " goto ELSE" + ifStack.peek() + "\n";
										
				}
				}
			}
			else {
				throw new NoViableAltException(_t);
			}
			
			}
			currentAST = __currentAST2085;
			_t = __t2085;
			_t = _t.getNextSibling();
			subexpr_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = subexpr_AST;
		_retTree = _t;
		return s;
	}
	
	public final String  forSubStatement(AST _t) throws RecognitionException {
		String s="";
		
		AST forSubStatement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST forSubStatement_AST = null;
		
			String s1="";
		
		
		try {      // for error handling
			AST __t2111 = _t;
			AST tmp82_AST = null;
			AST tmp82_AST_in = null;
			tmp82_AST = astFactory.create((AST)_t);
			tmp82_AST_in = (AST)_t;
			astFactory.addASTChild(currentAST, tmp82_AST);
			ASTPair __currentAST2111 = currentAST.copy();
			currentAST.root = currentAST.child;
			currentAST.child = null;
			match(_t,FORSUBSTATEMENT);
			_t = _t.getFirstChild();
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case INSTRUCTIONSTATEMENT:
			{
				s1=instructionStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case FUNCTIONCALLSTATEMENT:
			{
				s1=functioncallStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			case ASSIGNMENTSTATEMENT:
			{
				s1=assignmentStatement(_t);
				_t = _retTree;
				astFactory.addASTChild(currentAST, returnAST);
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			currentAST = __currentAST2111;
			_t = __t2111;
			_t = _t.getNextSibling();
			if ( inputState.guessing==0 ) {
				s += s1;
			}
			forSubStatement_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				if (_t!=null) {_t = _t.getNextSibling();}
			} else {
			  throw ex;
			}
		}
		returnAST = forSubStatement_AST;
		_retTree = _t;
		return s;
	}
	
	
	public static final String[] _tokenNames = {
		"<0>",
		"EOF",
		"<2>",
		"NULL_TREE_LOOKAHEAD",
		"\"begin\"",
		"\"end\"",
		"\"equ\"",
		"\"org\"",
		"\"addff\"",
		"\"subff\"",
		"\"subwff\"",
		"\"addwff\"",
		"\"swapff\"",
		"\"iorwff\"",
		"\"andwff\"",
		"\"xorwff\"",
		"\"movff\"",
		"\"if\"",
		"\"endif\"",
		"\"else\"",
		"\"for\"",
		"\"while\"",
		"\"do\"",
		"\"continue\"",
		"\"break\"",
		"\"function\"",
		"\"var\"",
		"\"array\"",
		"\"addwf\"",
		"\"andwf\"",
		"\"clrf\"",
		"\"clrw\"",
		"\"comf\"",
		"\"decf\"",
		"\"decfsz\"",
		"\"incf\"",
		"\"incfsz\"",
		"\"iorwf\"",
		"\"movf\"",
		"\"movwf\"",
		"\"nop\"",
		"\"rlf\"",
		"\"rrf\"",
		"\"subwf\"",
		"\"swapf\"",
		"\"xorwf\"",
		"\"bcf\"",
		"\"bsf\"",
		"\"btfsc\"",
		"\"btfss\"",
		"\"addlw\"",
		"\"andlw\"",
		"\"call\"",
		"\"clrwdt\"",
		"\"goto\"",
		"\"iorlw\"",
		"\"movlw\"",
		"\"retfie\"",
		"\"retlw\"",
		"\"return\"",
		"\"sleep\"",
		"\"sublw\"",
		"\"xorlw\"",
		"NEWLINE",
		"WS",
		"COMMENT",
		"LCURLY",
		"RCURLY",
		"LPAR",
		"RPAR",
		"COMMA",
		"LBRC",
		"RBRC",
		"SEMI",
		"COLON",
		"ASSIGN",
		"PLUS",
		"MINUS",
		"AND",
		"OR",
		"EQ",
		"NEQ",
		"LT",
		"GT",
		"LTE",
		"GTE",
		"DIGIT",
		"DECIMAL",
		"BINARY",
		"HEX",
		"IDENT",
		"NUMBER",
		"ADDRESS",
		"LITERAL",
		"LETTER",
		"PROGRAM",
		"SUBPROGRAMBODY",
		"BASICDECL",
		"VARDECL",
		"CONSTDECL",
		"ARRAYDECL",
		"STATEMENT",
		"ORGSTATEMENT",
		"INSTRUCTIONSTATEMENT",
		"ASMPPSTATEMENT",
		"ADDFFSTATEMENT",
		"SUBFFSTATEMENT",
		"ADDWFFSTATEMENT",
		"SUBWFFSTATEMENT",
		"SWAPFFSTATEMENT",
		"IORWFFSTATEMENT",
		"ANDWFFSTATEMENT",
		"XORWFFSTATEMENT",
		"MOVFFSTATEMENT",
		"ASMSTATEMENT",
		"ADDWFSTATEMENT",
		"ANDWFSTATEMENT",
		"CLRFSTATEMENT",
		"CLRWSTATEMENT",
		"COMFSTATEMENT",
		"DECFSTATEMENT",
		"DECFSZSTATEMENT",
		"INCFSTATEMENT",
		"INCFSZSTATEMENT",
		"IORWFSTATEMENT",
		"MOVFSTATEMENT",
		"MOVWFSTATEMENT",
		"NOPSTATEMENT",
		"RLFSTATEMENT",
		"RRFSTATEMENT",
		"SUBWFSTATEMENT",
		"SWAPFSTATEMENT",
		"XORWFSTATEMENT",
		"BSFSTATEMENT",
		"BCFSTATEMENT",
		"BTFSCSTATEMENT",
		"BTFSSSTATEMENT",
		"ADDLWSTATEMENT",
		"ANDLWSTATEMENT",
		"CALLSTATEMENT",
		"CLRWDTSTATEMENT",
		"GOTOSTATEMENT",
		"IORLWSTATEMENT",
		"MOVLWSTATEMENT",
		"RETFIESTATEMENT",
		"RETLWSTATEMENT",
		"SLEEPSTATEMENT",
		"SUBLWSTATEMENT",
		"XORLWSTATEMENT",
		"EXPR",
		"SUBEXPR",
		"IFSTATEMENT",
		"WHILESTATEMENT",
		"FORSUBSTATEMENT",
		"FORSTATEMENT",
		"DOWHILESTATEMENT",
		"ASSIGNMENTSTATEMENT",
		"FUNCTIONCALLSTATEMENT",
		"RETURNSTATEMENT",
		"LABELSTATEMENT",
		"FUNCTIONDECLARATION",
		"\",\""
	};
	
	}
	
